@@ -16,8 +16,8 @@
 ###
 ### You should have received a copy of the GNU General Public
 ### License along with this program; if not, write to the Free
-### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-### MA 02111-1307, USA
+### Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+### MA 02110-1301, USA
 
 
 
@@ -93,25 +93,31 @@ as.factorOrShingle <- function(x, subset = TRUE, drop = FALSE)
 ## lattice settings using trellis.par.set and lattice.options
 ## respectively
 
-updateList <-
-    function(x, val)
+updateList <- function(x, val)
 {
     if (is.null(x)) x <- list()
-    if (!is.list(x)) stop("x must be NULL or a list")
-    if (!is.list(val)) stop("val must be a list")
-    xnames <- names(x)
-    for (v in names(val))
-    {
-        existing <- v %in% xnames
-        if (existing && is.list(x[[v]]) && is.list(val[[v]]))
-            x[[v]] <- updateList(x[[v]], val[[v]])
-        else 
-            x[[v]] <- val[[v]]
-    }
-    x
+    modifyList(x, val)
 }
 
 
+##     function(x, val)
+## {
+##     if (is.null(x)) x <- list()
+##     if (!is.list(x)) stop("x must be NULL or a list")
+##     if (!is.list(val)) stop("val must be a list")
+##     xnames <- names(x)
+##     for (v in names(val))
+##     {
+##         existing <- v %in% xnames
+##         if (existing && is.list(x[[v]]) && is.list(val[[v]]))
+##             x[[v]] <- updateList(x[[v]], val[[v]])
+##         else 
+##             x[[v]] <- val[[v]]
+##     }
+##     x
+## }
+
+
 
 
 
@@ -172,7 +178,7 @@ lpolygon <-
         border <-
             if (border) "black"
             else "transparent"
-    xy <- xy.coords(x, y)
+    xy <- xy.coords(x, y, recycle = TRUE)
     if (with(xy, sum(!is.na(x) & !is.na(y))) > 0)
         grid.polygon(x = xy$x,
                      y = xy$y,
@@ -278,9 +284,9 @@ larrows <-
 
 
 
+ltext <- function(x, ...) UseMethod("ltext")
 
-
-ltext <-
+ltext.default <-
     function(x, y = NULL, labels = seq(along = x),
              col = add.text$col,
              alpha = add.text$alpha,
@@ -293,11 +299,10 @@ ltext <-
              adj = c(.5, .5),
              pos = NULL,
              offset = 0.5,
-             ## FIXME: need an offset argument
              ...)
 {
     add.text <- trellis.par.get("add.text")
-    xy <- xy.coords(x, y)
+    xy <- xy.coords(x, y, recycle = TRUE)
     if (length(xy$x) == 0) return()
     ux <- unit(xy$x, "native")
     uy <- unit(xy$y, "native")
@@ -342,9 +347,10 @@ ltext <-
 
 
 
+llines <- function (x, ...) UseMethod("llines")
 
 
-llines <-
+llines.default <-
     function(x, y = NULL, type = "l",
              col = plot.line$col,
              alpha = plot.line$alpha,
@@ -352,14 +358,14 @@ llines <-
              lwd = plot.line$lwd, ...)
 {
     plot.line <- trellis.par.get("plot.line")
-    lplot.xy(xy.coords(x, y), type = type,
+    lplot.xy(xy.coords(x, y, recycle = TRUE), type = type,
              col = col, lty = lty, lwd = lwd, alpha = alpha, ...)
 }
 
 
+lpoints <- function (x, ...) UseMethod("lpoints")
 
-
-lpoints <-
+lpoints.default <-
     function(x, y = NULL, type = "p",
              col = plot.symbol$col,
              pch = plot.symbol$pch,
@@ -371,7 +377,7 @@ lpoints <-
              cex = plot.symbol$cex, ...)
 {
     plot.symbol <- trellis.par.get("plot.symbol")
-    lplot.xy(xy.coords(x, y),
+    lplot.xy(xy.coords(x, y, recycle = TRUE),
              type = type,
              col = col,
              pch = pch,
