@@ -1,7 +1,7 @@
 #-*- perl -*-
 
 ## Copyright (C) 2001--2002 R Development Core Team
-## Copyright (C) 2003-4       The R Foundation
+## Copyright (C) 2003-4, 2006 The R Foundation
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -49,18 +49,46 @@ print <<_EOF_;
 ### * <HEADER>
 ###
 attach(NULL, name = "CheckExEnv")
-assign(".CheckExEnv", as.environment(2), pos = length(search())) # base
-## add some hooks to label plot pages for base and grid graphics
-setHook("plot.new", ".newplot.hook")
-setHook("persp", ".newplot.hook")
-setHook("grid.newpage", ".gridplot.hook")
-
+assign("nameEx", 
+       local({
+	   s <- "__{must remake R-ex/*.R}__"
+           function(new) {
+               if(!missing(new)) s <<- new else s
+           }
+       }),
+       pos = "CheckExEnv")
+## Add some hooks to label plot pages for base and grid graphics
+assign("base_plot_hook",
+       function() {
+           pp <- par(c("mfg","mfcol","oma","mar"))
+           if(all(pp\$mfg[1:2] == c(1, pp\$mfcol[2]))) {
+               outer <- (oma4 <- pp\$oma[4]) > 0; mar4 <- pp\$mar[4]
+               mtext(sprintf("help(\\"%s\\")", nameEx()), side = 4,
+                     line = if(outer)max(1, oma4 - 1) else min(1, mar4 - 1),
+              outer = outer, adj = 1, cex = .8, col = "orchid", las=3)
+           }
+       },
+       pos = "CheckExEnv")
+assign("grid_plot_hook",
+       function() {
+           pushViewport(viewport(width=unit(1, "npc") - unit(1, "lines"),
+                                 x=0, just="left"))
+           grid.text(sprintf("help(\\"%s\\")", nameEx()),
+                     x=unit(1, "npc") + unit(0.5, "lines"),
+                     y=unit(0.8, "npc"), rot=90,
+                     gp=gpar(col="orchid"))
+       },
+       pos = "CheckExEnv")
+setHook("plot.new",     get("base_plot_hook", pos = "CheckExEnv"))
+setHook("persp",        get("base_plot_hook", pos = "CheckExEnv"))
+setHook("grid.newpage", get("grid_plot_hook", pos = "CheckExEnv"))
 assign("cleanEx",
        function(env = .GlobalEnv) {
 	   rm(list = ls(envir = env, all.names = TRUE), envir = env)
            RNGkind("default", "default")
 	   set.seed(1)
    	   options(warn = 1)
+	   .CheckExEnv <- as.environment("CheckExEnv")
 	   delayedAssign("T", stop("T used instead of TRUE"),
 		  assign.env = .CheckExEnv)
 	   delayedAssign("F", stop("F used instead of FALSE"),
@@ -74,11 +102,10 @@ assign("cleanEx",
 	       warning("items ", paste(missitems, collapse=", "),
 		       " have been removed from the search path")
        },
-       env = .CheckExEnv)
-assign("..nameEx", "__{must remake R-ex/*.R}__", env = .CheckExEnv) # for now
-assign("ptime", proc.time(), env = .CheckExEnv)
+       pos = "CheckExEnv")
+assign("ptime", proc.time(), pos = "CheckExEnv")
 grDevices::postscript("$PKG-Ex.ps")
-assign("par.postscript", graphics::par(no.readonly = TRUE), env = .CheckExEnv)
+assign("par.postscript", graphics::par(no.readonly = TRUE), pos = "CheckExEnv")
 options(contrasts = c(unordered = "contr.treatment", ordered = "contr.poly"))
 options(warn = 1)    
 _EOF_
@@ -88,8 +115,8 @@ if($PKG eq "tcltk") {
 } elsif($PKG ne "base") {
     print "library('$PKG')\n\n";
 }
-print "assign(\".oldSearch\", search(), env = .CheckExEnv)\n";
-print "assign(\".oldNS\", loadedNamespaces(), env = .CheckExEnv)\n";
+print "assign(\".oldSearch\", search(), pos = 'CheckExEnv')\n";
+print "assign(\".oldNS\", loadedNamespaces(), pos = 'CheckExEnv')\n";
 
 ### * Loop over all R files, and edit a few of them ...
 foreach my $file (@Rfiles) {
@@ -111,7 +138,7 @@ foreach my $file (@Rfiles) {
     }
     close(FILE);
     if ($have_examples) {
-	print "cleanEx(); ..nameEx <- \"$nm\"\n\n";
+	print "cleanEx(); nameEx(\"$nm\");\n";
     }
 
     print "### * $nm\n\n";
@@ -122,7 +149,7 @@ foreach my $file (@Rfiles) {
 
     if($have_par) {
 	## if there were 'par()' calls, now reset them:
-	print "graphics::par(get(\"par.postscript\", env = .CheckExEnv))\n";
+	print "graphics::par(get(\"par.postscript\", pos = 'CheckExEnv'))\n";
     }
     if($have_contrasts) {
 	## if contrasts were set, now reset them:
@@ -136,7 +163,7 @@ foreach my $file (@Rfiles) {
 print <<_EOF_;
 ### * <FOOTER>
 ###
-cat("Time elapsed: ", proc.time() - get("ptime", env = .CheckExEnv),"\\n")
+cat("Time elapsed: ", proc.time() - get("ptime", pos = 'CheckExEnv'),"\\n")
 grDevices::dev.off()
 ###
 ### Local variables: ***
