@@ -344,7 +344,7 @@ static pager pagercreate()
     p = newconsoledata((consolefn) ? consolefn : FixedFont,
 		       pagerrow, pagercol, 0, 0,
 		       consolefg, consoleuser, consolebg,
-		       PAGER);
+		       PAGER, 0);
     if (!p) return NULL;
 
 /*    if (ismdi()) {
