@@ -39,6 +39,8 @@
 #include "rui.h"
 #include "preferences.h"
 
+#define gettext GA_gettext
+
 extern char fontname[LF_FACESIZE+1]; /* from console.c */
 extern int consolex, consoley; /* from console.c */
 extern int pagerMultiple, haveusedapager; /* from pager.c */
@@ -67,15 +69,15 @@ static char *FontsList[] = {"Courier", "Courier New", "FixedSys", "FixedFont", "
 
 static window wconfig;
 static button bApply, bSave, bLoad, bOK, bCancel;
-static label l_mdi, l_mwin, l_font, l_point, l_style, l_crows, l_ccols,
+static label l_mdi, l_mwin, l_font, l_point, l_style, l_lang, l_crows, l_ccols,
     l_cx, l_cy, l_prows, l_pcols, l_grx, l_gry,
     l_cols, l_bgcol, l_fgcol, l_usercol, l_highlightcol, l_cbb, l_cbl;
 static radiogroup g_mwin;
 static radiobutton rb_mdi, rb_sdi, rb_mwin, rb_swin;
 static listbox f_font, f_style, d_point, bgcol, fgcol, usercol, highlightcol;
-static checkbox toolbar, statusbar, tt_font, c_resize;
+static checkbox toolbar, statusbar, tt_font, c_resize, c_buff;
 static field f_crows, f_ccols, f_prows, f_pcols, f_cx, f_cy, f_cbb,f_cbl,
-    f_grx, f_gry;
+    f_grx, f_gry, f_lang;
 
 
 static void getChoices(Gui p)
@@ -84,6 +86,7 @@ static void getChoices(Gui p)
     p->toolbar = ischecked(toolbar);
     p->statusbar = ischecked(statusbar);
     p->pagerMultiple = ischecked(rb_mwin);
+    strcpy(p->language, gettext(f_lang));
     strcpy(p->font, gettext(f_font));
     p->tt_font = ischecked(tt_font);
     p->pointsize = atoi(gettext(d_point));
@@ -93,6 +96,7 @@ static void getChoices(Gui p)
     p->cx = atoi(gettext(f_cx));
     p->cy = atoi(gettext(f_cy));
     p->setWidthOnResize = ischecked(c_resize);
+    p->buffered = ischecked(c_buff);
     p->cbb = atoi(gettext(f_cbb));
     p->cbl = atoi(gettext(f_cbl));
     p->prows = atoi(gettext(f_prows));
@@ -115,6 +119,10 @@ void getActive(Gui gui)
     gui->statusbar = ((RguiMDI & RW_STATUSBAR) != 0);
     gui->MDI = ((RguiMDI & RW_MDI) != 0);
     gui->pagerMultiple = pagerMultiple;
+    {
+	char *p = getenv("LANGUAGE");
+	strcpy(gui->language, p ? p : "");
+    }
 
 /* Font, pointsize, style */
 
@@ -144,6 +152,7 @@ void getActive(Gui gui)
     gui->setWidthOnResize = setWidthOnResize;
     gui->cbb = p->lbuf->dim;
     gui->cbl = p->lbuf->ms;
+    gui->buffered = consolebuffered;
 
 /* Pager size */
     gui->prows = pagerrow;
@@ -170,6 +179,7 @@ static int has_changed(Gui a, Gui b)
 	a->toolbar != b->toolbar ||
 	a->statusbar != b->statusbar ||
 	a->pagerMultiple != b->pagerMultiple ||
+	strcmp(a->language, b->language) ||
 	strcmp(a->font, b->font) ||
 	a->tt_font != b->tt_font ||
 	a->pointsize != b->pointsize ||
@@ -198,11 +208,12 @@ static void cleanup()
     delobj(l_mdi); delobj(rb_mdi); delobj(rb_sdi);
     delobj(toolbar); delobj(statusbar);
     delobj(l_mwin); delobj(g_mwin); delobj(rb_mwin); delobj(rb_swin);
+    delobj(l_lang); delobj(f_lang);
     delobj(l_font); delobj(f_font); delobj(tt_font);
     delobj(l_point); delobj(d_point);
     delobj(l_style); delobj(f_style);
     delobj(l_crows); delobj(f_crows); delobj(l_ccols); delobj(f_ccols);
-    delobj(c_resize);
+    delobj(c_resize); delobj(c_buff);
     delobj(l_cx); delobj(f_cx); delobj(l_cy); delobj(f_cy);
     delobj(l_cbb); delobj(f_cbb); delobj(l_cbl); delobj(f_cbl);
     delobj(l_prows); delobj(f_prows); delobj(l_pcols); delobj(f_pcols);
@@ -231,6 +242,13 @@ void applyGUI(Gui newGUI)
        newGUI->statusbar != curGUI.statusbar)
 	askok(G_("The overall console properties cannot be changed\non a running console.\n\nSave the preferences and restart Rgui to apply them.\n"));
 
+    if(strcmp(newGUI->language, curGUI.language)) {
+	char *buf = malloc(50);
+	askok(G_("The language for menus cannot be changed on a\n running console.\n\nSave the preferences and restart Rgui to apply to menus.\n"));
+	sprintf(buf, "LANGUAGE=%s", newGUI->language);
+	putenv(buf);	
+    }
+    
 
 /*  Set a new font? */
     if(strcmp(newGUI->font, curGUI.font) ||
@@ -310,6 +328,7 @@ void applyGUI(Gui newGUI)
     }
 
     setWidthOnResize = newGUI->setWidthOnResize;
+    consolebuffered = newGUI->buffered;
 }
 
 static void do_apply()
@@ -402,6 +421,12 @@ static void save(button b)
 	    "## Initial position of the graphics window",
 	    "## (pixels, <0 values from opposite edge)",
 	    gettext(f_grx), gettext(f_gry));
+    fprintf(fp, "\n\n%s\nlanguage = %s\n",
+	    "## Language for messages",
+	    gettext(f_lang));
+    fprintf(fp, "\n\n## Default setting for console buffering: 'yes' or 'no'\n");
+    fprintf(fp, "buffered = %s\n",
+	    ischecked(c_buff) ? "yes" : "no");
     fclose(fp);
 }
 
@@ -587,6 +612,22 @@ int loadRconsole(Gui gui, char *optf)
 		    gui->setWidthOnResize = 0;
 		done = 1;
 	    }
+	    if (!strcmp(opt[0], "language")) {
+		strcpy(gui->language, opt[1]); 
+ 		done = 1;
+	    }
+	    if (!strcmp(opt[0], "buffered")) {
+		if (!strcmp(opt[1], "yes"))
+		    gui->buffered = 1;
+		else if (!strcmp(opt[1], "no"))
+		    gui->buffered = 0;		
+ 		done = 1;
+	    }
+	} else if (ok == 3) { /* opt[1] == "" */
+	    if (!strcmp(opt[0], "language")) {
+		strcpy(gui->language, opt[1]); 
+ 		done = 1;
+	    }
 	}
 	if (!done) {
 	    char  buf[128];
@@ -653,12 +694,16 @@ static void showDialog(Gui gui)
 	check(rb_sdi); cSDI(rb_sdi);
     }
 
-    l_mwin = newlabel("Pager style", rect(10, 50, 90, 20), AlignLeft);
+    l_mwin = newlabel("Pager style", rect(10, 40, 90, 20), AlignLeft);
     g_mwin = newradiogroup();
-    rb_mwin = newradiobutton("multiple windows", rect(150, 50, 150, 20), NULL);
-    rb_swin = newradiobutton("single window", rect(320, 50 , 150, 20), NULL);
+    rb_mwin = newradiobutton("multiple windows", rect(150, 40, 150, 20), NULL);
+    rb_swin = newradiobutton("single window", rect(150, 60 , 150, 20), NULL);
     if(gui->pagerMultiple) check(rb_mwin); else check(rb_swin);
 
+    l_lang = newlabel("Language for menus\nand messages", 
+		      rect(320, 40, 130, 40), AlignLeft);
+    f_lang = newfield(gui->language, rect(450, 45, 60, 20));
+
 /* Font, pointsize, style */
 
     l_font = newlabel("Font", rect(10, 100, 40, 20), AlignLeft);
@@ -691,7 +736,7 @@ static void showDialog(Gui gui)
     f_cy = newfield(buf, rect(480, 150, 40, 20));
 
     c_resize = newcheckbox("set options(width) on resize?",
-			   rect(50, 175, 200, 20), NULL);
+			   rect(20, 175, 200, 20), NULL);
     if(gui->setWidthOnResize) check(c_resize);
 
     l_cbb = newlabel("buffer bytes", rect(270, 175, 70, 20), AlignLeft);
@@ -701,22 +746,26 @@ static void showDialog(Gui gui)
     sprintf(buf, "%d", gui->cbl);
     f_cbl = newfield(buf, rect(480, 175, 40, 20));
 
+    c_buff = newcheckbox("buffer console by default?",
+			 rect(20, 190, 200, 20), NULL);
+    if(gui->buffered) check(c_buff);
+
 /* Pager size */
-    l_prows = newlabel("Pager   rows", rect(10, 210, 100, 20), AlignLeft);
+    l_prows = newlabel("Pager   rows", rect(10, 220, 100, 20), AlignLeft);
     sprintf(buf, "%d", gui->prows);
-    f_prows = newfield(buf, rect(110, 210, 30, 20));
-    l_pcols = newlabel("columns", rect(150, 210, 60, 20), AlignLeft);
+    f_prows = newfield(buf, rect(110, 220, 30, 20));
+    l_pcols = newlabel("columns", rect(150, 220, 60, 20), AlignLeft);
     sprintf(buf, "%d", gui->pcols);
-    f_pcols = newfield(buf, rect(220, 210, 30, 20));
+    f_pcols = newfield(buf, rect(220, 220, 30, 20));
 
 /* Graphics window */
     l_grx = newlabel("Graphics windows: initial left",
-		    rect(10, 250, 190, 20), AlignLeft);
+		    rect(10, 260, 190, 20), AlignLeft);
     sprintf(buf, "%d", gui->grx);
-    f_grx = newfield(buf, rect(200, 250, 40, 20));
-    l_gry = newlabel("top", rect(270, 250, 30, 20), AlignLeft);
+    f_grx = newfield(buf, rect(200, 260, 40, 20));
+    l_gry = newlabel("top", rect(270, 260, 30, 20), AlignLeft);
     sprintf(buf, "%d", gui->gry);
-    f_gry = newfield(buf, rect(300, 250, 40, 20));
+    f_gry = newfield(buf, rect(300, 260, 40, 20));
 
 /* Font colours */
     l_cols = newlabel("Console and Pager Colours",
