@@ -3,7 +3,7 @@ data.matrix <- function(frame)
     if(!is.data.frame(frame))
 	return(as.matrix(frame))
     d <- dim(frame)
-    if(all(d > 0)) {
+    if(d[2] > 0) {
 	log <- unlist(lapply(frame, is.logical))
 	num <- unlist(lapply(frame, is.numeric))
 	fac <- unlist(lapply(frame, is.factor))
@@ -17,7 +17,7 @@ data.matrix <- function(frame)
         if(length(cl) && any(cl))
             warning("class information lost from one or more columns")
     }
-    x <- matrix(nr=d[1], nc=d[2], dimnames=dimnames(frame))
+    x <- matrix(as.integer(NA), nr=d[1], nc=d[2], dimnames=dimnames(frame))
     for(i in seq(len=d[2])) {
 	xi <- frame[[i]]
 	x[,i] <-
