@@ -1,6 +1,13 @@
+## As from R 2.4.0, row.names can be either character or integer.
+## row.names() will always return character.
+## attr(, "row.names") will return either character or integer.
+##
+## Do not assume that the internal representation is either, since
+## 1:n is stored as the integer vector c(NA, n) to save space (and
+## the C-level code to get/set the attribute makes the appropriate
+## translations (whereas attributes does not).
 
 row.names <- function(x) UseMethod("row.names")
-## expand.grid was setting integer row.names
 row.names.data.frame <- function(x) as.character(attr(x, "row.names"))
 row.names.default <- function(x) if(!is.null(dim(x))) rownames(x)# else NULL
 
@@ -9,14 +16,21 @@ row.names.default <- function(x) if(!is.null(dim(x))) rownames(x)# else NULL
     if (!is.data.frame(x))
 	x <- as.data.frame(x)
     old <- attr(x, "row.names")
+    if(is.null(value)) {
+        n <- length(old)
+        attr(x, "row.names") <-
+            if(n > 2) c(as.integer(NA), n) else seq_len(n)
+        return(x)
+    }
     ## do this here, as e.g. POSIXlt changes length when coerced.
-    value <- as.character(value)
+    if(is.object(value) || !(is.integer(value)) )
+        value <- as.character(value)
     if (!is.null(old) && length(value) != length(old))
 	stop("invalid 'row.names' length")
     if (any(duplicated(value)))
 	stop("duplicate 'row.names' are not allowed")
     if (any(is.na(value)))
-	stop("missing 'row.names' are not allowed")
+	stop("missing values in 'row.names' are not allowed")
     attr(x, "row.names") <- value
     x
 }
@@ -50,7 +64,7 @@ t.data.frame <- function(x) {
 
 dim.data.frame <- function(x) c(length(attr(x,"row.names")), length(x))
 
-dimnames.data.frame <- function(x) list(attr(x,"row.names"), names(x))
+dimnames.data.frame <- function(x) list(row.names(x), names(x))
 
 "dimnames<-.data.frame" <- function(x, value)
 {
@@ -67,12 +81,13 @@ dimnames.data.frame <- function(x) list(attr(x,"row.names"), names(x))
     x
 }
 
-as.data.frame <- function(x, row.names = NULL, optional = FALSE) {
+as.data.frame <- function(x, row.names = NULL, optional = FALSE, ...) {
     if(is.null(x))			# can't assign class to NULL
 	return(as.data.frame(list()))
     UseMethod("as.data.frame")
 }
-as.data.frame.default <- function(x, row.names = NULL, optional = FALSE)
+
+as.data.frame.default <- function(x, ...)
     stop(gettextf("cannot coerce class \"%s\" into a data.frame", class(x)),
          domain = NA)
 
@@ -80,13 +95,13 @@ as.data.frame.default <- function(x, row.names = NULL, optional = FALSE)
 ###  Here are methods ensuring that the arguments to "data.frame"
 ###  are in a form suitable for combining into a data frame.
 
-as.data.frame.data.frame <- function(x, row.names = NULL, optional = FALSE)
+as.data.frame.data.frame <- function(x, row.names = NULL, ...)
 {
     cl <- oldClass(x)
     i <- match("data.frame", cl)
     if(i > 1)
 	class(x) <- cl[ - (1:(i-1))]
-    if(is.character(row.names)){
+    if(!is.null(row.names)){
 	if(length(row.names) == length(attr(x, "row.names")))
 	    attr(x, "row.names") <- row.names
 	else stop(gettextf("invalid 'row.names', length %d for a data frame with %d rows",
@@ -97,19 +112,23 @@ as.data.frame.data.frame <- function(x, row.names = NULL, optional = FALSE)
 }
 
 ## prior to 1.8.0 this coerced names - PR#3280
-as.data.frame.list <- function(x, row.names = NULL, optional = FALSE)
+as.data.frame.list <-
+    function(x, row.names = NULL, optional = FALSE, ...,
+             stringsAsFactors = default.stringsAsFactors())
 {
     ## need to protect names in x.
     cn <- names(x)
-    m <- match(c("row.names", "check.rows", "check.names"), cn, 0)
+    m <- match(c("row.names", "check.rows", "check.names", "stringsAsFactors"),
+               cn, 0)
     if(any(m > 0)) {
         cn[m] <- paste("..adfl.", cn[m], sep="")
         names(x) <- cn
     }
-    x <- eval(as.call(c(expression(data.frame), x, check.names = !optional)))
+    x <- eval(as.call(c(expression(data.frame), x, check.names = !optional,
+                        stringsAsFactors = stringsAsFactors)))
     if(any(m > 0)) names(x) <- sub("^\\.\\.adfl\\.", "", names(x))
     if(!is.null(row.names)) {
-	row.names <- as.character(row.names)
+	# row.names <- as.character(row.names)
 	if(length(row.names) != dim(x)[[1]])
             stop(gettextf("supplied %d row names for %d rows",
                           length(row.names), dim(x)[[1]]), domain = NA)
@@ -118,17 +137,17 @@ as.data.frame.list <- function(x, row.names = NULL, optional = FALSE)
     x
 }
 
-as.data.frame.vector <- function(x, row.names = NULL, optional = FALSE)
+as.data.frame.vector <- function(x, row.names = NULL, optional = FALSE, ...)
 {
     nrows <- length(x)
-    nm <- paste(deparse(substitute(x), width.cutoff=500), collapse=" ")
+    nm <- paste(deparse(substitute(x), width.cutoff = 500), collapse=" ")
     if(is.null(row.names)) {
 	if (nrows == 0)
 	    row.names <- character(0)
 	else if(length(row.names <- names(x)) == nrows &&
 		!any(duplicated(row.names))) {}
 	else if(optional) row.names <- character(nrows)
-	else row.names <- as.character(1:nrows)
+	else row.names <- seq_len(nrows)
     }
     names(x) <- NULL # remove names as from 2.0.0
     value <- list(x)
@@ -138,12 +157,12 @@ as.data.frame.vector <- function(x, row.names = NULL, optional = FALSE)
     value
 }
 
-as.data.frame.ts <- function(x, row.names=NULL, optional=FALSE)
+as.data.frame.ts <- function(x, ...)
 {
     if(is.matrix(x))
-	as.data.frame.matrix(x, row.names, optional)
+	as.data.frame.matrix(x, ...)
     else
-	as.data.frame.vector(x, row.names, optional)
+	as.data.frame.vector(x, ...)
 }
 
 as.data.frame.raw  <- as.data.frame.vector
@@ -153,25 +172,37 @@ as.data.frame.integer <- as.data.frame.vector
 as.data.frame.numeric <- as.data.frame.vector
 as.data.frame.complex <- as.data.frame.vector
 
-as.data.frame.character <- function(x, row.names = NULL, optional = FALSE)
-    as.data.frame.vector(factor(x), row.names, optional)
+default.stringsAsFactors <- function()
+{
+    val <- getOption("stringsAsFactors")
+    if(is.null(val)) val <- TRUE
+    if(!is.logical(val) || is.na(val) || length(val) != 1)
+        stop("options('stringsAsFactors') not set to TRUE or FALSE")
+    val
+}
+
+
+as.data.frame.character <-
+    function(x, ..., stringsAsFactors = default.stringsAsFactors())
+    as.data.frame.vector(if(stringsAsFactors) factor(x) else x, ...)
 
 as.data.frame.logical <- as.data.frame.vector
 
-as.data.frame.matrix <- function(x, row.names = NULL, optional = FALSE)
+as.data.frame.matrix <- function(x, row.names = NULL, optional = FALSE, ...,
+                                 stringsAsFactors = default.stringsAsFactors())
 {
     d <- dim(x)
-    nrows <- d[1]; ir <- seq(length = nrows)
-    ncols <- d[2]; ic <- seq(length = ncols)
+    nrows <- d[1]; ir <- seq_len(nrows)
+    ncols <- d[2]; ic <- seq_len(ncols)
     dn <- dimnames(x)
     ## surely it cannot be right to override the supplied row.names?
     ## changed in 1.8.0
-    if(missing(row.names)) row.names <- dn[[1]]
+    if(is.null(row.names)) row.names <- dn[[1]]
     collabs <- dn[[2]]
     if(any(empty <- nchar(collabs)==0))
 	collabs[empty] <- paste("V", ic, sep = "")[empty]
     value <- vector("list", ncols)
-    if(mode(x) == "character") {
+    if(mode(x) == "character" && stringsAsFactors) {
 	for(i in ic)
 	    value[[i]] <- as.factor(x[,i])
     } else {
@@ -179,7 +210,7 @@ as.data.frame.matrix <- function(x, row.names = NULL, optional = FALSE)
 	    value[[i]] <- as.vector(x[,i])
     }
     if(length(row.names) != nrows)
-	row.names <- if(optional) character(nrows) else as.character(ir)
+	row.names <- if(optional) character(nrows) else ir
     if(length(collabs) == ncols)
 	names(value) <- collabs
     else if(!optional)
@@ -189,7 +220,8 @@ as.data.frame.matrix <- function(x, row.names = NULL, optional = FALSE)
     value
 }
 
-as.data.frame.model.matrix <- function(x, row.names = NULL, optional = FALSE)
+as.data.frame.model.matrix <-
+    function(x, row.names = NULL, optional = FALSE, ...)
 {
     d <- dim(x)
     nrows <- d[1]
@@ -203,39 +235,39 @@ as.data.frame.model.matrix <- function(x, row.names = NULL, optional = FALSE)
                           length(row.names), nrows), domain = NA)
     }
     else if(optional) row.names <- character(nrows)
-    else row.names <- as.character(1:nrows)
+    else row.names <- seq_len(nrows)
     if(!optional) names(value) <- deparse(substitute(x))[[1]]
     attr(value, "row.names") <- row.names
     class(value) <- "data.frame"
     value
 }
 
-as.data.frame.array <- function(x, row.names = NULL, optional = FALSE)
+as.data.frame.array <- function(x, row.names = NULL, optional = FALSE, ...)
 {
     d <- dim(x)
     if(length(d) == 1) { ## same as as.data.frame.vector, but deparsed here
-        value <- as.data.frame.vector(drop(x), row.names, optional)
+        value <- as.data.frame.vector(drop(x), row.names, optional, ...)
         if(!optional) names(value) <- deparse(substitute(x))[[1]]
         value
     } else if (length(d) == 2) {
-        as.data.frame.matrix(x, row.names, optional)
+        as.data.frame.matrix(x, row.names, optional, ...)
     } else {
         dn <- dimnames(x)
         dim(x) <- c(d[1], prod(d[-1]))
         if(!is.null(dn)) {
             if(length(dn[[1]])) rownames(x) <- dn[[1]]
             for(i in 2:length(d))
-                if(is.null(dn[[i]])) dn[[i]] <- seq(len=d[i])
+                if(is.null(dn[[i]])) dn[[i]] <- seq_len(d[i])
             colnames(x) <- interaction(expand.grid(dn[-1]))
         }
-        as.data.frame.matrix(x, row.names, optional)
+        as.data.frame.matrix(x, row.names, optional, ...)
     }
 }
 
 ## will always have a class here
 "[.AsIs" <- function(x, i, ...) structure(NextMethod("["), class = class(x))
 
-as.data.frame.AsIs <- function(x, row.names = NULL, optional = FALSE)
+as.data.frame.AsIs <- function(x, row.names = NULL, optional = FALSE, ...)
 {
     ## why not remove class and NextMethod here?
     if(length(dim(x))==2)
@@ -249,7 +281,7 @@ as.data.frame.AsIs <- function(x, row.names = NULL, optional = FALSE)
             else if(length(row.names <- names(x)) == nrows &&
                     !any(duplicated(row.names))) {}
             else if(optional) row.names <- character(nrows)
-            else row.names <- as.character(1:nrows)
+            else row.names <- seq_len(nrows)
         }
         value <- list(x)
         if(!optional) names(value) <- nm
@@ -264,12 +296,14 @@ as.data.frame.AsIs <- function(x, row.names = NULL, optional = FALSE)
 ###  It does everything by calling the methods presented above.
 
 data.frame <-
-    function(..., row.names = NULL, check.rows = FALSE, check.names = TRUE)
+    function(..., row.names = NULL, check.rows = FALSE, check.names = TRUE,
+             stringsAsFactors = default.stringsAsFactors())
 {
     data.row.names <-
 	if(check.rows && missing(row.names))
 	    function(current, new, i) {
-		new <- as.character(new)
+		if(is.character(current)) new <- as.character(new)
+		if(is.character(new)) current <- as.character(current)
 		if(any(duplicated(new)))
 		    return(current)
 		if(is.null(current))
@@ -280,7 +314,7 @@ data.frame <-
 	    }
 	else function(current, new, i) {
 	    if(is.null(current)) {
-		if(any(dup <- duplicated(new <- as.character(new)))) {
+		if(any(dup <- duplicated(new))) {
 		    warning("some row.names duplicated: ",
                             paste(which(dup), collapse=","),
                             " --> row.names NOT used")
@@ -293,7 +327,8 @@ data.frame <-
     x <- list(...)
     n <- length(x)
     if(n < 1)
-	return(structure(list(), row.names = character(0),
+	return(structure(list(), names = character(0),
+                         row.names = character(0),
 			 class = "data.frame"))
     vnames <- names(x)
     if(length(vnames) != n)
@@ -302,7 +337,12 @@ data.frame <-
     vlist <- vnames <- as.list(vnames)
     nrows <- ncols <- integer(n)
     for(i in 1:n) {
-	xi <- as.data.frame(x[[i]], optional=TRUE)
+        ## do it this way until all as.data.frame methods have been updated
+	xi <- if(is.character(x[[i]]) || is.list(x[[i]]))
+                 as.data.frame(x[[i]], optional = TRUE,
+                               stringsAsFactors = stringsAsFactors)
+        else as.data.frame(x[[i]], optional = TRUE)
+
 	rowsi <- attr(xi, "row.names")
 	ncols[i] <- length(xi)
 	namesi <- names(xi)
@@ -372,8 +412,9 @@ data.frame <-
         warning("row names were found from a short variable and have been discarded")
         row.names <- NULL
     }
-    if(length(row.names) == 0) row.names <- seq(length = nr)
-    row.names <- as.character(row.names)
+    if(length(row.names) == 0) row.names <- seq_len(nr)
+    else if(is.object(row.names) || !is.integer(row.names))
+        row.names <- as.character(row.names)
     if(any(is.na(row.names)))
         stop("row names contain missing values")
     if(any(duplicated(row.names)))
@@ -402,10 +443,12 @@ data.frame <-
 	    return(as.matrix(x)[i])  # desperate measures
 	y <- NextMethod("[")
         nm <- names(y)
-	if(any(is.na(nm))) stop("undefined columns selected")
+        ## zero-column data frames prior to 2.4.0 had no names.
+	if(!is.null(nm) && any(is.na(nm))) stop("undefined columns selected")
         ## added in 1.8.0
         if(any(duplicated(nm))) names(y) <- make.unique(nm)
-	return(structure(y, class = oldClass(x), row.names = row.names(x)))
+	return(structure(y, class = oldClass(x),
+                         row.names = attr(x, "row.names")))
     }
 
     ## preserve the attributes for later use ...
@@ -424,14 +467,14 @@ data.frame <-
     }
     else { # df[i, j] or df[i , ]
 	if(is.character(i))
-	    i <- pmatch(i, rows, duplicates.ok = TRUE)
+	    i <- pmatch(i, as.character(rows), duplicates.ok = TRUE)
 	rows <- rows[i]
 	if(!missing(j)) { # df[i, j]
 	    x <- x[j]
 	    cols <- names(x)
 	    if(any(is.na(cols))) stop("undefined columns selected")
 	}
-	for(j in seq(along = x)) {
+	for(j in seq_along(x)) {
 	    xj <- x[[j]]
             ## had drop = drop prior to 1.8.0
 	    x[[j]] <- if(length(dim(xj)) != 2) xj[i] else xj[i, , drop = FALSE]
@@ -460,7 +503,7 @@ data.frame <-
 	names(x) <- cols
         ## row names might have NAs.
 	if(any(is.na(rows) | duplicated(rows))) {
-            rows[is.na(rows)] <- "NA"
+            rows[is.na(rows)] <- "NA"  # will coerce to character if needed
 	    rows <- make.unique(rows)
         }
         ## new in 1.8.0  -- might have duplicate columns
@@ -513,7 +556,7 @@ data.frame <-
                     stop("rhs is the wrong length for indexing by a logical matrix")
                 n <- 0
                 nv <- nrow(x)
-                for(v in seq(len = dim(i)[2])) {
+                for(v in seq_len(dim(i)[2])) {
                     thisvar <- i[, v, drop = TRUE]
                     nv <- sum(thisvar, na.rm = TRUE)
                     if(nv) {
@@ -555,7 +598,7 @@ data.frame <-
 	    ii <- match(i, rows)
 	    nextra <- sum(new.rows <- is.na(ii))
 	    if(nextra > 0) {
-		ii[new.rows] <- seq(from = nrows + 1, length = nextra)
+		ii[new.rows] <- seq.int(from = nrows + 1, length = nextra)
 		new.rows <- i[new.rows]
 	    }
 	    i <- ii
@@ -577,7 +620,7 @@ data.frame <-
 	    rows <- attr(x, "row.names")
 	    nrows <- length(rows)
 	}
-	iseq <- seq(along = rows)[i]
+	iseq <- seq_along(rows)[i]
 	if(any(is.na(iseq)))
 	    stop("non-existent rows not allowed")
     }
@@ -596,11 +639,11 @@ data.frame <-
 	    jseq <- jj
 	}
 	else if(is.logical(j) || min(j) < 0)
-	    jseq <- seq(along = x)[j]
+	    jseq <- seq_along(x)[j]
 	else {
 	    jseq <- j
 	    if(max(jseq) > nvars) {
-		new.cols <- paste("V", seq(from = nvars + 1, to = max(jseq)),
+		new.cols <- paste("V", seq.int(from = nvars + 1, to = max(jseq)),
                                   sep = "")
 		if(length(new.cols)  != sum(jseq > nvars))
 		    stop("new columns would leave holes after existing columns")
@@ -613,7 +656,7 @@ data.frame <-
 	    }
 	}
     }
-    else jseq <- seq(along = x)
+    else jseq <- seq_along(x)
     ## addition in 1.8.0
     if(any(duplicated(jseq)))
         stop("duplicate subscripts for columns")
@@ -648,7 +691,7 @@ data.frame <-
 	## value <- as.data.frame(value)
         value <- unclass(value) # to avoid data frame indexing
         lens <- sapply(value, NROW)
-        for(k in seq(along=lens)) {
+        for(k in seq_along(lens)) {
             N <- lens[k]
             if(n != N && length(dim(value[[k]])) == 2)
                 stop(gettextf("replacement element %d is a matrix/data frame of %d rows, need %d", k, N, n),
@@ -677,7 +720,7 @@ data.frame <-
 	warning(gettextf("replacement data has %d rows to replace %d rows",
                          nrowv, n), domain = NA)
     ncolv <- dimv[2]
-    jvseq <- seq(len=p)
+    jvseq <- seq_len(p)
     if(ncolv < p) jvseq <- rep(1:ncolv, length.out = p)
     else if(ncolv > p)
 	warning(gettextf("provided %d variables to replace %d variables",
@@ -691,7 +734,7 @@ data.frame <-
         attr(x, "row.names") <- rows
     }
     if(has.i)
-	for(jjj in seq(len=p)) {
+	for(jjj in seq_len(p)) {
 	    jj <- jseq[jjj]
 	    vjj <- value[[ jvseq[[jjj]] ]]
             if(jj <= nvars) {
@@ -766,7 +809,7 @@ data.frame <-
 	ii <- match(i, rows)
 	n <- sum(new.rows <- is.na(ii))
 	if(n > 0) {
-	    ii[new.rows] <- seq(from = nrows + 1, length = n)
+	    ii[new.rows] <- seq.int(from = nrows + 1, length = n)
 	    new.rows <- i[new.rows]
 	}
 	i <- ii
@@ -788,7 +831,7 @@ data.frame <-
 	rows <- attr(x, "row.names")
 	nrows <- length(rows)
     }
-    iseq <- seq(along = rows)[i]
+    iseq <- seq_along(rows)[i]
     if(any(is.na(iseq)))
 	stop("non-existent rows not allowed")
     if(is.character(j)) {
@@ -797,7 +840,7 @@ data.frame <-
 	    stop("replacing element in non-existent column: ", j[is.na(jseq)])
     }
     else if(is.logical(j) || min(j) < 0)
-	jseq <- seq(along = x)[j]
+	jseq <- seq_along(x)[j]
     else {
 	jseq <- j
 	if(max(jseq) > nvars)
@@ -866,7 +909,7 @@ xpdrows.data.frame <- function(x, old.rows, new.rows)
 	    x[[i]] <- y
 	}
     }
-    attr(x, "row.names") <- as.character(c(old.rows, new.rows))
+    attr(x, "row.names") <- c(old.rows, new.rows)
     x
 }
 
@@ -900,7 +943,7 @@ rbind.data.frame <- function(..., deparse.level = 1)
 	    else nmi
 	}
 	else if(nrow > 0 && identical(ri, 1:ni))
-	    seq(from = nrow + 1, length = ni)
+	    as.integer(seq.int(from = nrow + 1, length = ni))
 	else ri
     }
     allargs <- list(...)
@@ -909,7 +952,7 @@ rbind.data.frame <- function(..., deparse.level = 1)
     if(n == 0)
 	return(structure(list(),
 			 class = "data.frame",
-			 row.names = character()))
+			 row.names = integer()))
     nms <- names(allargs)
     if(is.null(nms))
 	nms <- character(length(allargs))
@@ -927,7 +970,7 @@ rbind.data.frame <- function(..., deparse.level = 1)
 	if(inherits(xi, "data.frame")) {
 	    if(is.null(cl))
 		cl <- oldClass(xi)
-	    ri <- row.names(xi)
+	    ri <- attr(xi, "row.names")
 	    ni <- length(ri)
 	    if(is.null(clabs))
 		clabs <- names(xi)
@@ -936,7 +979,7 @@ rbind.data.frame <- function(..., deparse.level = 1)
 		if( !is.null(pi) )
 		    perm[[i]] <- pi
 	    }
-	    rows[[i]] <- seq(from = nrow + 1, length = ni)
+	    rows[[i]] <- seq.int(from = nrow + 1, length = ni)
 	    rlabs[[i]] <- Make.row.names(nmi, ri, ni, nrow)
 	    nrow <- nrow + ni
 	    if(is.null(value)) {
@@ -973,7 +1016,7 @@ rbind.data.frame <- function(..., deparse.level = 1)
 	    if(ni[1] == ni[2])
 		ni <- ni[1]
 	    else stop("invalid list argument: all variables should have the same length")
-	    rows[[i]] <- ri <- seq(from = nrow + 1, length = ni)
+	    rows[[i]] <- ri <- seq.int(from = nrow + 1, length = ni)
 	    nrow <- nrow + ni
 	    rlabs[[i]] <- Make.row.names(nmi, ri, ni, nrow)
 	    if(length(nmi <- names(xi)) > 0) {
@@ -988,7 +1031,7 @@ rbind.data.frame <- function(..., deparse.level = 1)
 	}
 	else if(length(xi) > 0) {
 	    rows[[i]] <- nrow <- nrow + 1
-	    rlabs[[i]] <- if(nchar(nmi) > 0) nmi else nrow
+	    rlabs[[i]] <- if(nchar(nmi) > 0) nmi else as.integer(nrow)
 	}
     }
     nvar <- length(clabs)
@@ -996,7 +1039,7 @@ rbind.data.frame <- function(..., deparse.level = 1)
 	nvar <- max(sapply(allargs, length))	# only vector args
     if(nvar == 0)
 	return(structure(list(), class = "data.frame",
-			 row.names = character()))
+			 row.names = integer()))
     pseq <- 1:nvar
     if(is.null(value)) {
 	value <- list()
@@ -1044,7 +1087,9 @@ rbind.data.frame <- function(..., deparse.level = 1)
             }
 	}
     }
-    rlabs <- make.unique(as.character(unlist(rlabs)), sep = "")
+    rlabs <- unlist(rlabs)
+    if(any(duplicated(rlabs)))
+        rlabs <- make.unique(as.character(unlist(rlabs)), sep = "")
     if(is.null(cl)) {
 	as.data.frame(value, row.names = rlabs)
     } else {
@@ -1150,12 +1195,12 @@ Math.data.frame <- function (x, ...)
     if (all(mode.ok)) {
 	r <- lapply(x, var.f)
 	class(r) <- oldClass(x)
-	row.names(r) <- row.names(x)
+	attr(r, "row.names") <- attr(x, "row.names")
 	return(r)
     }
     else {
 	vnames <- names(x)
-	if (is.null(vnames)) vnames <- seq(along=x)
+	if (is.null(vnames)) vnames <- seq_along(x)
 	stop("non-numeric variable in data frame: ", vnames[!mode.ok])
     }
 }
@@ -1169,9 +1214,7 @@ Ops.data.frame <- function(e1, e2 = NULL)
     value <- list()
     ## set up call as op(left, right)
     FUN <- get(.Generic, envir = parent.frame(), mode="function")
-    f <- if (unary)
-	quote(FUN(left))
-    else quote(FUN(left, right))
+    f <- if (unary) quote(FUN(left)) else quote(FUN(left, right))
     lscalar <- rscalar <- FALSE
     if(lclass && rclass) {
 	rn <- row.names(e1)
@@ -1209,7 +1252,7 @@ Ops.data.frame <- function(e1, e2 = NULL)
 			    rep.int(1:ncol(e2), rep.int(nrow(e2), ncol(e2))))
 	}
     }
-    for(j in seq(along=cn)) {
+    for(j in seq_along(cn)) {
 	left <- if(!lscalar) e1[[j]] else e1
 	right <-if(!rscalar) e2[[j]] else e2
 	value[[j]] <- eval(f)
