@@ -3,6 +3,6 @@ pictex <-
 	     bg = "white", fg = "black")
 {
     .External(PicTeX, file, bg, fg, width, height, as.logical(debug))
-              
-    par(mar = c(5,4,2,4)+0.1)
+
+    graphics::par(mar = c(5,4,2,4)+0.1)
 }
