@@ -4,7 +4,8 @@ windows <- function(width = 7, height = 7, pointsize = 12,
                     bg = "transparent", canvas = "white",
                     gamma = getOption("gamma"),
                     xpos = NA, ypos = NA,
-                    buffered = getOption("windowsBuffered"))
+                    buffered = getOption("windowsBuffered"),
+                    restoreConsole = FALSE)
 {
     rescale <- match.arg(rescale)
     rescale <- match(rescale, c("R", "fit", "fixed"))
@@ -14,14 +15,15 @@ windows <- function(width = 7, height = 7, pointsize = 12,
     if(missing(ypinch))
         if(!length(ypinch <- getOption("ypinch"))) ypinch <- NA
     ypinch <- as.double(ypinch)
-    invisible(.External("devga", "", width, height, pointsize, record, rescale,
-                        xpinch, ypinch, canvas,
+    invisible(.External("devga", "", width, height, pointsize, record,
+                        rescale, xpinch, ypinch, canvas,
                         if(is.null(gamma)) 1 else gamma,
                         as.integer(xpos), as.integer(ypos), buffered,
-                        .PSenv, bg, PACKAGE = "grDevices"))
+                        .PSenv, bg, restoreConsole, PACKAGE = "grDevices"))
 }
 
-win.graph <- function(width = 7, height = 7, pointsize = 12)
+win.graph <- function(width = 7, height = 7, pointsize = 12,
+                      restoreConsole = FALSE)
 {
     gamma <- getOption("gamma")
     if(!length(xpinch <- getOption("xpinch"))) xpinch <- NA
@@ -32,40 +34,43 @@ win.graph <- function(width = 7, height = 7, pointsize = 12)
                         if(is.null(gamma)) 1 else gamma,
                         as.integer(NA), as.integer(NA),
                         getOption("windowsBuffered"),
-                        .PSenv, NA, PACKAGE = "grDevices"))
+                        .PSenv, NA, restoreConsole, PACKAGE = "grDevices"))
 }
 
-win.print <- function(width = 7, height = 7, pointsize = 12, printer = "")
+win.print <- function(width = 7, height = 7, pointsize = 12, printer = "",
+                      restoreConsole = TRUE)
     invisible(.External("devga", paste("win.print:", printer, sep=""),
                         width, height, pointsize, FALSE, 1,
                         NA, NA, "white", 1, as.integer(NA), as.integer(NA),
-                        FALSE, .PSenv, NA, PACKAGE = "grDevices"))
+                        FALSE, .PSenv, NA, restoreConsole,
+                        PACKAGE = "grDevices"))
 
-win.metafile <- function(filename = "", width = 7, height = 7, pointsize = 12)
+win.metafile <- function(filename = "", width = 7, height = 7, pointsize = 12,
+                         restoreConsole = TRUE)
     invisible(.External("devga", paste("win.metafile:", filename, sep=""),
                         width, height, pointsize, FALSE, 1, NA, NA, "white", 1,
                         as.integer(NA), as.integer(NA), FALSE, .PSenv, NA,
-                        PACKAGE = "grDevices"))
+                        restoreConsole, PACKAGE = "grDevices"))
 
 png <- function(filename = "Rplot%03d.png", width = 480, height = 480,
-                pointsize = 12, bg = "white", res = NA)
+                pointsize = 12, bg = "white", res = NA, restoreConsole = TRUE)
     invisible(.External("devga", paste("png:", filename, sep=""),
                         width, height, pointsize, FALSE, 1, NA, NA, bg, 1,
                         as.integer(res), as.integer(NA), FALSE, .PSenv, NA,
-                        PACKAGE = "grDevices"))
+                        restoreConsole, PACKAGE = "grDevices"))
 
 bmp <- function(filename = "Rplot%03d.bmp", width = 480, height = 480,
-                pointsize = 12, bg = "white", res = NA)
+                pointsize = 12, bg = "white", res = NA, restoreConsole = TRUE)
     invisible(.External("devga", paste("bmp:", filename, sep=""),
                         width, height, pointsize, FALSE, 1, NA, NA, bg, 1,
                         as.integer(res), as.integer(NA), FALSE, .PSenv, NA,
-                        PACKAGE = "grDevices"))
+                        restoreConsole, PACKAGE = "grDevices"))
 
 jpeg <- function(filename = "Rplot%03d.jpg", width = 480, height = 480,
-                 pointsize = 12, quality=75, bg = "white", res = NA)
+                 pointsize = 12, quality=75, bg = "white", res = NA, restoreConsole = TRUE)
     invisible(.External("devga", paste("jpeg:", quality, ":",filename, sep=""),
                         width, height, pointsize, FALSE, 1, NA, NA, bg, 1,
-                        as.integer(res), as.integer(NA), FALSE, .PSenv, NA,
+                        as.integer(res), as.integer(NA), FALSE, .PSenv, NA, restoreConsole,
                         PACKAGE = "grDevices"))
 
 bringToTop <- function(which = dev.cur(), stay = FALSE)
@@ -81,7 +86,8 @@ bringToTop <- function(which = dev.cur(), stay = FALSE)
 savePlot <- function(filename = "Rplot",
                      type = c("wmf", "emf", "png", "jpeg", "jpg", "bmp",
                      "ps", "eps", "pdf"),
-                     device = dev.cur())
+                     device = dev.cur(),
+                     restoreConsole = TRUE)
 {
     type <- match.arg(type)
     devlist <- dev.list()
@@ -91,7 +97,7 @@ savePlot <- function(filename = "Rplot",
     if(devname != "windows") stop("can only copy from 'windows' devices")
     if(filename == "clipboard" && type == "wmf") filename <- ""
     if(nchar(filename) > 0) filename <- paste(filename, type, sep=".")
-    invisible(.External("savePlot", device, filename, type,
+    invisible(.External("savePlot", device, filename, type, restoreConsole,
                         PACKAGE = "grDevices"))
 }
 
@@ -126,34 +132,33 @@ print.SavedPlots <- function(x, ...)
 assign(".Windows.Fonts", list(), envir = .Windowsenv)
 
 # Check that the font has the correct structure and information
-checkWindowsFont <- function(font) {
-  # For now just use the simple format that is used in Rdevga
-  # i.e., just a font family name, possibly with "TT" as the first
-  # two characters to indicate a TrueType font
-  if (!is.character(font) || length(font) != 1)
-    stop("invalid Windows font:  must be a single font family name")
-  font
+checkWindowsFont <- function(font)
+{
+    # For now just use the simple format that is used in Rdevga
+    # i.e., just a font family name, possibly with "TT" as the first
+    # two characters to indicate a TrueType font
+    if (!is.character(font) || length(font) != 1)
+        stop("invalid Windows font:  must be a single font family name")
+    font
 }
 
-setWindowsFonts <- function(fonts, fontNames) {
-  fonts <- lapply(fonts, checkWindowsFont)
-  fontDB <- get(".Windows.Fonts", envir=.Windowsenv)
-  existingFonts <- fontNames %in% names(fontDB)
-  if (sum(existingFonts) > 0)
-    fontDB[fontNames[existingFonts]] <- fonts[existingFonts]
-  if (sum(existingFonts) < length(fontNames))
-    fontDB <- c(fontDB, fonts[!existingFonts])
-  assign(".Windows.Fonts", fontDB, envir=.Windowsenv)
+setWindowsFonts <- function(fonts, fontNames)
+{
+    fonts <- lapply(fonts, checkWindowsFont)
+    fontDB <- get(".Windows.Fonts", envir=.Windowsenv)
+    existingFonts <- fontNames %in% names(fontDB)
+    if (sum(existingFonts) > 0)
+        fontDB[fontNames[existingFonts]] <- fonts[existingFonts]
+    if (sum(existingFonts) < length(fontNames))
+        fontDB <- c(fontDB, fonts[!existingFonts])
+    assign(".Windows.Fonts", fontDB, envir=.Windowsenv)
 }
 
-printFont <- function(font) {
-  paste(font, "\n", sep="")
-}
+printFont <- function(font) paste(font, "\n", sep="")
 
-printFonts <- function(fonts) {
-  cat(paste(names(fonts), ": ", unlist(lapply(fonts, printFont)),
-            sep="", collapse=""))
-}
+printFonts <- function(fonts)
+    cat(paste(names(fonts), ": ", unlist(lapply(fonts, printFont)),
+              sep="", collapse=""))
 
 # If no arguments spec'ed, return entire font database
 # If no named arguments spec'ed, all args should be font names
@@ -161,36 +166,36 @@ printFonts <- function(fonts) {
 # Else, must specify new fonts to enter into database (all
 # of which must be valid PostScript font descriptions and
 # all of which must be named args)
-windowsFonts <- function(...) {
-  ndots <- length(fonts <- list(...))
-  if (ndots == 0)
-    get(".Windows.Fonts", envir=.Windowsenv)
-  else {
-    fontNames <- names(fonts)
-    nnames <- length(fontNames)
-    if (nnames == 0) {
-      if (!all(sapply(fonts, is.character)))
-        stop("invalid arguments in 'windowsFonts' (must be font names)")
-      else
-        get(".Windows.Fonts", envir=.Windowsenv)[unlist(fonts)]
-    } else {
-      if (ndots != nnames)
-        stop("invalid arguments in 'windowsFonts' (need named args)")
-      setWindowsFonts(fonts, fontNames)
+windowsFonts <- function(...)
+{
+    ndots <- length(fonts <- list(...))
+    if (ndots == 0)
+        get(".Windows.Fonts", envir=.Windowsenv)
+    else {
+        fontNames <- names(fonts)
+        nnames <- length(fontNames)
+        if (nnames == 0) {
+            if (!all(sapply(fonts, is.character)))
+                stop("invalid arguments in 'windowsFonts' (must be font names)")
+            else
+                get(".Windows.Fonts", envir=.Windowsenv)[unlist(fonts)]
+        } else {
+            if (ndots != nnames)
+                stop("invalid arguments in 'windowsFonts' (need named args)")
+            setWindowsFonts(fonts, fontNames)
+        }
     }
-  }
 }
 
 # Create a valid windows font description
-windowsFont <- function(family) {
-  checkWindowsFont(family)
-}
+windowsFont <- function(family) checkWindowsFont(family)
+
 
 windowsFonts(# Default Serif font is Times
-                serif=windowsFont("TT Times New Roman"),
-                # Default Sans Serif font is Helvetica
-                sans=windowsFont("TT Arial"),
-                # Default Monospace font is Courier
-                mono=windowsFont("TT Courier New"),
-                # Default Symbol font is Symbol
-                symbol=windowsFont("TT Symbol"))
+             serif = windowsFont("TT Times New Roman"),
+             # Default Sans Serif font is Helvetica
+             sans = windowsFont("TT Arial"),
+             # Default Monospace font is Courier
+             mono = windowsFont("TT Courier New"),
+             # Default Symbol font is Symbol
+             symbol = windowsFont("TT Symbol"))
