@@ -1,8 +1,6 @@
 require(graphics)
-if(dev.cur() <= 1) get(getOption("device"))()
 
-opar <- par(ask = interactive() &&
-            (.Device %in% c("X11", "GTK", "gnome", "windows","quartz")))
+opar <- par(ask = dev.interactive(orNone = TRUE))
 
 ## --- Japanese characters in the Hershey Vector Fonts
 
