@@ -16,14 +16,14 @@ setGeneric <-
              useAsDefault = NULL, genericFunction = NULL)
 {
     if(exists(name, "package:base") &&
-       typeof(get(name, "package:base")) != "closure") {
+       typeof(get(name, "package:base")) != "closure") { # primitives
 
-        getGeneric(name) # will fail if this can't have methods
-        msg <- gettextf("'%s' is a primitive function;  methods can be defined, but the generic function is implicit, and cannot be changed.", name)
-        if(nargs() == 1)
-            warning(msg, domain = NA)
-        else
+        fdef <- getGeneric(name) # will fail if this can't have methods
+        msg <- gettextf("\"%s\" is a primitive function;  methods can be defined, but the generic function is implicit, and cannot be changed.", name)
+        if(nargs() > 1)
             stop(msg, domain = NA)
+        ## generics for primitives are global, so can & must always be cached
+        .cacheGeneric(name, fdef)
         return(name)
     }
     stdGenericBody <- substitute(standardGeneric(NAME), list(NAME = name))
@@ -35,8 +35,11 @@ setGeneric <-
         fdef <- useAsDefault
     ## Use the previous function definition to get the default
     ## and to set the package if not supplied.
-    if(is(fdef, "genericFunction")) {
-        prevDefault <- finalDefaultMethod(getMethods(fdef))
+    doUncache <- FALSE
+    if(is.object(fdef) && is(fdef, "genericFunction")) {
+        doUncache <- TRUE
+        oldDef <- fdef
+        prevDefault <- finalDefaultMethod(fdef@default)
         if(is.null(package))
             package <- fdef@package
     }
@@ -52,7 +55,7 @@ setGeneric <-
     else if(is.function(fdef))
         body(fdef, envir = as.environment(where)) <- stdGenericBody
     if(!is.null(def)) {
-        if(is.primitive(def) || !is(def, "function"))
+        if(is.primitive(def) || !is.function(def))
             stop(gettextf("if the `def' argument is supplied, it must be a function that calls standardGeneric(\"%s\") to dispatch methods", name), domain = NA)
         fdef <- def
         if(is.null(genericFunction) && .NonstandardGenericTest(body(fdef), name, stdGenericBody))
@@ -63,7 +66,7 @@ setGeneric <-
         package <- getPackageName(where)
     if(is.null(fdef))
         stop("must supply a function skeleton, explicitly or via an existing function")
-    if(!is(fdef, "genericFunction")) {
+    if(!(is.object(fdef) && is(fdef, "genericFunction"))) {
         if(is.function(useAsDefault))
             fdeflt <- useAsDefault
         else if(identical(useAsDefault, FALSE))
@@ -76,8 +79,12 @@ setGeneric <-
                             package = package, signature = signature,
                             genericFunction = genericFunction)
     }
+    if(doUncache)
+      .uncacheGeneric(name, oldDef)
     assign(name, fdef, where)
-    if(length(fdef@group)> 0 && !is.null(getGeneric(fdef@group[[1]], where = where)))
+    .cacheGeneric(name, fdef)
+    if(!.UsingMethodsTables() &&
+       length(fdef@group)> 0 && !is.null(getGeneric(fdef@group[[1]], where = where)))
         methods <- getAllMethods(name, fdef, where)
     else
         methods <- fdef@default # empty or containing the default
@@ -97,6 +104,8 @@ isGeneric <-
   ## the generic.  (This argument is not available in S-Plus.)
   function(f, where = topenv(parent.frame()), fdef = NULL, getName = FALSE)
 {
+    if(is.null(fdef))
+        fdef <- .getGenericFromCache(f, where)
     if(is.null(fdef))
         fdef <- getFunction(f, where=where, mustFind = FALSE)
     if(is.null(fdef))
@@ -114,7 +123,7 @@ isGeneric <-
     else if(missing(f) || .identC(gen, f))
         return(TRUE)
     else {
-        warning(gettextf("function '%s' appears to be a generic function, but with generic name '%s'", f, gen), domain = NA)
+        warning(gettextf("function \"%s\" appears to be a generic function, but with generic name \"%s\"", f, gen), domain = NA)
         return(FALSE)
     }
 }
@@ -136,13 +145,14 @@ removeGeneric <-
     }
     found <- is(fdef, "genericFunction")
     if(found) {
+        .uncacheGeneric(f, fdef)
         removeMethodsObject(f, where)
         rm(list = fdef@generic, pos = where)
     }
     else {
         if(!is.character(f))
             f <- deparse(f)
-        warning(gettextf("generic function '%s' not found for removal", f),
+        warning(gettextf("generic function \"%s\" not found for removal", f),
                 domain = NA)
     }
     return(found)
@@ -164,24 +174,50 @@ getMethods <-
     ## method search; however, they are not stored with the original generic
     ## function, since they might change in future sessions.
 
+  ## Note for methods tables.  The function getMethods() continues to
+  ## return a methods list object, even when methods tables are being
+  ## used for dispatch, but now this is the metadata from where, and
+  ## _not_ a fully merged list, since avoiding this computation ws a
+  ## major motivation for the tables.
+
     function(f, where = topenv(parent.frame()))
 {
+    nowhere <- missing(where) # remember it now, R changes the truth later!
     if(is.character(f))
         fdef <- getGeneric(f, where = where)
-    else if(is.function(f))
+    else if(is(f, "genericFunction")) {
+        if(nowhere)
+          where <- .genEnv(f)
         fdef <- f
+        f <- fdef@generic
+    }
     else
-        stop(gettextf("invalid argument 'f', expected function or its name, got an object of class \"%s\"", class(f)), domain = NA)
-    if(!is.null(fdef))
-        getMethodsForDispatch(f, fdef) # else NULL
+        stop(gettextf("invalid argument \"f\", expected a function or its name, got an object of class \"%s\"", class(f)), domain = NA)
+    if(!is.null(fdef)) { # else NULL
+        ## getMethods() always returns a methods  list.  When using
+        ## methods tables, this only makes sense as the metadata from
+        ## where (by default, the location of the generic)
+        if(.UsingMethodsTables()) {
+            gwhere <-  ( if(nowhere)
+                        findFunction(f, TRUE, where)[[1]] # can't be empty since fdef is nonnull
+                        else where)
+            value <- getMethodsMetaData(f, where = gwhere)
+            if(is.null(value)) # return empty methods list
+              new("MethodsList", argument = fdef@default@argument)
+            else
+              value
+        }
+        else
+          getMethodsForDispatch(f, fdef)
+    }
 }
 
 getMethodsForDispatch <- function(f, fdef)
 {
-    if(is(fdef, "genericMethods"))
-        fdef@.Methods
+  ev <- environment(fdef)
+    if(.UsingMethodsTables())
+        .getMethodsTable(fdef, ev)
     else {
-        ev <- environment(fdef)
         if(exists(".Methods", envir = ev, inherits = FALSE))
             get(".Methods", envir = ev) ## else NULL
     }
@@ -222,7 +258,7 @@ getMethodsForDispatch <- function(f, fdef)
     ev <- environment(fdef)
     if(!is(fdef, "genericFunction") ||
        !exists(".Methods", envir = ev, inherits = FALSE))
-        stop(gettextf("internal error: did not get a valid generic function object for function '%s'", f), domain = NA)
+        stop(gettextf("internal error: did not get a valid generic function object for function \"%s\"", f), domain = NA)
     assign(".Methods", envir = ev, mlist)
 }
 
@@ -231,13 +267,17 @@ cacheMethod <-
   ## Support function:  DON'T USE DIRECTLY (does no checking)
   function(f, sig, def, args = names(sig), fdef) {
     ev <- environment(fdef)
-    methods <- get(".Methods", envir = ev)
-    methods <- insertMethod(methods, sig, args, def, TRUE)
-    assign(".Methods", methods, envir = ev)
-    deflt <- finalDefaultMethod(methods)
-    if(is.primitive(deflt))
+    if(.UsingMethodsTables())
+      .cacheMethodInTable(fdef, sig, def, .getMethodsTable(fdef, ev))
+    else {
+      methods <- get(".Methods", envir = ev)
+      methods <- insertMethod(methods, sig, args, def, TRUE)
+      assign(".Methods", methods, envir = ev)
+      deflt <- finalDefaultMethod(methods)
+      if(is.primitive(deflt))
         setPrimitiveMethods(f, deflt, "set", fdef, methods)
-    methods
+      methods
+    }
   }
 
 
@@ -260,7 +300,16 @@ setMethod <-
         f <- fdef@generic
         gwhere <- .genEnv(f)
     }
-    ## slight subtlety:  calling getGeneric vs calling isGeneric
+    else if(is.function(f)) {
+          if(is.primitive(f)) {
+            f <- .primname(f)
+            fdef <- genericForPrimitive(f)
+            gwhere <- .genEnv(f)
+         }
+          else
+            stop("A function for argument \"f\" must be a generic function")
+    }
+      ## slight subtlety:  calling getGeneric vs calling isGeneric
     ## For primitive functions, getGeneric returns the (hidden) generic function,
     ## even if no methods have been defined.  An explicit generic MUST NOT be
     ## for these functions, dispatch is done inside the evaluator.
@@ -270,7 +319,7 @@ setMethod <-
         fdef <- getGeneric(f, where = if(identical(gwhere, baseenv())) where else gwhere)
     }
     if(.lockedForMethods(fdef, where))
-        stop(gettextf("the environment '%s' is locked; cannot assign methods for function '%s'", getPackageName(where), f), domain = NA)
+        stop(gettextf("the environment \"%s\" is locked; cannot assign methods for function \"%s\"", getPackageName(where), f), domain = NA)
     hasMethods <- !is.null(fdef)
     deflt <- getFunction(f, generic = FALSE, mustFind = FALSE, where = where)
     ## where to insert the methods in generic
@@ -278,7 +327,7 @@ setMethod <-
         allWhere <- findFunction(f, where = where)
         generics <-logical(length(allWhere))
         if(length(allWhere)>0) { # put methods into existing generic
-            for(i in seq(along = allWhere)) {
+            for(i in seq_along(allWhere)) {
                 fi <- get(f, allWhere[[i]])
                 geni <- is(fi, "genericFunction")
                 generics[[i]] <- geni
@@ -295,7 +344,7 @@ setMethod <-
                     stop(gettextf("the 'where' environment (%s) is a locked namespace; cannot assign methods there",
                                   getPackageName(where)), domain = NA)
                 msg <-
-                    gettextf("copying the generic function '%s' to environment '%s', because the previous version was in a sealed namespace (%s)",
+                    gettextf("copying the generic function \"%s\" to environment \"%s\", because the previous version was in a sealed namespace (%s)",
                              f,
                              getPackageName(where),
                              getPackageName(gwhere))
@@ -308,10 +357,10 @@ setMethod <-
     if(!hasMethods)
       fdef <- deflt
     if(is.null(fdef))
-      stop(gettextf("no existing definition for function '%s'", f),
+      stop(gettextf("no existing definition for function \"%s\"", f),
            domain = NA)
     if(!hasMethods) {
-        message(gettextf("Creating a new generic function for '%s' in '%s'",
+        message(gettextf("Creating a new generic function for \"%s\" in \"%s\"",
                          f, getPackageName(where)),
                 domain = NA)
         ## create using the visible non-generic as a pattern and default method
@@ -321,11 +370,13 @@ setMethod <-
     else if(identical(gwhere, NA)) {
         ## better be a primitive since getGeneric returned a generic, but none was found
         if(is.null(elNamed(.BasicFunsList, f)))
-            stop(gettextf("apparent internal error: a generic function was found for '%s', but no corresponding object was found searching from '%s'",
+            stop(gettextf("apparent internal error: a generic function was found for \"%s\", but no corresponding object was found searching from \"%s\"",
                           f, getPackageName(where)), domain = NA)
+        if(!isGeneric(f))
+          setGeneric(f) #  turn on this generic and cache it.
     }
     if(isSealedMethod(f, signature, fdef))
-        stop(gettextf("the method for function '%s' and signature %s is sealed and cannot be re-defined", f, .signatureString(fdef, signature)), domain = NA)
+        stop(gettextf("the method for function \"%s\" and signature %s is sealed and cannot be re-defined", f, .signatureString(fdef, signature)), domain = NA)
     signature <- matchSignature(signature, fdef, where)
     switch(typeof(definition),
            closure = {
@@ -355,12 +406,26 @@ setMethod <-
            },
            stop(gettextf("invalid method definition: expected a function, got an object of class \"%s\"", class(definition)), domain = NA)
            )
+    fenv <- environment(fdef)
+    ## check length against active sig. length, reset if necessary in .addToMetaTable
+    nSig <- .getGenericSigLength(fdef, fenv, TRUE)
+    signature <- .matchSigLength(signature, fdef, fenv, TRUE)
     margs  <- (fdef@signature)[1:length(signature)]
     definition <- asMethodDefinition(definition, signature, sealed)
+    if(is(definition, "MethodDefinition"))
+      definition@generic <- fdef@generic
     whereMethods <- .getOrMakeMethodsList(f, where, fdef)
     whereMethods <- insertMethod(whereMethods, signature, margs, definition)
-    allMethods <- getMethods(fdef)
-    allMethods <- insertMethod(allMethods, signature, margs, definition)
+    allMethods <- getMethodsForDispatch(f, fdef)
+    if(is.environment(allMethods)) {
+        ## cache in both direct and inherited tables
+      .cacheMethodInTable(fdef, signature, definition, allMethods) #direct
+      .cacheMethodInTable(fdef, signature, definition) # inherited, by default
+      if(!identical(where, baseenv()))
+         .addToMetaTable(fdef, signature, definition,where, nSig)
+    }
+    else # but currently unused (9/06)-- should be deleted after testing
+      allMethods <- insertMethod(allMethods, signature, margs, definition)
     resetGeneric(f, fdef, allMethods, gwhere, deflt) # Note: gwhere not used by resetGeneric
     ## assigns the methodslist object
     ## and deals with flags for primitives & for updating group members
@@ -370,13 +435,22 @@ setMethod <-
 }
 
 removeMethod <- function(f, signature = character(), where = topenv(parent.frame())) {
-    fdef <- getGeneric(f, where = where)
+    if(is.function(f)) {
+      if(is(f, "genericFunction"))
+         { fdef <- f; f <- f@generic}
+      else if(is.primitive(f))
+        { f <- .primname(f); fdef <- genericForPrimitive(f)}
+      else
+        stop("Function supplied as argument \"f\" must be a generic")
+    }
+    else
+      fdef <- getGeneric(f, where = where)
     if(is.null(fdef)) {
-        warning(gettextf("no generic function '%s' found", f), domain = NA)
+        warning(gettextf("no generic function \"'%s\" found", f), domain = NA)
         return(FALSE)
     }
     if(is.null(getMethod(fdef, signature, optional=TRUE))) {
-        warning(gettextf("no method found for function '%s' and signature %s",
+        warning(gettextf("no method found for function \"%s\" and signature %s",
                          fdef@generic,
                          paste(dQuote(signature), collapse =", ")),
                 domain = NA)
@@ -386,11 +460,22 @@ removeMethod <- function(f, signature = character(), where = topenv(parent.frame
     TRUE
 }
 
+## an extension to removeMethod that resets inherited methods as well
+.undefineMethod <- function(f, signature = character(), where = topenv(parent.frame())) {
+    fdef <- getGeneric(f, where = where)
+    if(is.null(fdef)) {
+        warning(gettextf("no generic function \"%s\" found", f), domain = NA)
+        return(FALSE)
+    }
+    if(!is.null(getMethod(fdef, signature, optional=TRUE)))
+      setMethod(f, signature, NULL, where = where)
+  }
+
 findMethod <- function(f, signature, where = topenv(parent.frame())) {
     fM <- mlistMetaName(f)
     where <- .findAll(fM, where)
     found <- logical(length(where))
-    for(i in seq(along = where)) {
+    for(i in seq_along(where)) {
         wherei <- where[[i]]
         mi <- get(fM, wherei, inherits=FALSE)
         mi <- getMethod(f, signature, where = wherei, optional = TRUE, mlist = mi)
@@ -412,24 +497,30 @@ getMethod <-
   ## Return the function that is defined as the method for this generic function and signature
   ## (classes to be matched to the arguments of the generic function).
   function(f, signature = character(), where = topenv(parent.frame()), optional = FALSE,
-           mlist)
+           mlist, fdef = getGeneric(f, !optional, where = where))
 {
-    fdef <- getGeneric(f, !optional, where = where)
+    if(!is(fdef, "genericFunction")) # must be the optional case, else an error in getGeneric
+        return(NULL)
     if(missing(mlist)) {
         if(missing(where))
-            mlist <- getMethods(f)
+            mlist <- getMethodsForDispatch(f, fdef)
         else
             mlist <- getMethods(f, where)
     }
-    if(!is(fdef, "genericFunction")) # must be the optional case, else an error in getGeneric
-        return(NULL)
+    if(is.environment(mlist)) {
+        value <- .findMethodInTable(signature, mlist, fdef)
+        if(is.null(value) && !optional)
+          stop(gettextf('No method found for function "%s" and signature %s',
+                        f, paste(signature, collapse = ", ")))
+        return(value)
+    }
     i <- 1
     argNames <- fdef@signature
     signature <- matchSignature(signature, fdef)
     Classes <- signature # a copy just for possible error message
     while(length(signature) > 0 && is(mlist, "MethodsList")) {
         if(!identical(argNames[[i]], as.character(mlist@argument)))
-            stop(gettextf("apparent inconsistency in the methods for function '%s'; argument '%s' in the signature corresponds to '%s' in the methods list object",
+            stop(gettextf("apparent inconsistency in the methods for function \"%s\"; argument \"%s\" in the signature corresponds to \"%s\" in the methods list object",
                           .genericName(f), argNames[[i]], as.character(mlist@argument)), domain = NA)
         Class <- signature[[1]]
         signature <- signature[-1]
@@ -449,7 +540,7 @@ getMethod <-
     else {
         ## for friendliness, look for (but don't return!) an S3 method
         if(length(Classes) == 1 && exists(paste(.genericName(f), Classes, sep="."), where))
-            stop(gettextf("no S4 method for function '%s' and signature %s; consider getS3method() if you wanted the S3 method",
+            stop(gettextf("no S4 method for function \"%s\" and signature %s; consider getS3method() if you wanted the S3 method",
                           .genericName(f), Classes), domain = NA)
         if(length(Classes) > 0) {
             length(argNames) <- length(Classes)
@@ -458,7 +549,7 @@ getMethod <-
         }
         else
             Classes <- "\"ANY\""
-        stop(gettextf("no method defined for function '%s' and signature %s",
+        stop(gettextf("no method defined for function \"%s\" and signature %s",
                       .genericName(f), Classes), domain = NA)
     }
 }
@@ -497,6 +588,34 @@ selectMethod <-
              mlist = (if(is.null(fdef)) NULL else getMethodsForDispatch(f, fdef)),
              fdef = getGeneric(f, !optional))
 {
+    if(is.environment(mlist))  {# using methods tables
+        fenv <- environment(fdef)
+        nsig <- .getGenericSigLength(fdef, fenv, FALSE)
+        if(length(signature) < nsig)
+          signature[(length(signature)+1):nsig] <- "ANY"
+        if(missing(useInherited))
+          useInherited <- is.na(match(signature, "ANY"))
+        allmethods <- .getMethodsTable(fdef, fenv, FALSE, TRUE)
+        method <- .findMethodInTable(signature, mlist, fdef)
+        if(is.null(method)) {
+            if(any(useInherited))
+              ## look in the supplied (usually standard) table, cache w. inherited
+              methods <- .findInheritedMethods(signature, fdef,
+                                               mtable = allmethods,
+                                               table = mlist,
+                                               useInherited = useInherited)
+            else # just look in the direct table
+              methods <- list()
+            if(length(methods)>0)
+              return(methods[[1]])
+            else if(optional)
+              return(NULL)
+            else stop(gettextf("No method found for signature %s", paste(signature,
+                                                                        collapse=", ")))
+        }
+        else
+          return(method)
+  }
     evalArgs <- is.environment(signature)
     if(evalArgs)
         env <- signature
@@ -515,7 +634,7 @@ selectMethod <-
         if(optional)
             return(mlist)
         else
-            stop(gettextf("'%s' has no methods defined", f), domain = NA)
+            stop(gettextf('"%s" has no methods defined', f), domain = NA)
     }
     selection <- .Call("R_selectMethod", f, env, mlist, evalArgs, PACKAGE = "methods")
     if(is.null(selection) && !identical(useInherited, FALSE)) {
@@ -570,10 +689,16 @@ existsMethod <-
   ## this signature.
   function(f, signature = character(), where = topenv(parent.frame()))
 {
-    if(isGeneric(f, where))
-        !is.null(getMethod(f, signature, where = where, optional = TRUE))
-    else
-        FALSE
+    fdef <- getGeneric(f, FALSE, where = where)
+    if(is.null(fdef))
+      FALSE
+    else  {
+        if(missing(where))
+          method <- getMethod(f, signature, fdef = fdef, optional = TRUE)
+        else
+          method <- getMethod(f, signature, where = where, optional = TRUE)
+        !is.null(method)
+    }
 }
 
 dumpMethods <-
@@ -611,12 +736,15 @@ signature <-
   function(...)
 {
     value <- list(...)
-    for(i in seq(along=value)) {
+    names <- names(value)
+    for(i in seq_along(value)) {
         sigi <- el(value, i)
         if(!is.character(sigi) || length(sigi) != 1)
             stop(gettextf("bad class specified for element %d (should be a single character string)", i), domain = NA)
     }
-    value
+      value <- as.character(value)
+      names(value) <- names
+      value
 }
 
 showMethods <-
@@ -633,15 +761,18 @@ showMethods <-
     ##
     function(f = character(), where = topenv(parent.frame()), classes = NULL,
              includeDefs = FALSE, inherited = TRUE,
-             showEmpty = TRUE, printTo = stdout())
+             showEmpty, printTo = stdout())
 {
-## FIXME: improve default for 'where'  and 'showEmpty' (to FALSE)
+    if(missing(showEmpty))
+      showEmpty <- !missing(f)
     if(identical(printTo, FALSE)) {
         tmp <- tempfile()
         on.exit(unlink(tmp))
         con <- file(tmp, "w")
     }
     else con <- printTo
+    ## must resolve showEmpty in line; using an equivalent default
+    ## fails because R resets the "missing()" result for f later on (grumble)
     if(is(f, "function"))
         f <- as.character(substitute(f))
     if(!is(f, "character"))
@@ -652,39 +783,44 @@ showMethods <-
     if(length(f) == 0)
 	cat(file = con, "No applicable functions\n")
     else if(length(f) > 1) {
-        value <- character()
+        if(identical(printTo, FALSE))
+          stop(gettextf("The special case of printTo=FALSE only works when a single generic function is specified"))
         for(ff in f) { ## recall for each
-            mlist <- getMethods(ff, where)
-            if(length(mlist@methods) == 0)
+            fdef <- getGeneric(ff, where = where)
+            if(is.null(fdef))
                 next
-            value <- c(value,
-                       Recall(ff, where=where, classes=classes,
-                              includeDefs=includeDefs, inherited=inherited,
-                              showEmpty=showEmpty, printTo=printTo))
+            Recall(ff, where=where, classes=classes,
+                   includeDefs=includeDefs, inherited=inherited,
+                   showEmpty=showEmpty, printTo=printTo)
         }
-        if(length(value) > 0)
-            return(value)
-        else
-            return()
+
     }
     else { ## f of length 1 --- the "working horse" :
         out <- paste("\nFunction \"", f, "\":\n", sep="")
         if(!isGeneric(f, where))
             cat(file = con, out, "<not a generic function>\n")
         else {
-            mlist <-  getMethods(f, where)
-            if(is.null(mlist))
-                cat(file = con, out, "<no applicable methods>\n")
+            if(.UsingMethodsTables()) {
+		## maybe no output for showEmpty=FALSE
+		.showMethodsTable(getGeneric(f, where = where), includeDefs, inherited,
+				  classes = classes, showEmpty = showEmpty,
+				  printTo = con)
+            }
             else {
-##NotYet         linML <- linearizeMlist(mlist, inherited,
-##NotYet                                  drop.empty4class = classes)
-##NotYet         ##_ if(showEmpty || length(linML@methods) > 0) {
-##NotYet             cat(file = con, out)
-##NotYet             showMlist(linML = linML, includeDefs = includeDefs,
-##NotYet                       printTo = con)
-##NotYet         ##_ }
+              mlist <-  getMethods(f, where)
+              if(is.null(mlist))
+                cat(file = con, out, "<no applicable methods>\n")
+              else {
+                ##NotYet         linML <- linearizeMlist(mlist, inherited,
+                ##NotYet                                  drop.empty4class = classes)
+                ##NotYet         ##_ if(showEmpty || length(linML@methods) > 0) {
+                ##NotYet             cat(file = con, out)
+                ##NotYet             showMlist(linML = linML, includeDefs = includeDefs,
+                ##NotYet                       printTo = con)
+                ##NotYet         ##_ }
                 cat(file = con, out)
                 showMlist(mlist, includeDefs, inherited, classes, printTo = con)
+              }
             }
         }
     }
@@ -692,17 +828,26 @@ showMethods <-
         close(con)
         readLines(tmp)
     }
+    else
+      invisible(printTo)
 }
 
+## this should be made obsolete
 removeMethodsObject <-
     function(f, where = topenv(parent.frame()))
 {
-  what <- mlistMetaName(f)
+    fdef <- getGeneric(f, where=where)
+    if(!is(fdef, "genericFunction")) {
+      warning(gettextf(
+    "No generic function found for \"%s\"; no action taken in removeMethodsObject", f))
+      return(FALSE)
+  }
+  what <- mlistMetaName(f, fdef@package)
   if(!exists(what, where))
       return(FALSE)
   where <- as.environment(where)
   if(environmentIsLocked(where)) {
-      warning(gettextf("the environment/package '%s' is locked; cannot remove methods data for '%s'",
+      warning(gettextf("the environment/package \"%s\" is locked; cannot remove methods data for \"%s\"",
                        getPackageName(where), f), domain = NA)
       return(FALSE)
   }
@@ -731,18 +876,25 @@ removeMethods <-
     ## the peculiar order of computations and the explicit use of missing(where).
     fdef <- getGeneric(f, where = where)
     if(!is(fdef, "genericFunction")) {
-        warning(gettextf("'%s' is not a generic function in '%s'; methods not removed",
+        warning(gettextf("\"%s\" is not a generic function in \"%s\"; methods not removed",
                 f, getPackageName(where)), domain = NA)
         return(FALSE)
     }
-    mlist <- getMethods(fdef)
-    default <- finalDefaultMethod(mlist)
-    fMetaName <- mlistMetaName(f)
+
+    methods <- getMethodsForDispatch(f, fdef) # list or table
+    if(is.environment(methods)) {
+##      remove(list=objects(methods, all=TRUE), envir = methods)
+      mlist <- getMethods(fdef) # always a methods list
+    }
+    else
+      mlist <- methods
+    default <- getMethod(fdef, "ANY", optional = TRUE)
+    fMetaName <- mlistMetaName(fdef)
     allWhere <- .findAll(fMetaName, where)
     if(!all)
         allWhere <- allWhere[1]
     value <- rep(TRUE, length(allWhere))
-    for(i in seq(along=allWhere)) {
+    for(i in seq_along(allWhere)) {
         db <- allWhere[[i]]
         obj <- get(fMetaName, db)
         ## remove non-empty methods list objects
@@ -752,12 +904,13 @@ removeMethods <-
     ## cacheGenericsMetaData is called to clear primitive methods if there
     ## are none for this generic on other databases.
     cacheGenericsMetaData(f, fdef, FALSE, where)
-    allWhere <- allWhere[value] # process functions only where methods successfully removed
-    for(i in seq(along=allWhere)) {
+    .uncacheGeneric(f, fdef) # in case it gets removed or re-assigned
+   allWhere <- allWhere[value] # process functions only where methods successfully removed
+    for(i in seq_along(allWhere)) {
         db <- as.environment(allWhere[[i]])
         if(isGeneric(f, db)) { # note use of isGeneric to work for primitives
             if(environmentIsLocked(db)) {
-                warning(gettextf("cannot restore previous version of '%s' in locked environment/package '%s'",
+                warning(gettextf("cannot restore previous version of \"%s\" in locked environment/package \"%s\"",
                         f, getPackageName(db)), domain = NA)
                 value[[i]] <- FALSE
             }
@@ -766,7 +919,7 @@ removeMethods <-
                 default <- as(default, "function") # strict, removes slots
                 rm(list=f, pos = db)
                 if(!existsFunction(f, FALSE, db)) {
-                    message(gettextf("restoring default function definition of \'%s'",
+                    message(gettextf("restoring default function definition of \"%s\"",
                                      f), domain = NA)
                     assign(f, default, db)
                 }
@@ -775,7 +928,7 @@ removeMethods <-
             ## else, leave the generic in place
             else {
                 mlist@methods <- mlist@allMethods <- list()
-                resetGeneric(f, fdef, mlist, db, default)
+                resetGeneric(f, fdef, methods, db, default)
                 assignMethodsMetaData(f, mlist, fdef, db, default)
             }
             break
@@ -785,19 +938,23 @@ removeMethods <-
 }
 
 
-resetGeneric <- function(f, fdef = getGeneric(f, where = where), mlist = getMethods(fdef), where = topenv(parent.frame()), deflt = finalDefaultMethod(mlist)) {
+resetGeneric <- function(f, fdef = getGeneric(f, where = where), mlist = getMethodsForDispatch(f, fdef), where = topenv(parent.frame()), deflt = finalDefaultMethod(mlist)) {
     if(!is(fdef, "genericFunction")) {
         if(missing(mlist)) {
-            warning(gettextf("cannot reset '%s', the definition is not a generic function object", f), domain = NA)
+            warning(gettextf("cannot reset \"%s\", the definition is not a generic function object", f), domain = NA)
             return(NULL)
         }
         else
-            stop(gettextf("error in updating generic function '%s'; the function definition is not a generic function (class \"%s\")", f, class(fdef)),
+            stop(gettextf("error in updating generic function \"%s\"; the function definition is not a generic function (class \"%s\")", f, class(fdef)),
                  domain = NA)
     }
     ## uncache
-    mlist@allMethods <- mlist@methods
-    .genericAssign(f, fdef, mlist, where, deflt)
+    if(is.environment(mlist))
+        .updateMethodsInTable(fdef, where, "reset")
+    else {
+        mlist@allMethods <- mlist@methods
+        .genericAssign(f, fdef, mlist, where, deflt)
+    }
     f
 }
 
@@ -824,7 +981,7 @@ setGroupGeneric <-
     ## By definition, the body must generate an error.
     body(def, envir = environment(def)) <- substitute(
               stop(MSG, domain = NA),
-              list(MSG = gettextf("function '%s' is a group generic; do not call it directly", name)))
+              list(MSG = gettextf("function \"%s\" is a group generic; do not call it directly", name)))
     if(is.character(knownMembers))
         knownMembers <- as.list(knownMembers) # ? or try to find them?
     setGeneric(name, def, group = group, valueClass = valueClass,
@@ -873,7 +1030,7 @@ callGeneric <- function(...)
             call <- match.call(fdef, call)
             anames <- names(call)
             matched <- !is.na(match(anames, names(formals(fdef))))
-            for(i in seq(along = anames))
+            for(i in seq_along(anames))
                 if(matched[[i]])
                     call[[i]] <- as.name(anames[[i]])
         }
