@@ -58,7 +58,7 @@ makePrototypeFromClassDef <-
         pnames <- names(attributes(prototype))
     if(length(slots) == 0 && !is.null(prototype))
             return(prototype)
-    for(i in seq(along=extends)) {
+    for(i in seq_along(extends)) {
         what <- el(supers, i)
         exti <- extends[[i]]
         if(identical(exti@simple, FALSE))
@@ -131,7 +131,7 @@ makePrototypeFromClassDef <-
         snames <- snames[iData]
         slots <- slots[iData]
     }
-    for(j in seq(along = snames)) {
+    for(j in seq_along(snames)) {
         name <- el(snames, j)
         i <- match(name, pnames)
         if(is.na(i)) {
@@ -234,7 +234,7 @@ completeClassDefinition <-
         if(length(ext) > 0) {
             superProps <- vector("list", length(ext)+1)
             superProps[[1]] <- properties
-            for(i in seq(along=ext)) {
+            for(i in seq_along(ext)) {
                 eClass <- ext[[i]]
                 if(isClass(eClass, where = where))
                     superProps[[i+1]] <- getClassDef(eClass, where = where)@slots
@@ -271,7 +271,7 @@ completeClassDefinition <-
         }
         ## ensure that each element of the slots is a valid class reference
         undefClasses <- rep(FALSE, length(properties))
-        for(i in seq(along = properties)) {
+        for(i in seq_along(properties)) {
             cli <- properties[[i]]
             if(is.null(packageSlot(cli))) {
                 cliDef <- getClassDef(cli, where)
@@ -348,7 +348,7 @@ superClassDepth <-
     ext <- ClassDef@contains
     ## remove indirect and maybe non-simple superclasses (latter for inferring slots)
     ok <- rep(TRUE, length(ext))
-    for(i in seq(along=ext)) {
+    for(i in seq_along(ext)) {
         exti <- ext[[i]]
         if(.isIndirectExtension(exti) ||
            (simpleOnly && ! exti @simple))
@@ -359,7 +359,7 @@ superClassDepth <-
     notSoFar <- is.na(match(immediate, soFar))
     immediate <- immediate[notSoFar]
     super <- list(label=immediate, depth = rep(1, length(immediate)), ext = ext)
-    for(i  in seq(along = immediate)) {
+    for(i  in seq_along(immediate)) {
         what <- immediate[[i]]
         if(!is.na(match(what, soFar)))
            ## watch out for loops (e.g., matrix/array have mutual is relationship)
@@ -420,6 +420,7 @@ assignClassDef <-
       if(!.identC(Class, clName))
           stop(gettextf("assigning as \"%s\" a class representation with internal name \"%s\"",
                         Class, def@className), domain = NA)
+      .cacheClass(clName, def)
       assign(classMetaName(Class), def, where)
   }
 
@@ -437,13 +438,13 @@ assignClassDef <-
                   access = list(), className = character(), package = character(),
                   subclasses = list(), versionKey = .newExternalptr(),
                   sealed = FALSE)
-    proto <- list()
+    proto <- defaultPrototype()
     pnames <- names(protoSlots)
-    for(i in seq(along=protoSlots))
+    for(i in seq_along(protoSlots))
         slot(proto, pnames[[i]], FALSE) <- protoSlots[[i]]
     classRepClass <- .classNameFromMethods("classRepresentation")
     class(proto) <- classRepClass
-    object <- list()
+    object <- defaultPrototype()
     class(object) <- classRepClass
     slot(object, "slots", FALSE) <- defSlots
     slot(object, "className", FALSE) <- classRepClass
@@ -498,7 +499,7 @@ newBasic <-
                "matrix" = (if (length(list(...)) > 0) matrix(...) else matrix(0, 0, 0)),
 #               "ts" = ts(...),
 # break dependence on package stats
-               "ts" = (if(length(list(...))) ts(...)
+               "ts" = (if(length(list(...))) stats::ts(...)
                else structure(NA, .Tsp = c(1, 1, 1), class = "ts")),
                   {
                       args <- list(...)
@@ -518,11 +519,22 @@ newBasic <-
       stop(msg, domain = NA)
 }
 
+
+## this non-exported function turns on or off
+## the use of the S4 type as class prototype
+.useS4Prototype <- function(on = TRUE, where  = .methodsNamespace) {
+    if(on)
+     pp <- .Call("Rf_allocS4Object",PACKAGE="methods")
+    else
+     pp <-  list()
+    .assignOverBinding(".defaultPrototype", where=where, pp, FALSE)
+}
+
 defaultPrototype <-
     ## the starting prototype for a non-virtual class
     ## Should someday be a non-vector sexp type
     function()
-    list()
+    .defaultPrototype
 
 reconcilePropertiesAndPrototype <-
   ## makes a list or a structure look like a prototype for the given class.
@@ -613,7 +625,7 @@ reconcilePropertiesAndPrototype <-
       }
       ## check for conflicts in the slots
       allProps <- properties
-      for(i in seq(along=superClasses)) {
+      for(i in seq_along(superClasses)) {
           cl <- superClasses[[i]]
           clDef <- getClassDef(cl, where)
           if(is(clDef, "classRepresentation")) {
@@ -675,10 +687,10 @@ reconcilePropertiesAndPrototype <-
       ## An important detail is that these are
       ## set using slot<- with check=FALSE (because the slot will not be there already)
       ## what <- is.na(match(slots, pnames))
-      what <- seq(along=properties)
+      what <- seq_along(properties)
       props <- properties[what]
       what <- slots[what]
-      for(i in seq(along=what)) {
+      for(i in seq_along(what)) {
           propName <- el(what, i)
           if(!identical(propName, ".Data") &&
              is.null(attr(prototype, propName)))
@@ -755,12 +767,13 @@ showExtends <-
 {
     what <- names(ext)
     how <- character(length(ext))
-    for(i in seq(along=ext)) {
+    for(i in seq_along(ext)) {
         eli <- el(ext, i)
         if(is(eli, "SClassExtension")) {
             how[i] <-
                 if(length(eli@by) > 0)
-                    paste("by class", paste("\"", eli@by, "\"", sep="", collapse = ", "))
+                    paste("by class", paste("\"", eli@by,
+                      "\", distance ",  eli@distance, sep="", collapse = ", "))
                 else if(identical(eli@dataPart, TRUE))
                     "from data part"
                 else "directly"
@@ -846,7 +859,7 @@ possibleExtends <- function(class1, class2, ClassDef1, ClassDef2)
 completeExtends <-    function(ClassDef, class2, extensionDef, where) {
     ## check for indirect extensions => already completed
     ext <- ClassDef@contains
-    for(i in seq(along = ext)) {
+    for(i in seq_along(ext)) {
         if(.isIndirectExtension(ext[[i]])) {
             ClassDef <- .uncompleteClassDefinition(ClassDef, "contains")
             break
@@ -865,7 +878,7 @@ completeExtends <-    function(ClassDef, class2, extensionDef, where) {
             .transitiveSubclasses(ClassDef@className, class2, extensionDef, ClassDef@subclasses)
         ## insert the new is relationship, but without any recursive completion
         ## (asserted not to be needed if the subclass slot is complete)
-        for(i in seq(along = subclasses)) {
+        for(i in seq_along(subclasses)) {
             obji <- subclasses[[i]]
             ## don't override existing relations
             ## TODO:  have a metric that picks the "closest" relationship
@@ -881,7 +894,7 @@ completeSubclasses <-
     function(classDef, class2, extensionDef, where, classDef2 = getClassDef(class2, where)) {
     ## check for indirect extensions => already completed
     ext <- classDef@subclasses
-    for(i in seq(along = ext)) {
+    for(i in seq_along(ext)) {
         if(.isIndirectExtension(ext[[i]])) {
             classDef <- .uncompleteClassDefinition(classDef, "subclasses")
             break
@@ -893,7 +906,7 @@ completeSubclasses <-
             .transitiveExtends(class2, classDef@className, extensionDef, classDef@contains)
         ## insert the new is relationship, but without any recursive completion
         ## (asserted not to be needed if the subclass slot is complete)
-        for(i in seq(along = contains)) {
+        for(i in seq_along(contains)) {
             obji <- contains[[i]]
             cli <- contains[[i]]@superClass
             cliDef <- getClassDef(cli, where)
@@ -917,7 +930,7 @@ completeSubclasses <-
     superClassCase <- identical(slotName, "contains")
     fromTo <- ClassDef@className
     what <- names(ext)
-    for(i in seq(along=ext)) {
+    for(i in seq_along(ext)) {
         by <- what[[i]]
         if(isClass(by, where = where)) {
             byDef <- getClass(by, where = where)
@@ -933,7 +946,7 @@ completeSubclasses <-
             }
         }
         else
-            stop(gettextf("the '%s' list for class \"%s\", includes an undefined class \"%s\"",
+            stop(gettextf("the \"%s\" list for class \"%s\", includes an undefined class \"%s\"",
                           if(superClassCase) "superClass" else "subClass",
                           className,.className(by)),
                  domain = NA)
@@ -943,7 +956,7 @@ completeSubclasses <-
         ok <- is.na(match(what, className))
         ## A class may not contain itself, directly or indirectly
         ## but a non-simple cyclic relation, involving setIs, is allowed
-        for(i in seq(along = what)[!ok]) {
+        for(i in seq_along(what)[!ok]) {
             exti <- ext[[i]]
             simple <- exti@simple
             if(simple) {
@@ -1134,7 +1147,7 @@ setDataPart <- function(object, value) {
                 subclasses <- ClassDef@subclasses
                 what <- names(subclasses)
                 value <- cl
-                for(i in seq(along = what)) {
+                for(i in seq_along(what)) {
                     ext <- subclasses[[i]]
                     ##TODO:  the following heuristic test for an "original"
                     ## subclass should be replaced by a suitable class (extending SClassExtension)
@@ -1179,7 +1192,7 @@ setDataPart <- function(object, value) {
 ## from that class to `by'
 .transitiveExtends <- function(from, by, byExt, moreExts) {
     what <- names(moreExts)
-    for(i in seq(along = moreExts)) {
+    for(i in seq_along(moreExts)) {
         toExt <- moreExts[[i]]
         to <- what[[i]]
         toExt <- .combineExtends(byExt, toExt, by, to)
@@ -1190,7 +1203,7 @@ setDataPart <- function(object, value) {
 
 .transitiveSubclasses <- function(by, to, toExt, moreExts) {
     what <- names(moreExts)
-    for(i in seq(along = moreExts)) {
+    for(i in seq_along(moreExts)) {
         byExt <- moreExts[[i]]
         byExt <- .combineExtends(byExt, toExt, by, to)
         moreExts[[i]] <- byExt
@@ -1251,6 +1264,7 @@ setDataPart <- function(object, value) {
         toExt@replace <- f
         toExt@by <- toExt@subClass
         toExt@subClass <- byExt@subClass
+        toExt@distance <- toExt@distance + byExt@distance
         toExt
 }
 
@@ -1317,7 +1331,7 @@ newClassRepresentation <- function(...) {
     value <- new("classRepresentation")
     slots <- list(...)
     slotNames <- names(slots)
-    for(i in seq(along = slotNames))
+    for(i in seq_along(slotNames))
         slot(value, slotNames[[i]], FALSE) <- slots[[i]]
     value
 }
@@ -1328,7 +1342,7 @@ newClassRepresentation <- function(...) {
     value <- new("classRepresentation")
     slots <- list(...)
     slotNames <- names(slots)
-    for(i in seq(along = slotNames))
+    for(i in seq_along(slotNames))
         slot(value, slotNames[[i]], FALSE) <- slots[[i]]
     value
 }
@@ -1372,7 +1386,7 @@ substituteFunctionArgs <- function(def, newArgs, args = formalArgs(def), silent
                           paste(checkFor[!is.na(match(checkFor, locals))], collapse = ", "),
                           paste(newArgs, collapse = ", ")), domain = NA)
         ll <- vector("list", 2*n)
-        for(i in seq(length = n)) {
+        for(i in seq_len(n)) {
             ll[[i]] <- as.name(args[[i]])
             ll[[n+i]] <- as.name(newArgs[[i]])
         }
@@ -1403,7 +1417,7 @@ substituteFunctionArgs <- function(def, newArgs, args = formalArgs(def), silent
 # the bootstrap version of setting slots in completeClassDefinition
 .mergeClassDefSlots <- function(ClassDef, ...) {
     slots <- list(...); slotNames <- names(slots)
-    for(i in seq(along = slots))
+    for(i in seq_along(slots))
         slot(ClassDef, slotNames[[i]], FALSE) <- slots[[i]]
     ClassDef
 }
@@ -1411,7 +1425,7 @@ substituteFunctionArgs <- function(def, newArgs, args = formalArgs(def), silent
 ## the real version:  differs only in checking the slot values
 ..mergeClassDefSlots <- function(ClassDef, ...) {
     slots <- list(...); slotNames <- names(slots)
-    for(i in seq(along = slots))
+    for(i in seq_along(slots))
         slot(ClassDef, slotNames[[i]]) <- slots[[i]]
     ClassDef
 }
@@ -1460,7 +1474,7 @@ substituteFunctionArgs <- function(def, newArgs, args = formalArgs(def), silent
        .identC(package, get(".packageName", topEnv)))
         return(topEnv) # kludge for source'ing package code
     if(!require(package, character.only = TRUE))
-        stop(gettextf("unable to find required package '%s'", package),
+        stop(gettextf("unable to find required package \"%s\"", package),
              domain = NA)
     value <- .asEnvironmentPackage(package)
     assign(package, value, envir = .PackageEnvironments)
@@ -1481,7 +1495,7 @@ substituteFunctionArgs <- function(def, newArgs, args = formalArgs(def), silent
         ## need to allow for versioned installs: prefer exact match.
         m <- charmatch(pkg, search())
         if(is.na(m))
-            stop(gettextf("Package '%s' is not loaded", package), domain = NA)
+            stop(gettextf("Package \"%s\" is not loaded", package), domain = NA)
         as.environment(search()[m])
     }
 }
@@ -1531,8 +1545,9 @@ substituteFunctionArgs <- function(def, newArgs, args = formalArgs(def), silent
 ## they use getClassDef and .getGeneric resp.  Also, .getEnv returns baseenv() rather
 ## than generating an error if no generic found (so getGeneric can return gen'c for prim'ves)
 
-.genEnv <-  function(f, default = .requirePackage("methods")) {
-        package <- packageSlot(f)
+.genEnv <-  function(f, default = .requirePackage("methods"), package = "") {
+        if(nchar(package) == 0)
+          package <- packageSlot(f)
         if(is.null(package)) {
             ## use the default, but check that the object is there, and if not
             ## try a couple of other heuristics
@@ -1555,3 +1570,95 @@ substituteFunctionArgs <- function(def, newArgs, args = formalArgs(def), silent
         else
             .requirePackage(package)
     }
+
+## cache and retrieve class definitions  If there is a conflict with
+## packages a list of  classes will be cached
+## See .cacheGeneric, etc. for analogous computations for generics
+.classTable <- new.env(TRUE, baseenv())
+
+.cacheClass <- function(name, def) {
+    if(exists(name, envir = .classTable, inherits = FALSE)) {
+        newpkg <- def@package
+        prev <- get(name, envir = .classTable)
+        if(is(prev, "classRepresentation")) {
+            if(identical(prev, def))
+               return()
+            pkg <- prev@package # start a per-package list
+            if(identical(pkg, newpkg)) # redefinition
+              return(assign(name, def, envir = .classTable))
+            prev <- list(prev)
+            names(prev) <- pkg
+        }
+        i <- match(newpkg, names(prev))
+        if(is.na(i))
+           prev[[newpkg]] <- def
+        else if(identical(def, prev[[i]]))
+          return()
+        else
+            prev[[i]] <- def
+        def <- prev
+    }
+    assign(name, def, envir = .classTable)
+}
+
+.uncacheClass <- function(name, def) {
+    if(exists(name, envir = .classTable, inherits = FALSE)) {
+        newpkg <- def@package
+        prev <- get(name, envir = .classTable)
+        if(is(prev, "classRepresentation"))  # we might worry if  prev not identical?
+            return(remove(list = name, envir = .classTable))
+         i <- match(newpkg, names(prev))
+        if(!is.na(i))
+           prev[[i]] <- NULL
+        else # we might warn about unchaching more than once
+          return()
+        if(length(prev) == 0)
+          return(remove(list = name, envir = .classTable))
+        else if(length(prev) == 1)
+          prev <- prev[[1]]
+        assign(name, prev, envir  = .ClassTable)
+    }
+}
+
+.getClassFromCache <- function(name, where) {
+    if(exists(name, envir = .classTable, inherits = FALSE)) {
+        value <- get(name, envir = .classTable)
+        if(is.list(value)) { # multiple classes with this name
+            pkg <- packageSlot(name)
+            if(is.null(pkg) && is.character(where))
+              pkg <- where
+            else
+              pkg <- getPackageName(where)
+            pkgs <- names(value)
+            i <- match(pkg, pkgs,0)
+            if(i > 0)
+              return(value[[i]])
+            i <- match("methods", pkgs,0)
+            if(i > 0)
+               return(value[[i]])
+            else
+              return(NULL)
+        }
+        value
+    }
+    else
+      NULL
+}
+
+.scanUnionClass <- function(class, def) {
+    subs <- def@subclasses
+    subNames <- names(subs)
+    for(i in seq(along = subs)) {
+        what <- subNames[[i]]
+        subDef <- getClassDef(what)
+        if(is.null(subDef))
+          warning(
+           gettextf("Undefined member, \"%s\", of union \"%s\" (not added during attach)",
+                    what, def@className))
+        else {
+            subDef@contains[[class]] <- subs[[i]]
+            .cacheClass(what, subDef)
+        }
+    }
+}
+
