@@ -6,23 +6,23 @@ setClass <-
 {
     oldDef <- getClassDef(Class, where)
     if(is(oldDef, "classRepresentation") && oldDef@sealed)
-        stop(paste("\"", Class, "\" has a sealed class definition and cannot be redefined", sep=""))
+        stop(gettextf("\"%s\" has a sealed class definition and cannot be redefined", Class), domain = NA)
     if(is(representation, "classRepresentation")) {
         ## supplied a class definition object
         classDef <- representation
         if(!(missing(prototype) && missing(contains) && missing(validity) && missing(access)
              && missing(version) && missing(package)))
-            stop("Only arguments Class and where can be supplied when argument \"representation\" is a classRepresentation object")
+            stop("only arguments 'Class' and 'where' can be supplied when argument 'representation' is a 'classRepresentation' object")
         if(length(classDef@package)==0)
             classDef@package <- package # the default
         superClasses <- allNames(classDef@contains)
     }
-    else { 
+    else {
         ## catch the special case of a single class name as the representation
         if(is.character(representation) && length(representation) == 1 &&
            is.null(names(representation)))
             representation <- list(representation)
-        slots <- nchar(allNames(representation))>0
+        slots <- nchar(allNames(representation)) > 0
         superClasses <- c(as.character(representation[!slots]), contains)
         properties <- representation[slots]
         classDef <- makeClassRepresentation(Class, properties,superClasses, prototype, package,
@@ -31,6 +31,8 @@ setClass <-
     }
     classDef <- completeClassDefinition(Class, classDef, where, doExtends = FALSE)
     oldDef <- getClassDef(Class, where)
+    if(is(oldDef, "classRepresentation"))
+      .uncacheClass(Class, oldDef)
     if(length(superClasses) == 0)
         assignClassDef(Class, classDef, where)
     else {
@@ -41,23 +43,20 @@ setClass <-
         for(class2 in superClasses)
             if(is(try(setIs(Class, class2, classDef = classDef, where = where)), "try-error"))
                 badContains <- c(badContains, class2)
-        if(length(badContains)>0) {
+        if(length(badContains) > 0) {
             msg <- paste(dQuote(badContains), collapse = ", ")
             if(is(try(removeClass(Class, where)), "try-error"))
-                stop("Error in contained classes (", msg, ") for class \"", Class,
-                     "\" and unable to remove definition from \"",
-                     , getPackageName(where), "\"")
+                stop(gettextf("error in contained classes (%s) for class \"%s\" and unable to remove definition from \"%s\"",
+                              msg, Class, getPackageName(where)), domain = NA)
             if(is.null(oldDef))
-                stop("Error in contained classes (", msg, ") for class \"", Class,
-                     "\"; class definition removed from \"", getPackageName(where), "\"")
+                stop(gettextf("error in contained classes (%s) for class \"%s\"; class definition removed from \"%s\"",
+                              msg, Class, getPackageName(where)), domain = NA)
             else if(is(try(setClass(Class, oldDef, where=where)), "try-error"))
-                stop("Error in contained classes (", msg, ") for class \"", Class,
-                     "\" and unable to restore previous definition from \"",
-                     , getPackageName(where), "\"")
+                stop(gettextf("error in contained classes (%s) for class \"%s\" and unable to restore previous definition from \"%s\"",
+                              msg, Class, getPackageName(where)), domain = NA)
             else
-                stop("Error in contained classes (", msg, ") for class \"", Class,
-                     "\"; previous definition restored to \"",
-                     getPackageName(where), "\"")
+                stop(gettextf("error in contained classes (%s) for class \"%s\"; previous definition restored to \"%s\"",
+                              msg, Class, getPackageName(where)), domain = NA)
         }
         if(sealed) {
             classDef@sealed <- TRUE
@@ -77,17 +76,22 @@ representation <-
     ## unlike the S-Plus function, this does not form the class representation,
     ## since set SClass works separately with the slots and extends arguments.
     anames <- allNames(value)
-    for(i in seq(along=value)) {
+    for(i in seq_along(value)) {
         ei <- el(value, i)
         if(!is.character(ei) || length(ei) != 1)
-            stop(paste("Element", i, "of the representation was not a single character string"))
+            stop(gettextf("element %d of the representation was not a single character string", i), domain = NA)
     }
     includes <- as.character(value[nchar(anames)==0])
     if(any(duplicated(includes)))
-        stop(paste("Duplicate class names among superclasses:", paste(includes[duplicated(includes)], collapse = ", ")))
+        stop(gettextf("duplicate class names among superclasses: %s",
+                      paste(dQuote(includes[duplicated(includes)]),
+                            collapse = ", ")),
+             domain = NA)
     slots <- anames[nchar(anames)>0]
     if(any(duplicated(slots)))
-       stop(paste("Duplicated slot names: ", paste(slots[duplicated(slots)], collapse=""), sep=""))
+       stop(gettextf("duplicated slot names: %s",
+                     paste(sQuote(slots[duplicated(slots)]), collapse="")),
+            domain = NA)
     value
 }
 
@@ -105,13 +109,13 @@ prototype <- function(...)
     if(dataPart) {
         if(sum(data) > 1)
             stop("only one data object (unnamed argument to prototype) allowed")
-        obj <- unclass(props[[seq(along=data)[data] ]])
+        obj <- unclass(props[[seq_along(data)[data] ]])
         props <- props[!data]
         names <- names[!data]
     }
     else
-        obj <- list()
-    for(i in seq(along = names))
+        obj <- defaultPrototype()
+    for(i in seq_along(names))
         slot(obj, names[[i]], FALSE) <- props[[i]]
     new("classPrototypeDef", object = obj, slots = names, dataPart = dataPart)
 }
@@ -170,6 +174,8 @@ getClassDef <-
     ## generates inf. loop in booting methods package (also for new())
     if(.identC(class(Class), "classRepresentation"))
         return(Class)
+    value <- .getClassFromCache(Class, where)
+    if(is.null(value)) {
     if(length(Class)>1)
         ## S3 class; almost certainly has no packageSLot, but we'll continue anyway
         cname <- classMetaName(Class[[1]])
@@ -179,27 +185,30 @@ getClassDef <-
     ## a string with a package slot strongly implies the class definition
     ## should be in that package.
     if(!is.null(package)) {
-        whereP <- .requirePackage(package) 
+        whereP <- .requirePackage(package)
         if(exists(cname, whereP))
             value <- get(cname, whereP)
     }
     if(is.null(value) && exists(cname, where))
         value <- get(cname, where)
+    }
     value
 }
 
 getClass <-
   ## Get the complete definition of the class supplied as a string,
   ## including all slots, etc. in classes that this class extends.
-  function(Class, .Force = FALSE, where = .classEnv(Class, topenv(parent.frame()), FALSE))
+  function(Class, .Force = FALSE,
+	   where = .classEnv(Class, topenv(parent.frame()), FALSE))
 {
     value <- getClassDef(Class, where)
     if(is.null(value)) {
-            if(!.Force)
-                stop(paste("\"", Class, "\" is not a defined class", sep=""))
-            else
-                value <- makeClassRepresentation(Class, package = "base", virtual = TRUE, where = where)
-      }
+	if(!.Force)
+	    stop(gettextf("\"%s\" is not a defined class", Class), domain = NA)
+	else
+	    value <- makeClassRepresentation(Class, package = "base",
+					     virtual = TRUE, where = where)
+    }
     value
 }
 
@@ -226,7 +235,8 @@ checkSlotAssignment <- function(obj, name, value)
     ClassDef <- getClass(cl) # fails if cl not a defined class (!)
     slotClass <- elNamed(ClassDef@slots, name)
     if(is.null(slotClass))
-        stop(paste("\"", name, "\" is not a slot in class \"", class(obj), "\"", sep = ""))
+        stop(gettextf("\"%s\" is not a slot in class \"%s\"", name, class(obj)),
+             domain = NA)
     valueClass <- class(value)
     if(.identC(slotClass, valueClass))
        return(value)
@@ -234,18 +244,16 @@ checkSlotAssignment <- function(obj, name, value)
     ## the class environment of obj (change validObject too if a better way is found)
     ok <- possibleExtends(valueClass, slotClass, ClassDef2 = getClassDef(slotClass, where = .classEnv(ClassDef)))
     if(identical(ok, FALSE))
-       stop(paste("Assignment of an object of class \"",
-                  class(value), "\" is not valid for slot \"", name,
-                  "\" in an object of class \"",
-                  class(obj), "\"; is(value, \"", slotClass,
-                   "\") is not TRUE", sep=""))
+       stop(gettextf("assignment of an object of class \"%s\" is not valid for slot \"%s\" in an object of class \"%s\"; is(value, \"%s\") is not TRUE",
+                     class(value),  name, class(obj), slotClass),
+            domain = NA)
     else if(identical(ok, TRUE))
         value
     else
        as(value, slotClass, strict=FALSE, ext = ok)
 }
 
-        
+
 
 ## "@" <-
 ##   function(object, name)
@@ -259,41 +267,55 @@ checkSlotAssignment <- function(obj, name, value)
      "slot<-"(object, name, TRUE, value)
    }
 
-slotNames <-
-  ##  The names of the class's slots.  The argument is either the name of a class, or
-  ## an object from the relevant class.
-  function(x)
+##  The names of the class's slots.  The argument is either the name
+##  of a class, or an object from the relevant class.
+
+## NOTA BENE:  .slotNames() shouldn't be needed,
+##             rather slotNames() should be changed (to work like .slotNames())!
+slotNames <- function(x)
+    if(is(x, "classRepresentation")) names(x@slots) else .slotNames(x)
+
+.slotNames <- function(x)
 {
-    if(is(x, "classRepresentation"))
-        names(x@slots)
-    else {
-        if(is.character(x) && length(x) == 1)
-            classDef <- getClassDef(x)
-        else
-            classDef <- getClassDef(class(x))
-        if(is.null(classDef))
-            character()
-        else
-            names(classDef@slots)
-    }
+    classDef <-
+	getClassDef(if(is.character(x) && length(x) == 1) x else class(x))
+    if(is.null(classDef))
+	character()
+    else
+	names(classDef@slots)
 }
 
-removeClass <-  function(Class, where) {
+
+removeClass <-  function(Class, where = topenv(parent.frame())) {
     if(missing(where)) {
-        where <- findClass(Class, topenv(parent.frame()))
-        if(length(where) == 0) {
-            warning("\"", Class, "\" is not a class (no action taken)")
+       classEnv <- .classEnv(Class, where, FALSE)
+        classWhere <- findClass(Class, where = classEnv)
+        if(length(classWhere) == 0) {
+            warning(gettextf("Class definition for \"%s\" not found  (no action taken)", Class),
+                    domain = NA)
             return(FALSE)
         }
-        if(length(where) > 1)
-            warning("\"", Class, "\" has multiple definitions visible; only the first removed")
-        where <- where[[1]]
+        if(length(classWhere) > 1)
+            warning(gettextf("class \"%s\" has multiple definitions visible; only the first removed", Class), domain = NA)
+        classWhere <- classWhere[[1]]
     }
+    else classWhere <- where
+    classDef <- getClassDef(Class, where=classWhere)
+    if(length(classDef@subclasses)>0) {
+      subclasses <- names(classDef@subclasses)
+      found <- sapply(subclasses, isClass, where = where)
+      if(any(found))
+        warning("The class to be removed  (\"", Class,
+    "\") has defined subclasses that should also be removed: (",
+                paste(as.character(subclasses[found]), collapse = ", "), ")")
+    }
+    .uncacheClass(Class, classDef)
+    .undefineMethod("initialize", Class, classWhere)
     what <- classMetaName(Class)
-    rm(list=what, pos=where)
+    rm(list=what, pos=classWhere)
     TRUE
 }
-    
+
 
 isClass <-
   ## Is this a formally defined class?
@@ -327,12 +349,7 @@ new <-
         ClassDef <- Class
     else
         ClassDef <- getClass(Class, where = topenv(parent.frame()))
-    if(identical(ClassDef@virtual, TRUE)) {
-        stop("Trying to use new() on a virtual class")
-    }
-    else
-        value <- ClassDef@prototype
-    class(value) <- ClassDef@className
+    value <- .Call("R_do_new_object", ClassDef, PACKAGE = "base")
     initialize(value, ...)
 }
 
@@ -359,56 +376,82 @@ getClasses <-
 }
 
 
-validObject <- function(object, test = FALSE) {
+validObject <- function(object, test = FALSE, complete = FALSE)
+{
     Class <- class(object)
-  classDef <- getClassDef(Class)
+    classDef <- getClassDef(Class)
     where <- .classEnv(classDef)
-  anyStrings <- function(x) if(identical(x, TRUE)) character() else x
-  ## perform, from bottom up, the default and any explicit validity tests
-  ## First, validate the slots.
-  errors <- character()
-  slotTypes <- classDef@slots
-  slotNames <- names(slotTypes)
-  for(i in seq(along=slotTypes)) {
-    classi <- slotTypes[[i]]
-    sloti <- slot(object, slotNames[[i]])
-    ## note that the use of possibleExtends is shared with checkSlotAssignment(), in case a
-    ## future revision improves on it!
-    ok <- possibleExtends(class(sloti), classi, ClassDef2 = getClassDef(classi, where = where))
-    if(identical(ok, FALSE))
-      errors <- c(errors, paste("Invalid object for slot \"", slotNames[[i]],
-                               "\" in class \"", Class,
-                               "\": got class \"", class(sloti), 
-                               "\", should be or extend class \"", classi, "\"", sep = ""))
-  }
-  extends <- rev(classDef@contains); i <- 1
-  while(length(errors) == 0 && i <= length(extends)) {
-      exti <- extends[[i]]
-    superClass <- exti@superClass
-    i <- i+1
-    if(!exti@simple && !is(object, superClass))
-        next ## skip conditional relations that don't hold for this object
-    superDef <- getClassDef(superClass, where = where)
-      if(is.null(superDef)) {
-          errors <- c(errors, paste("Super class \"", superClass, "\" not defined in the environment of the object's class", sep=""))
+    anyStrings <- function(x) if(identical(x, TRUE)) character() else x
+    ## perform, from bottom up, the default and any explicit validity tests
+    ## First, validate the slots.
+    errors <- character()
+    slotTypes <- classDef@slots
+    slotNames <- names(slotTypes)
+    for(i in seq_along(slotTypes)) {
+        classi <- slotTypes[[i]]
+        sloti <- slot(object, slotNames[[i]])
+        classDefi <- getClassDef(classi, where = where)
+        if(is.null(classDefi)) {
+          errors <- c(errors, paste("class for slot \"", slotNames[[i]],
+                                    "\" (\"", classi, "\")", sep=""))
           next
-      }
-      validityMethod <- superDef@validity
-      if(is(validityMethod, "function"))
+        }
+        ## note that the use of possibleExtends is shared with checkSlotAssignment(), in case a
+        ## future revision improves on it!
+        ok <- possibleExtends(class(sloti), classi, ClassDef2 = classDefi)
+        if(identical(ok, FALSE)) {
+            errors <- c(errors,
+                        paste("invalid object for slot \"", slotNames[[i]],
+                              "\" in class \"", Class,
+                              "\": got class \"", class(sloti),
+                              "\", should be or extend class \"", classi, "\"", sep = ""))
+            next
+        }
+        if(!complete)
+          next
+        validityMethod <- classDefi@validity
+        if(is(validityMethod, "function")) {
+            errori <- anyStrings(validityMethod(sloti))
+            if(length(errori)>0) {
+                errori <- paste("In slot \"", slotNames[[i]],
+                                "\" of class \"", class(sloti), "\": ",
+                                sep = "", errori)
+                errors <- c(errors, errori)
+            }
+        }
+    }
+    extends <- rev(classDef@contains); i <- 1
+    while(length(errors) == 0 && i <= length(extends)) {
+        exti <- extends[[i]]
+        superClass <- exti@superClass
+        i <- i+1
+        if(!exti@simple && !is(object, superClass))
+            next ## skip conditional relations that don't hold for this object
+        superDef <- getClassDef(superClass, where = where)
+        if(is.null(superDef)) {
+            errors <- c(errors, paste("superclass \"", superClass, "\" not defined in the environment of the object's class", sep=""))
+            next
+        }
+        validityMethod <- superDef@validity
+        if(is(validityMethod, "function"))
             errors <- c(errors, anyStrings(validityMethod(as(object, superClass))))
-  }
-  validityMethod <- classDef@validity
-  if(length(errors) == 0 && is(validityMethod, "function")) {
-    errors <- c(errors, anyStrings(validityMethod(object)))
-  }
-  if(length(errors) > 0) {
-    if(test)
-      errors
-    else if(length(errors) > 1)
-      stop(paste("Invalid \"", Class, "\" object: ", paste(paste(1:length(errors), errors, sep=": ")),
-                 sep="", collapse = "\n"))
-    else stop(paste("Invalid \"", Class, "\" object: ", errors, sep=""))
-  }
+    }
+    validityMethod <- classDef@validity
+    if(length(errors) == 0 && is(validityMethod, "function")) {
+        errors <- c(errors, anyStrings(validityMethod(object)))
+    }
+    if(length(errors) > 0) {
+        if(test)
+            errors
+        else {
+            msg <- gettextf("invalid class \"%s\" object:", Class)
+            if(length(errors) > 1)
+                stop(paste(msg,
+                           paste(paste(1:length(errors), errors, sep=": ")),
+                           collapse = "\n"), domain = NA)
+            else stop(msg, " ", errors, domain = NA)
+    }
+}
   else
     TRUE
 }
@@ -450,18 +493,16 @@ resetClass <- function(Class, classDef, where) {
             if(missing(classDef)) {
                 classDef <- getClassDef(Class, where)
                 if(is.null(classDef)) {
-                    warning("Class ", Class,"\" not found on ",
-                            getPackageName(where), "; resetClass will have no effect")
+                    warning(gettextf("class \"%s\" not found on \"%s\"; 'resetClass' will have no effect", Class, getPackageName(where)), domain = NA)
                     return(classDef)
                 }
             }
             else if(!is(classDef, "classRepresentation"))
-                stop("Argument classDef must be a string or a class representation; got an object of class \"",
-                     class(classDef), "\"")
+                stop(gettextf("argument 'classDef' must be a string or a class representation; got an object of class \"%s\"", class(classDef)), domain = NA)
             package <- getPackageName(where)
         }
         if(classDef@sealed)
-            warning("Class \"", Class,"\" is sealed; resetClass will have no effect")
+            warning(gettextf("class \"%s\" is sealed; 'resetClass' will have no effect", Class), domain = NA)
         else {
             classDef <-  .uncompleteClassDefinition(classDef)
             classDef <- completeClassDefinition(Class, classDef, where)
@@ -491,7 +532,7 @@ initialize <- function(.Object, ...) {
         dataPart <- elNamed(slotDefs, ".Data")
         if(is.null(dataPart)) dataPart <- "missing"
         if(length(supers) > 0) {
-            for(i in rev(seq(along = supers))) {
+            for(i in rev(seq_along(supers))) {
                 obj <- el(supers, i)
                 Classi <- class(obj)
                 ## test some cases that let information be copied into the
@@ -512,27 +553,29 @@ initialize <- function(.Object, ...) {
                     ## look for the common extensions, choose the first
                     ## one in the extensions of Class
                     which <- match(thisExtends, extendsi)
-                    which <- seq(along=which)[!is.na(which)]
+                    which <- seq_along(which)[!is.na(which)]
                     if(length(which) >0 ) {
                         Classi <- thisExtends[which[1]]
                         as(.Object, Classi) <- as(obj, Classi, strict = FALSE)
                     }
                     else
-                        stop(paste("Can't use object of class \"", Classi,
-                                   "\" in new():  Class \"", Class, "\" does not extend that class", sep=""))
+                        stop(gettextf("cannot use object of class \"%s\" in new():  class \"%s\" does not extend that class", Classi, Class), domain = NA)
                 }
             }
         }
         if(length(elements)>0) {
             snames <- names(elements)
             if(any(duplicated(snames)))
-                stop(paste("Duplicated slot names:",
-                           paste(snames[duplicated(snames)], collapse = ", ")))
+                stop(gettextf("duplicated slot names: %s",
+                              paste(sQuote(snames[duplicated(snames)]),
+                                    collapse = ", ")), domain = NA)
             which  <- match(snames, names(slotDefs))
             if(any(is.na(which)))
-                stop(paste("Invalid names for slots of class ",
-                           Class, ": ", paste(snames[is.na(which)], collapse=", "), sep=""))
-            for(i in seq(along=snames)) {
+                stop(gettextf("invalid names for slots of class \"%s\": %s",
+                              Class,
+                              paste(snames[is.na(which)], collapse=", ")),
+                     domain = NA)
+            for(i in seq_along(snames)) {
                 slotName <- el(snames, i)
                 slotClass <- elNamed(slotDefs, slotName)
                 slotClassDef <- getClassDef(slotClass, package=ClassDef@package)
@@ -563,7 +606,7 @@ findClass <- function(Class, where = topenv(parent.frame()), unique = "") {
     }
     else
         pkg <- ""
-    if(missing(where) && nchar(pkg)) 
+    if(missing(where) && nchar(pkg))
             where <- .requirePackage(pkg)
     else
         where <- as.environment(where)
@@ -571,12 +614,14 @@ findClass <- function(Class, where = topenv(parent.frame()), unique = "") {
     where <- .findAll(what, where)
     if(length(where) != 1 && nchar(unique)>0) {
             if(length(where) == 0)
-                stop("No definition of \"", Class, "\" to use for ", unique)
+                stop(gettextf("no definition of \"%s\" to use for %s",
+                              Class, unique), domain = NA)
             if(length(where) > 1) {
-                where <- where[[1]]
-                warning("Multiple definitions of class \"", Class,
-                        "\" visible; using the definition on package \"",
-                        getPackageName(where), "\" for ", unique)
+                where <- where[1]
+                ## problem: 'unique'x is text passed in, so do not translate
+                warning(sprintf("multiple definitions of class \"%s\" visible; using the definition on package \"%s\" for %s",
+                                 Class, getPackageName(where[[1]]), unique),
+                        domain = NA)
             }
     }
     where
