@@ -140,7 +140,7 @@ plot.acf <-
             str(par("mfrow","cex", "cex.main","cex.axis","cex.lab","cex.sub"))
         }
     }
-    
+
     if (is.null(ylim)) {
         ## Calculate a common scale
         ylim <- range(x$acf[, 1:nser, 1:nser], na.rm = TRUE)
@@ -198,8 +198,8 @@ ccf <- function(x, y, lag.max = NULL,
     type <- match.arg(type)
     if(is.matrix(x) || is.matrix(y))
         stop("univariate time series only")
-    X <- na.action(ts.union(as.ts(x), as.ts(y)))
-    colnames(X) <- c(deparse(substitute(x)), deparse(substitute(y)))
+    X <- na.action(ts.intersect(as.ts(x), as.ts(y)))
+    colnames(X) <- c(deparse(substitute(x))[1], deparse(substitute(y))[1])
     acf.out <- acf(X, lag.max = lag.max, plot = FALSE, type = type)
     lag <- c(rev(acf.out$lag[-1,2,1]), acf.out$lag[,1,2])
     y   <- c(rev(acf.out$acf[-1,2,1]), acf.out$acf[,1,2])
@@ -212,12 +212,13 @@ ccf <- function(x, y, lag.max = NULL,
     } else return(acf.out)
 }
 
-"[.acf" <- function(x, i, j=1:nser)
+"[.acf" <- function(x, i, j)
 {
-    nser <- ncol(x$lag)
-    ii <- match(i, x$lag[,1,1], nomatch=NA)
-    x$acf <- x$acf[ii,j,j, drop=FALSE]
-    x$lag <- x$lag[ii,j,j, drop=FALSE]
+    if(missing(j)) j <- seq_len(ncol(x$lag))
+    ii <- if(missing(i)) seq_len(nrow(x$lag))
+    else match(i, x$lag[, 1, 1], nomatch = NA)
+    x$acf <- x$acf[ii, j, j, drop = FALSE]
+    x$lag <- x$lag[ii, j, j, drop = FALSE]
     x
 }
 
