@@ -6,13 +6,22 @@ print.family <- function(x, ...)
     cat("Link function:", x$link, "\n\n")
 }
 
-power <- function(lambda = 1) {
-    if(lambda <= 0)
-	make.link("log")
-    else if(lambda == 1)
-        make.link("identity")
-    else
-        make.link(lambda)
+power <- function(lambda = 1)
+{
+    if(!is.numeric(lambda) || is.na(lambda))
+        stop("invalid argument 'lambda'")
+    if(lambda <= 0) return(make.link("log"))
+    if(lambda == 1) return(make.link("identity"))
+    linkfun <- function(mu) mu^lambda
+    linkinv <- function(eta)
+        pmax(eta^(1/lambda), .Machine$double.eps)
+    mu.eta <- function(eta)
+        pmax((1/lambda) * eta^(1/lambda - 1), .Machine$double.eps)
+    valideta <- function(eta) all(eta>0)
+    link <- paste("mu^", round(lambda, 3), sep="")
+    structure(list(linkfun = linkfun, linkinv = linkinv,
+                   mu.eta = mu.eta, valideta = valideta, name = link),
+              class="link-glm")
 }
 
 ## Written by Simon Davies Dec 1995
@@ -20,34 +29,23 @@ power <- function(lambda = 1) {
 ## added valideta(eta) function..
 make.link <- function (link)
 {
-    if (is.character(link) && length(grep("^power", link) > 0))
+    ## first two deprecated in 2.4.0
+    if (is.character(link) && length(grep("^power", link) > 0)) {
+        ## this seems intended to catch calls like "power(3)"
+        warning('calling make.link("power(z)") is deprecated', domain = NA)
         return(eval(parse(text = link)))
-    else if(!is.character(link) && !is.na(lambda <- as.numeric(link))) {
-        linkfun <- function(mu) mu^lambda
-        linkinv <- function(eta)
-            pmax(eta^(1/lambda), .Machine$double.eps)
-        mu.eta <- function(eta)
-            pmax((1/lambda) * eta^(1/lambda - 1), .Machine$double.eps)
-        valideta <- function(eta) all(eta>0)
-    }
-    else
+    } else if(!is.character(link) && !is.na(lambda <- as.numeric(link))) {
+        warning('calling make.link(number) is deprecated', domain = NA)
+        return(power(lambda))
+    } else
         switch(link,
                "logit" = {
-                   linkfun <- function(mu) .Call("logit_link", mu, PACKAGE="stats")
-                   linkinv <- function(eta) .Call("logit_linkinv", eta, PACKAGE="stats")
-##               {
-##                   thresh <- -log(.Machine$double.eps)
-##                   eta <- pmin(pmax(eta, -thresh), thresh)
-##                   exp(eta)/(1 + exp(eta))
-##               }
-                   mu.eta <- function(eta) .Call("logit_mu_eta", eta, PACKAGE="stats")
-##                   {
-##                       thresh <- -log(.Machine$double.eps)
-##                       res <- rep.int(.Machine$double.eps, length(eta))
-##                       res[abs(eta) < thresh] <-
-##                           (exp(eta)/(1 + exp(eta))^2)[abs(eta) < thresh]
-##                       res
-##                   }
+                   linkfun <- function(mu)
+                       .Call("logit_link", mu, PACKAGE="stats")
+                   linkinv <- function(eta)
+                       .Call("logit_linkinv", eta, PACKAGE="stats")
+                   mu.eta <- function(eta)
+                       .Call("logit_mu_eta", eta, PACKAGE="stats")
                    valideta <- function(eta) TRUE
                },
                "probit" = {
@@ -118,24 +116,45 @@ make.link <- function (link)
                ## else :
                stop(sQuote(link), " link not recognised")
                )# end switch(.)
-    list(linkfun = linkfun, linkinv = linkinv,
-	 mu.eta = mu.eta, valideta = valideta)
+    structure(list(linkfun = linkfun, linkinv = linkinv,
+                   mu.eta = mu.eta, valideta = valideta, name = link),
+              class="link-glm")
 }
 
 poisson <- function (link = "log")
 {
     linktemp <- substitute(link)
-    ## this is a function used in  glm().
-    ## It holds everything personal to the family,
-    ## converts link into character string
+    ## idea is that we can specify a character string or a name for
+    ## the standard links, and if the name is not one of those, look
+    ## for an object of the appropiate class.
     if (!is.character(linktemp)) {
 	linktemp <- deparse(linktemp)
-	if (linktemp == "link")
+        ## the idea here seems to be that we can have a character variable
+        ## 'link' naming the link (undocumented).  Deprecate in 2.4.0.
+	if (linktemp == "link") {
+            warning("use of poisson(link=link) is deprecated\n", domain = NA)
 	    linktemp <- eval(link)
+            if(!is.character(linktemp) || length(linktemp) != 1)
+                stop("'link' is invalid", domain=NA)
+        }
     }
-    if (any(linktemp == c("log", "identity", "sqrt")))
+    okLinks <- c("log", "identity", "sqrt")
+    if (linktemp %in% okLinks)
 	stats <- make.link(linktemp)
-    else stop(gettextf('link "%s" not available for Poisson family; available links are "identity", "log" and "sqrt"', linktemp), domain = NA)
+    else if (is.character(link)) {
+        stats <- make.link(link)
+        linktemp <- link
+    } else {
+        ## what else shall we allow?  At least objects of class link-glm.
+        if(inherits(link, "link-glm")) {
+            stats <- link
+            if(!is.null(stats$name)) linktemp <- stats$name
+        } else {
+            stop(gettextf('link "%s" not available for poisson family; available links are %s',
+			  linktemp, paste(sQuote(okLinks), collapse =", ")),
+		 domain = NA)
+        }
+    }
     variance <- function(mu) mu
     validmu <- function(mu) all(mu>0)
     dev.resids <- function(y, mu, wt)
@@ -166,17 +185,36 @@ poisson <- function (link = "log")
 quasipoisson <- function (link = "log")
 {
     linktemp <- substitute(link)
-    ## this is a function used in  glm().
-    ## It holds everything personal to the family,
-    ## converts link into character string
     if (!is.character(linktemp)) {
 	linktemp <- deparse(linktemp)
-	if (linktemp == "link")
+        ## the idea here seems to be that we can have a character variable
+        ## 'link' naming the link.
+	if (linktemp == "link") {
+            warning("use of quasipoisson(link=link) is deprecated\n",
+                    domain = NA)
 	    linktemp <- eval(link)
+            if(!is.character(linktemp) || length(linktemp) != 1)
+                stop("'link' is invalid", domain=NA)
+        }
+    }
+
+    okLinks <- c("log", "identity", "sqrt")
+    if (linktemp %in% okLinks)
+        stats <- make.link(linktemp)
+    else if (is.character(link)) {
+        stats <- make.link(link)
+        linktemp <- link
+    } else {
+        ## what else shall we allow?  At least objects of class link-glm.
+        if(inherits(link, "link-glm")) {
+            stats <- link
+            if(!is.null(stats$name)) linktemp <- stats$name
+        } else {
+	    stop(gettextf('link "%s" not available for quasipoisson family; available links are %s',
+			  linktemp, paste(sQuote(okLinks), collapse =", ")),
+		 domain = NA)
+        }
     }
-    if (any(linktemp == c("log", "identity", "sqrt")))
-	stats <- make.link(linktemp)
-    else stop(gettextf('link "%s" not available for quasipoisson family; available links are "identity", "log" and "sqrt"', linktemp), domain = NA)
     variance <- function(mu) mu
     validmu <- function(mu) all(mu>0)
     dev.resids <- function(y, mu, wt)
@@ -205,17 +243,34 @@ quasipoisson <- function (link = "log")
 gaussian <- function (link = "identity")
 {
     linktemp <- substitute(link)
-    ## This is a function used in  glm();
-    ## it holds everything personal to the family
-    ## converts link into character string
     if (!is.character(linktemp)) {
 	linktemp <- deparse(linktemp)
-	if (linktemp == "link")
+        ## the idea here seems to be that we can have a character variable
+        ## 'link' naming the link.
+	if (linktemp == "link") {
+            warning("use of gaussian(link=link) is deprecated\n", domain = NA)
 	    linktemp <- eval(link)
+            if(!is.character(linktemp) || length(linktemp) != 1)
+                stop("'link' is invalid", domain=NA)
+        }
     }
-    if (any(linktemp == c("inverse", "log", "identity")))
+    okLinks <- c("inverse", "log", "identity")
+    if (linktemp %in% okLinks)
 	stats <- make.link(linktemp)
-    else stop(gettextf('link "%s" not available for gaussian family, available links are "inverse", "log" and "identity"', linktemp), domain = NA)
+    else if (is.character(link)) {
+        stats <- make.link(link)
+        linktemp <- link
+    } else {
+        ## what else shall we allow?  At least objects of class link-glm.
+        if(inherits(link, "link-glm")) {
+            stats <- link
+            if(!is.null(stats$name)) linktemp <- stats$name
+        } else {
+	    stop(gettextf('link "%s" not available for gaussian family; available links are %s',
+			  linktemp, paste(sQuote(okLinks), collapse =", ")),
+		 domain = NA)
+        }
+    }
     structure(list(family = "gaussian",
 		   link = linktemp,
 		   linkfun = stats$linkfun,
@@ -245,18 +300,34 @@ gaussian <- function (link = "identity")
 binomial <- function (link = "logit")
 {
     linktemp <- substitute(link)
-    ## this is a function used in  glm();
-    ## it holds everything personal to the family
-    ## converts link into character string
     if (!is.character(linktemp)) {
 	linktemp <- deparse(linktemp)
-	if (linktemp == "link")
+        ## the idea here seems to be that we can have a character variable
+        ## 'link' naming the link.
+	if (linktemp == "link") {
+            warning("use of binomial(link=link) is deprecated\n", domain = NA)
 	    linktemp <- eval(link)
+            if(!is.character(linktemp) || length(linktemp) != 1)
+                stop("'link' is invalid", domain=NA)
+        }
+    }
+    okLinks <- c("logit", "probit", "cloglog", "cauchit", "log")
+    if (linktemp %in% okLinks)
+        stats <- make.link(linktemp)
+    else if (is.character(link)) {
+        stats <- make.link(link)
+        linktemp <- link
+    } else {
+        ## what else shall we allow?  At least objects of class link-glm.
+        if(inherits(link, "link-glm")) {
+            stats <- link
+            if(!is.null(stats$name)) linktemp <- stats$name
+        } else {
+	    stop(gettextf('link "%s" not available for binomial family; available links are %s',
+			  linktemp, paste(sQuote(okLinks), collapse =", ")),
+	     domain = NA)
+        }
     }
-    if (any(linktemp == c("logit", "probit", "cloglog", "cauchit", "log")))
-	stats <- make.link(linktemp)
-    else stop(gettextf('link "%s" not available for binomial family, available links are "logit", ""probit", "cloglog", "cauchit" and "log"', linktemp),
-              domain = NA)
     variance <- function(mu) mu * (1 - mu)
     validmu <- function(mu) all(mu>0) && all(mu<1)
     dev.resids <- function(y, mu, wt)
@@ -310,17 +381,35 @@ binomial <- function (link = "logit")
 quasibinomial <- function (link = "logit")
 {
     linktemp <- substitute(link)
-    ## this is a function used in  glm();
-    ## it holds everything personal to the family
-    ## converts link into character string
     if (!is.character(linktemp)) {
 	linktemp <- deparse(linktemp)
-	if (linktemp == "link")
+        ## the idea here seems to be that we can have a character variable
+        ## 'link' naming the link.
+	if (linktemp == "link") {
+            warning("use of quasibinomial(link=link) is deprecated\n", domain = NA)
 	    linktemp <- eval(link)
+            if(!is.character(linktemp) || length(linktemp) != 1)
+                stop("'link' is invalid", domain=NA)
+        }
+    }
+
+    okLinks <- c("logit", "probit", "cloglog", "cauchit", "log")
+    if (linktemp %in% okLinks)
+        stats <- make.link(linktemp)
+    else if (is.character(link)) {
+        stats <- make.link(link)
+        linktemp <- link
+    } else {
+        ## what else shall we allow?  At least objects of class link-glm.
+        if(inherits(link, "link-glm")) {
+            stats <- link
+            if(!is.null(stats$name)) linktemp <- stats$name
+        } else {
+	    stop(gettextf('link "%s" not available for quasibinomial family; available links are %s',
+			  linktemp, paste(sQuote(okLinks), collapse =", ")),
+	     domain = NA)
+        }
     }
-    if (any(linktemp == c("logit", "probit", "cloglog", "cauchit", "log")))
-	stats <- make.link(linktemp)
-    else stop(sQuote(linktemp), ' link not available for quasibinomial family, available links are "logit", "probit", "cloglog", "cauchit" and "log"')
     variance <- function(mu) mu * (1 - mu)
     validmu <- function(mu) all(mu>0) && all(mu<1)
     dev.resids <- function(y, mu, wt)
@@ -361,17 +450,32 @@ quasibinomial <- function (link = "logit")
 Gamma <- function (link = "inverse")
 {
     linktemp <- substitute(link)
-    ## This is a function used in  glm();
-    ## it holds everything personal to the family
-    ## converts link into character string
     if (!is.character(linktemp)) {
 	linktemp <- deparse(linktemp)
-	if (linktemp == "link")
+        ## the idea here seems to be that we can have a character variable
+        ## 'link' naming the link.
+	if (linktemp == "link") {
+            warning("use of Gamma(link=link) is deprecated\n", domain = NA)
 	    linktemp <- eval(link)
+            if(!is.character(linktemp) || length(linktemp) != 1)
+                stop("'link' is invalid", domain=NA)
+        }
     }
-    if (any(linktemp == c("inverse", "log", "identity")))
+    okLinks <- c("inverse", "log", "identity")
+    if (linktemp %in% okLinks)
 	stats <- make.link(linktemp)
-    else stop(gettextf('link "%s" not available for gamma family, available links are "inverse", ""log" and "identity"', linktemp), domain = NA)
+    else if(is.character(link)) stats <- make.link(link)
+    else {
+        ## what else shall we allow?  At least objects of class link-glm.
+        if(inherits(link, "link-glm")) {
+            stats <- link
+            if(!is.null(stats$name)) linktemp <- stats$name
+        } else {
+	    stop(gettextf('link "%s" not available for gamma family; available links are %s',
+			  linktemp, paste(sQuote(okLinks), collapse =", ")),
+	     domain = NA)
+        }
+    }
     variance <- function(mu) mu^2
     validmu <- function(mu) all(mu>0)
     dev.resids <- function(y, mu, wt)
@@ -406,19 +510,35 @@ Gamma <- function (link = "inverse")
 inverse.gaussian <- function(link = "1/mu^2")
 {
     linktemp <- substitute(link)
-    ## This is a function used in  glm();
-    ## it holds everything personal to the family
-    ## converts link into character string
     if (!is.character(linktemp)) {
 	linktemp <- deparse(linktemp)
-	if (linktemp == "link")
+        ## the idea here seems to be that we can have a character variable
+        ## 'link' naming the link.
+	if (linktemp == "link") {
+            warning("use of inverse.gaussian(link=link) is deprecated\n",
+                    domain = NA)
 	    linktemp <- eval(link)
+            if(!is.character(linktemp) || length(linktemp) != 1)
+                stop("'link' is invalid", domain=NA)
+        }
     }
-    if (any(linktemp == c("inverse", "log", "identity", "1/mu^2")))
+    okLinks <- c("inverse", "log", "identity", "1/mu^2")
+    if (linktemp %in% okLinks)
 	stats <- make.link(linktemp)
-    else stop(gettextf('link "%s" not available for inverse gauss family, available links are "inverse", "1/mu^2", "log" and "identity"', linktemp),
-              domain = NA)
-    ##	stats <- make.link("1/mu^2")
+    else if (is.character(link)) {
+        stats <- make.link(link)
+        linktemp <- link
+    } else {
+        ## what else shall we allow?  At least objects of class link-glm.
+        if(inherits(link, "link-glm")) {
+            stats <- link
+            if(!is.null(stats$name)) linktemp <- stats$name
+        } else {
+	    stop(gettextf('link "%s" not available for inverse.gaussian family; available links are %s',
+			  linktemp, paste(sQuote(okLinks), collapse =", ")),
+                 domain = NA)
+        }
+    }
     variance <- function(mu) mu^3
     dev.resids <- function(y, mu, wt)  wt*((y - mu)^2)/(y*mu^2)
     aic <- function(y, n, mu, wt, dev)
@@ -448,73 +568,80 @@ inverse.gaussian <- function(link = "1/mu^2")
 quasi <- function (link = "identity", variance = "constant")
 {
     linktemp <- substitute(link)
-    ##this is a function used in  glm()
-    ##it holds everything personal to the family
-    ##converts link into character string
-    if ( is.expression(linktemp) || is.call(linktemp) )
-        linktemp <- link
-    else if (!is.character(linktemp))
-        linktemp <- deparse(linktemp)
-    if( is.character(linktemp) )
+    if (!is.character(linktemp)) linktemp <- deparse(linktemp)
+    if (linktemp %in% c("logit", "probit", "cloglog", "identity",
+                        "inverse", "log", "1/mu^2", "sqrt"))
         stats <- make.link(linktemp)
-    else
-        stats <- linktemp
-    ##converts variance into character string
-    variancetemp <- substitute(variance)
-    if (!is.character(variancetemp)) {
-	variancetemp <- deparse(variancetemp)
-	if (linktemp == "variance")
-	    variancetemp <- eval(variance)
+    else if (is.character(link)) {
+        stats <- make.link(link)
+        linktemp <- link
+    } else {
+        stats <- link
+        linktemp <- if(!is.null(stats$name)) stats$name else deparse(linktemp)
     }
-    switch(variancetemp,
-	   "constant" = {
-	       variance <- function(mu) rep.int(1, length(mu))
-	       dev.resids <- function(y, mu, wt) wt * ((y - mu)^2)
-	       validmu <- function(mu) TRUE
+    vtemp <- substitute(variance)
+    if (!is.character(vtemp)) vtemp <- deparse(vtemp)
+    variance_nm <- vtemp
+    switch(vtemp,
+           "constant" = {
+               varfun <- function(mu) rep.int(1, length(mu))
+               dev.resids <- function(y, mu, wt) wt * ((y - mu)^2)
+                   validmu <- function(mu) TRUE
                initialize <- expression({n <- rep.int(1, nobs); mustart <- y})
-	   },
-	   "mu(1-mu)" = {
-	       variance <- function(mu) mu * (1 - mu)
-	       validmu <- function(mu) all(mu>0) && all(mu<1)
-	       dev.resids <- function(y, mu, wt)
-		   2 * wt * (y * log(ifelse(y == 0, 1, y/mu)) +
-			     (1 - y) * log(ifelse(y == 1, 1, (1 - y)/(1 - mu))))
+           },
+           "mu(1-mu)" = {
+               varfun <- function(mu) mu * (1 - mu)
+               validmu <- function(mu) all(mu>0) && all(mu<1)
+               dev.resids <- function(y, mu, wt)
+                   2 * wt * (y * log(ifelse(y == 0, 1, y/mu)) +
+                             (1 - y) * log(ifelse(y == 1, 1, (1 - y)/(1 - mu))))
                initialize <- expression({n <- rep.int(1, nobs)
-                   mustart <- pmax(0.001, pmin(0.999, y))})
-	   },
-	   "mu" = {
-	       variance <- function(mu) mu
-	       validmu <- function(mu) all(mu>0)
-	       dev.resids <- function(y, mu, wt)
-		   2 * wt * (y * log(ifelse(y == 0, 1, y/mu)) - (y - mu))
+                                         mustart <- pmax(0.001, pmin(0.999, y))})
+           },
+           "mu" = {
+               varfun <- function(mu) mu
+               validmu <- function(mu) all(mu>0)
+               dev.resids <- function(y, mu, wt)
+                   2 * wt * (y * log(ifelse(y == 0, 1, y/mu)) - (y - mu))
                ## 0.1 fudge here matches poisson: S has 1/6.
                initialize <- expression({n <- rep.int(1, nobs)
-                   mustart <- y + 0.1 * (y == 0)})
-	   },
-	   "mu^2" = {
-	       variance <- function(mu) mu^2
-	       validmu <- function(mu) all(mu>0)
-	       dev.resids <- function(y, mu, wt)
+                                         mustart <- y + 0.1 * (y == 0)})
+           },
+           "mu^2" = {
+               varfun <- function(mu) mu^2
+               validmu <- function(mu) all(mu>0)
+               dev.resids <- function(y, mu, wt)
 		   pmax(-2 * wt * (log(ifelse(y == 0, 1, y)/mu) - (y - mu)/mu), 0)
                initialize <- expression({n <- rep.int(1, nobs)
-                   mustart <- y + 0.1 * (y == 0)})
-	   },
-	   "mu^3" = {
-	       variance <- function(mu) mu^3
-	       validmu <- function(mu) all(mu>0)
-	       dev.resids <- function(y, mu, wt)
-		   wt * ((y - mu)^2)/(y * mu^2)
+                                         mustart <- y + 0.1 * (y == 0)})
+           },
+           "mu^3" = {
+               varfun <- function(mu) mu^3
+               validmu <- function(mu) all(mu>0)
+               dev.resids <- function(y, mu, wt)
+                   wt * ((y - mu)^2)/(y * mu^2)
                initialize <- expression({n <- rep.int(1, nobs)
-                   mustart <- y + 0.1 * (y == 0)})
-	   },
-	   stop(gettextf('\'variance\' "%s" is invalid: possible values are "mu(1-mu)", "mu", "mu^2", "mu^3" and "constant"', variancetemp), domain = NA)
-	   )# end switch(.)
+                                         mustart <- y + 0.1 * (y == 0)})
+           },
+           variance_nm <- NA
+           )# end switch(.)
+
+    if(is.na(variance_nm)) {
+        if(is.character(variance))
+            stop(gettextf('\'variance\' "%s" is invalid: possible values are "mu(1-mu)", "mu", "mu^2", "mu^3" and "constant"', variance_nm), domain = NA)
+        ## so we really meant the object.
+        varfun <- variance$varfun
+        validmu <- variance$validmu
+        dev.resids <- variance$dev.resids
+        initialize <- variance$initialize
+        variance_nm <- variance$name
+    }
     aic <- function(y, n, mu, wt, dev) NA
     structure(list(family = "quasi",
 		   link = linktemp,
 		   linkfun = stats$linkfun,
 		   linkinv = stats$linkinv,
-		   variance = variance,
+		   variance = varfun,
 		   dev.resids = dev.resids,
 		   aic = aic,
 		   mu.eta = stats$mu.eta,
@@ -522,6 +649,6 @@ quasi <- function (link = "identity", variance = "constant")
 		   validmu = validmu,
 		   valideta = stats$valideta,
                    ## character form of the var fun is needed for gee
-                   varfun = variancetemp),
+                   varfun = variance_nm),
 	      class = "family")
 }
