@@ -17,8 +17,8 @@ str.logLik <- function(object, digits = max(2, getOption("digits") - 3), ...)
 }
 
 ## rather silly (but potentially used in pkg nlme):
-as.data.frame.logLik <- function (x, row.names = NULL, optional = FALSE)
-    as.data.frame(c(x), row.names=row.names, optional=optional)
+as.data.frame.logLik <- function (x, ...)
+    as.data.frame(c(x), ...)
 
 ## >> logLik.nls() in ../../nls/R/nls.R
 
