@@ -1,5 +1,6 @@
 .onLoad <- function(lib, pkg)
 {
+    cat("Loading Tcl/Tk interface ... ")
     if(nchar(Sys.getenv("MY_TCLTK"))) {
         library.dynam("tcltk", pkg, lib)
     } else {
@@ -15,7 +16,9 @@
     .C("tcltk_start", PACKAGE="tcltk")
     extra <- system.file("exec", package = "tcltk")
     extra <- gsub("\\\\", "/", extra)
-    invisible(addTclPath(extra))
+    addTclPath(extra)
+    cat("done\n")
+    invisible()
 }
 
 .onUnload <- function(libpath) {
