@@ -85,9 +85,14 @@ function(pattern, fields = c("alias", "concept", "title"),
 	   !identical(attr(db, "ctype"), Sys.getlocale("LC_CTYPE"))
 	   )
 	    rebuild <- TRUE
+        ## We also need to rebuild if 'packages' was used before and has
+        ## changed.
+        if (!is.null(package) &&
+            any(! package %in% db$Base[, "Package"]))
+            rebuild <- TRUE
     }
     if(rebuild) {
-	if(verbose) cat("Rebuilding the data base ..\n")
+	if(verbose) cat("Rebuilding the data base ...\n")
 	## Check whether we can save the hsearch db lateron.
 	if(all(is.na(mem.limits()))) {
 	    save_db <- save_db_to_memory <- TRUE
@@ -102,16 +107,10 @@ function(pattern, fields = c("alias", "concept", "title"),
 		save_db <- TRUE
 	}
 
-	## If we cannot save the help db only use the given packages.
-	## <NOTE>
-	## Why don't we just use the given packages?  The current logic
-	## for rebuilding cannot figure out that rebuilding is needed
-	## the next time (unless we use the same given packages) ...
 	packages_in_hsearch_db <- if(!is.null(package))
 	    package
 	else
 	    .packages(all.available = TRUE, lib.loc = lib.loc)
-	## </NOTE>
 
 	## Create the hsearch db.
 	contents_DCF_fields <-
@@ -157,9 +156,9 @@ function(pattern, fields = c("alias", "concept", "title"),
 		    hDB[[1]][, "LibPath"] <- path
 		    ## Put the hsearch index for the np-th package into the
 		    ## np-th row of the matrix used for aggregating.
-		    dbMat[np, seq(along = hDB)] <- hDB
+		    dbMat[np, seq_along(hDB)] <- hDB
 		} else if(verbose)
-		    cat("package",p, "has empty hsearch data - strangely\n")
+		    cat("package", p, "has empty hsearch data - strangely\n")
 	    }
 	    else warning("no hsearch.rds meta data for package ", p)
 	}
@@ -192,7 +191,7 @@ function(pattern, fields = c("alias", "concept", "title"),
 	## number of the package in the global index.
 	for(i in which(sapply(db, NROW) > 0)) {
 	    db[[i]][, "ID"] <-
-		paste(rep.int(seq(along = packages_in_hsearch_db),
+		paste(rep.int(seq_along(packages_in_hsearch_db),
 			      sapply(dbMat[, i], NROW)),
 		      db[[i]][, "ID"],
 		      sep = "/")
@@ -207,12 +206,12 @@ function(pattern, fields = c("alias", "concept", "title"),
 	    ## over groups of identical encodings.
 	    for(enc in unique(encoding)) {
 		IDs <- IDs_to_iconv[encoding == enc]
-		for(i in seq(along = db)) {
+		for(i in seq_along(db)) {
 		    ind <- db[[i]][, "ID"] %in% IDs
 		    db[[i]][ind, ] <- iconv(db[[i]][ind, ], enc, "")
 		}
 	    }
-	    if(verbose) cat("done\n")
+	    if(verbose) cat(" done\n")
 	}
 	## Let us be defensive about invalid multi-byte character data
 	## here.  We simple remove all Rd objects with at least one
@@ -223,14 +222,14 @@ function(pattern, fields = c("alias", "concept", "title"),
 			  u[rowSums(is.na(nchar(u, "c"))) > 0, "ID"]))
 	if(length(bad_IDs)) {
 	    warning("removing all entries with invalid multi-byte character data")
-	    for(i in seq(along = db)) {
+	    for(i in seq_along(db)) {
 		ind <- db[[i]][, "ID"] %in% bad_IDs
 		db[[i]] <- db[[i]][!ind, ]
 	    }
 	}
 
 	if(save_db) {
-	    if(verbose) cat("saving the database ..")
+	    if(verbose) cat("saving the database ...")
 	    attr(db, "LibPaths") <- lib.loc
 	    attr(db, "mtime") <- Sys.time()
 	    attr(db, "ctype") <- Sys.getlocale("LC_CTYPE")
@@ -238,7 +237,7 @@ function(pattern, fields = c("alias", "concept", "title"),
 		.hsearch_db(db)
 	    else {
 		## If we cannot save to memory, serialize to a file ...
-		.saveRDS(db, file = db_file)
+		.saveRDS(db, file = db_file, compress = TRUE)
 		## and store a promise to unserialize from this file.
 		.hsearch_db(substitute(.readRDS(con),
 				       list(con = db_file)))
@@ -256,13 +255,13 @@ function(pattern, fields = c("alias", "concept", "title"),
 	    NROW(db$Keywords), " keywords),\n",
 	    sep = "")
     if(!is.null(package)) {
-	## Argument 'package' was given but we built a larger hsearch db
-	## to save for future invocations.  Need to check that all given
-	## packages exist, and only search the given ones.
+	## Argument 'package' was given.  Need to check that all given
+	## packages exist in the db, and only search the given ones.
 	pos_in_hsearch_db <-
 	    match(package, unique(db$Base[, "Package"]), nomatch = 0)
+        ## This should not happen for R >= 2.4.0
 	if(any(pos_in_hsearch_db) == 0)
-	    stop(gettextf("could not find package '%s'",
+	    stop(gettextf("no information in the data base for package '%s': need 'rebuild = TRUE'?",
 			  package[pos_in_hsearch_db == 0][1]), domain = NA)
 	db <-
 	    lapply(db,
