@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
  */
 
 /* <UTF8>
@@ -45,6 +45,7 @@ static int R_mkdir(char *path)
 #ifdef Win32
     char local[PATH_MAX];
     strcpy(local, path);
+    /* need DOS paths on Win 9x */
     R_fixbackslash(local);
     return mkdir(local);
 #endif
@@ -161,8 +162,7 @@ do_unzip(char *zipname, char *dest, int nfiles, char **files,
     return err;
 }
 
-SEXP 
-do_int_unzip(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden do_int_unzip(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP  fn, ans, names = R_NilValue;
     char  zipname[PATH_MAX], *topics[500], dest[PATH_MAX], *p;
@@ -324,7 +324,7 @@ static int null_fflush(Rconnection con)
     return 0;
 }
 
-Rconnection R_newunz(char *description, char *mode)
+Rconnection attribute_hidden R_newunz(char *description, char *mode)
 {
     Rconnection new;
     new = (Rconnection) malloc(sizeof(struct Rconn));
@@ -362,7 +362,9 @@ Rconnection R_newunz(char *description, char *mode)
 
        /* =================== second part ====================== */
 
-/* From minizip contribution to zlib 1.1.3, reformatted by indent */
+/* From minizip contribution to zlib 1.1.3, reformatted by indent,
+   unz_copyright is now static.
+*/
 
 /* unzip.c -- IO on .zip files using zlib
    Version 0.15 beta, Mar 19th, 1998,
@@ -422,7 +424,7 @@ extern int errno;
 #define SEEK_SET    0
 #endif
 
-const char unz_copyright[] =
+static const char unz_copyright[] =
 " unzip 0.15 Copyright 1998 Gilles Vollant ";
 
 /* unz_file_info_interntal contain internal info about a file in zipfile*/
@@ -515,7 +517,7 @@ static int
 unzlocal_getShort(FILE * fin, uLong * pX)
 {
     uLong x;
-    int   i, err;
+    int   i = 0 /* -Wall */, err;
 
     err = unzlocal_getByte(fin, &i);
     x = (uLong) i;
@@ -529,7 +531,7 @@ static int
 unzlocal_getLong(FILE * fin, uLong * pX)
 {
     uLong x;
-    int   i, err;
+    int   i = 0 /* -Wall */, err;
 
     err = unzlocal_getByte(fin, &i);
     x = (uLong) i;
