@@ -14,8 +14,8 @@
  *
  *  A copy of the GNU General Public License is available via WWW at
  *  http://www.gnu.org/copyleft/gpl.html.  You can also obtain it by
- *  writing to the Free Software Foundation, Inc., 59 Temple Place,
- *  Suite 330, Boston, MA  02111-1307  USA.
+ *  writing to the Free Software Foundation, Inc., 51 Franklin Street
+ *  Fifth Floor, Boston, MA 02110-1301  USA.
 
 
  *  This is an implementation of modal event handling in R graphics
@@ -34,7 +34,7 @@
 #include <Rdevices.h>
 
 
-SEXP do_getGraphicsEvent(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden do_getGraphicsEvent(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP prompt, onMouseDown, onMouseMove, onMouseUp, onKeybd;
     GEDevDesc *dd;
