@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
  */
 
 /* <UTF8> char here is either ASCII or handled as a whole */
@@ -33,7 +33,7 @@ static R_InternetRoutines routines, *ptr = &routines;
 
 
 /*
-SEXP do_download(SEXP call, SEXP op, SEXP args, SEXP env);
+SEXP attribute_hidden do_download(SEXP call, SEXP op, SEXP args, SEXP env);
 Rconnection R_newurl(char *description, char *mode);
 Rconnection R_newsock(char *host, int port, int server, char *mode);
 
@@ -78,10 +78,10 @@ static void internet_Init(void)
 {
     int res;
 #ifdef Win32
-    res = UseInternet2 ? moduleCdynload("internet2", 1, 1) : 
-	moduleCdynload("internet", 1, 1);
+    res = UseInternet2 ? R_moduleCdynload("internet2", 1, 1) : 
+	R_moduleCdynload("internet", 1, 1);
 #else
-    res = moduleCdynload("internet", 1, 1);
+    res = R_moduleCdynload("internet", 1, 1);
 #endif
     initialized = -1;
     if(!res) return;
@@ -92,7 +92,7 @@ static void internet_Init(void)
 }
 
 
-SEXP do_download(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden do_download(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     if(!initialized) internet_Init();
     if(initialized > 0)
@@ -103,7 +103,7 @@ SEXP do_download(SEXP call, SEXP op, SEXP args, SEXP env)
     }
 }
 
-Rconnection R_newurl(char *description, char *mode)
+Rconnection attribute_hidden R_newurl(char *description, char *mode)
 {
     if(!initialized) internet_Init();
     if(initialized > 0)
@@ -129,7 +129,7 @@ void *R_HTTPOpen(const char *url)
 {
     if(!initialized) internet_Init();
     if(initialized > 0)
-	return (*ptr->HTTPOpen)(url, 0);
+	return (*ptr->HTTPOpen)(url, NULL, 0);
     else {
 	error(_("internet routines cannot be loaded"));
 	return NULL;
