@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998--2005  The R Development Core Team.
+ *  Copyright (C) 1998--2006  The R Development Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,6 +36,13 @@
 #include <config.h>
 #endif
 
+#if defined(HAVE_GLIBC2)
+/* for isnan in Rinlinedfuns.h */
+# define _SVID_SOURCE 1
+#endif
+
+#include <R_ext/RS.h> /* for S4 allocation */
+
 #if defined(Win32) && defined(LEA_MALLOC)
 #include <stddef.h>
 extern void *Rm_malloc(size_t n);
@@ -50,14 +57,14 @@ extern void *Rm_realloc(void * p, size_t n);
 
 /* Declarations for Valgrind.
 
-   These are controlled by the 
+   These are controlled by the
      --with-valgrind-instrumentation=
    option to configure, which sets VALGRIND_LEVEL to the
    supplied value (default 0) and defines NVALGRIND if
    the value is 0.
 
    level 0 is no additional instrumentation
-   level 1 marks uninitialized numeric, logical, integer vectors 
+   level 1 marks uninitialized numeric, logical, integer vectors
            and R_alloc memory
    level 2 marks free memory as inaccessible
 
@@ -78,7 +85,7 @@ extern void *Rm_realloc(void * p, size_t n);
 #endif
 
 
-#ifndef VALGRIND_LEVEL 
+#ifndef VALGRIND_LEVEL
 #define VALGRIND_LEVEL 0
 #endif
 
@@ -104,6 +111,11 @@ static int gc_count = 0;
 # define FORCE_GC 0
 #endif
 
+#ifdef R_MEMORY_PROFILING
+static void R_ReportAllocation(R_size_t);
+static void R_ReportNewPage();
+#endif
+
 extern SEXP framenames;
 
 #define GC_PROT(X) {int __t = gc_inhibit_torture; \
@@ -209,7 +221,7 @@ static R_size_t R_MaxVSize = R_SIZE_T_MAX;
 static R_size_t R_MaxNSize = R_SIZE_T_MAX;
 static int vsfac = 1; /* current units for vsize: changes at initialization */
 
-R_size_t attribute_hidden R_GetMaxVSize(void) 
+R_size_t attribute_hidden R_GetMaxVSize(void)
 {
     if (R_MaxVSize == R_SIZE_T_MAX) return R_SIZE_T_MAX;
     return R_MaxVSize*vsfac;
@@ -221,12 +233,12 @@ void attribute_hidden R_SetMaxVSize(R_size_t size)
     if (size / vsfac >= R_VSize) R_MaxVSize = (size+1)/sizeof(VECREC);
 }
 
-R_size_t attribute_hidden R_GetMaxNSize(void) 
-{ 
+R_size_t attribute_hidden R_GetMaxNSize(void)
+{
     return R_MaxNSize;
 }
 
-void R_SetMaxNSize(R_size_t size)
+void attribute_hidden R_SetMaxNSize(R_size_t size)
 {
     if (size >= R_NSize) R_MaxNSize = size;
 }
@@ -418,6 +430,7 @@ static R_size_t R_NodesInUse = 0;
   case CPLXSXP: \
   case WEAKREFSXP: \
   case RAWSXP: \
+  case S4SXP: \
     break; \
   case STRSXP: \
   case EXPRSXP: \
@@ -610,6 +623,9 @@ static void GetNewPage(int node_class)
     page = malloc(R_PAGE_SIZE);
     if (page == NULL)
 	mem_err_heap((R_size_t) NodeClassSize[node_class]);
+#ifdef R_MEMORY_PROFILING
+    R_ReportNewPage();
+#endif
     page->next = R_GenHeap[node_class].pages;
     R_GenHeap[node_class].pages = page;
     R_GenHeap[node_class].PageCount++;
@@ -622,7 +638,7 @@ static void GetNewPage(int node_class)
 	SNAP_NODE(s, base);
 #if  VALGRIND_LEVEL > 1
 	if (NodeClassSize[node_class]>0)
-	    VALGRIND_MAKE_NOACCESS(DATAPTR(s), NodeClassSize[node_class]*sizeof(VECREC));	
+	    VALGRIND_MAKE_NOACCESS(DATAPTR(s), NodeClassSize[node_class]*sizeof(VECREC));
 #endif
 	s->sxpinfo = UnmarkedNodeTemplate.sxpinfo;
 	SET_NODE_CLASS(s, node_class);
@@ -919,7 +935,7 @@ static SEXP NewWeakRef(SEXP key, SEXP val, SEXP fin, Rboolean onexit)
 	break;
     default: error(_("can only weakly reference/finalize reference objects"));
     }
-	
+
     PROTECT(key);
     PROTECT(val = NAMED(val) ? duplicate(val) : val);
     PROTECT(fin);
@@ -1126,14 +1142,21 @@ void R_RegisterCFinalizer(SEXP s, R_CFinalizer_t fun)
 
 SEXP attribute_hidden do_regFinaliz(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
+    int onexit;
+
     checkArity(op, args);
 
     if (TYPEOF(CAR(args)) != ENVSXP && TYPEOF(CAR(args)) != EXTPTRSXP)
-	errorcall(call, _("first argument must be environment or external pointer"));
+	errorcall(call, 
+		  _("first argument must be environment or external pointer"));
     if (TYPEOF(CADR(args)) != CLOSXP)
 	errorcall(call, _("second argument must be a function"));
-    
-    R_RegisterFinalizer(CAR(args), CADR(args));
+
+    onexit = asLogical(CADDR(args));
+    if(onexit == NA_LOGICAL)
+	errorcall(call, _("third argument must be 'TRUE' or 'FALSE'"));
+
+    R_RegisterFinalizerEx(CAR(args), CADR(args), onexit);
     return R_NilValue;
 }
 
@@ -1236,10 +1259,8 @@ static void RunGenCollect(R_size_t size_needed)
     FORWARD_NODE(R_EmptyEnv);
     FORWARD_NODE(R_Warnings);	           /* Warnings, if any */
 
-#ifdef NEW_CONDITION_HANDLING
     FORWARD_NODE(R_HandlerStack);          /* Condition handler stack */
     FORWARD_NODE(R_RestartStack);          /* Available restarts stack */
-#endif
 
     for (i = 0; i < HSIZE; i++)	           /* Symbol table */
 	FORWARD_NODE(R_SymbolTable[i]);
@@ -1266,10 +1287,8 @@ static void RunGenCollect(R_size_t size_needed)
         FORWARD_NODE(ctxt->sysparent);     /* calling environment */
 	FORWARD_NODE(ctxt->call);          /* the call */
 	FORWARD_NODE(ctxt->cloenv);        /* the closure environment */
-#ifdef NEW_CONDITION_HANDLING
 	FORWARD_NODE(ctxt->handlerstack);  /* the condition handler stack */
 	FORWARD_NODE(ctxt->restartstack);  /* the available restarts stack */
-#endif
     }
 
     FORWARD_NODE(framenames); 		   /* used for interprocedure
@@ -1316,7 +1335,7 @@ static void RunGenCollect(R_size_t size_needed)
 
     /* mark nodes ready for finalizing */
     CheckFinalizers();
-    
+
     /* process the weak reference chain */
     for (s = R_weak_refs; s != R_NilValue; s = WEAKREF_NEXT(s)) {
 	FORWARD_NODE(s);
@@ -1349,7 +1368,7 @@ static void RunGenCollect(R_size_t size_needed)
     for (i = 0; i < NUM_NODE_CLASSES; i++)
 	R_GenHeap[i].Free = NEXT_NODE(R_GenHeap[i].New);
 
- 
+
     /* update heap statistics */
     R_Collected = R_NSize;
     R_SmallVallocSize = 0;
@@ -1425,6 +1444,17 @@ SEXP attribute_hidden do_gcinfo(SEXP call, SEXP op, SEXP args, SEXP rho)
     return old;
 }
 
+/* reports memory use to profiler in eval.c */
+
+void attribute_hidden get_current_mem(unsigned long *smallvsize,
+				      unsigned long *largevsize,
+				      unsigned long *nodes)
+{
+    *smallvsize = R_SmallVallocSize;
+    *largevsize = R_LargeVallocSize;
+    *nodes = R_NodesInUse * sizeof(SEXPREC);
+    return;
+}
 
 SEXP attribute_hidden do_gc(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
@@ -1451,13 +1481,13 @@ SEXP attribute_hidden do_gc(SEXP call, SEXP op, SEXP args, SEXP rho)
     INTEGER(value)[3] = 10. * (R_VSize - VHEAP_FREE())/Mega * vsfac + 0.999;
     INTEGER(value)[6] = 10. * R_NSize/Mega * sizeof(SEXPREC) + 0.999;
     INTEGER(value)[7] = 10. * R_VSize/Mega * vsfac + 0.999;
-    INTEGER(value)[8] = (R_MaxNSize < R_SIZE_T_MAX) ? 
+    INTEGER(value)[8] = (R_MaxNSize < R_SIZE_T_MAX) ?
 	(10. * R_MaxNSize/Mega * sizeof(SEXPREC) + 0.999) : NA_INTEGER;
-    INTEGER(value)[9] = (R_MaxVSize < R_SIZE_T_MAX) ? 
+    INTEGER(value)[9] = (R_MaxVSize < R_SIZE_T_MAX) ?
 	(10. * R_MaxVSize/Mega * vsfac + 0.999) : NA_INTEGER;
     if (reset_max){
 	    R_N_maxused = INTEGER(value)[0];
-	    R_V_maxused = INTEGER(value)[1];	    
+	    R_V_maxused = INTEGER(value)[1];
     }
     INTEGER(value)[10] = (R_N_maxused < INT_MAX) ? R_N_maxused : NA_INTEGER;
     INTEGER(value)[11] = (R_V_maxused < INT_MAX) ? R_V_maxused : NA_INTEGER;
@@ -1563,9 +1593,7 @@ void attribute_hidden InitMemory()
 #endif
     R_weak_refs = R_NilValue;
 
-#ifdef NEW_CONDITION_HANDLING
     R_HandlerStack = R_RestartStack = R_NilValue;
-#endif
 }
 
 /* Since memory allocated from the heap is non-moving, R_alloc just
@@ -1592,6 +1620,8 @@ char *R_alloc(long nelem, int eltsize)
     if (dsize > 0) { /* precaution against integer overflow */
 	SEXP s;
 #if SIZEOF_LONG > 4
+	/* In this case by allocating larger units we can get up to
+	   size(double) * (2^31 - 1) bytes, approx 16Gb */
 	if(dsize < R_LEN_T_MAX)
 	    s = allocString(size); /**** avoid extra null byte?? */
 	else if(dsize < sizeof(double) * (R_LEN_T_MAX - 1))
@@ -1615,6 +1645,8 @@ char *R_alloc(long nelem, int eltsize)
     else return NULL;
 }
 
+
+
 /* S COMPATIBILITY */
 
 char *S_alloc(long nelem, int eltsize)
@@ -1631,16 +1663,18 @@ char *S_alloc(long nelem, int eltsize)
 
 char *S_realloc(char *p, long new, long old, int size)
 {
-    int i, nold;
+    int /*i,*/ nold;
     char *q;
     /* shrinking is a no-op */
     if(new <= old) return p;
     q = R_alloc(new, size);
     nold = old * size;
-    for(i = 0; i < nold; i++)
+    memcpy(q, p, nold);
+    memset(q + nold, 0, new*size - nold);
+    /* for(i = 0; i < nold; i++)
 	q[i] = p[i];
     for(i = nold; i < new*size; i++)
-	q[i] = 0;
+        q[i] = 0; */
     return q;
 }
 
@@ -1682,7 +1716,7 @@ static SEXP allocSExpNonCons(SEXPTYPE t)
     TAG(s) = R_NilValue;
     ATTRIB(s) = R_NilValue;
 #if VALGRIND_LEVEL > 2
-    VALGRIND_MAKE_READABLE(s, sizeof(*s)); 
+    VALGRIND_MAKE_READABLE(s, sizeof(*s));
 #endif
     return s;
 }
@@ -1693,16 +1727,16 @@ SEXP cons(SEXP car, SEXP cdr)
 {
     SEXP s;
     if (FORCE_GC || NO_FREE_NODES()) {
-      PROTECT(car);
-      PROTECT(cdr);
-      R_gc_internal(0);
-      UNPROTECT(2);
-      if (NO_FREE_NODES())
-	mem_err_cons();
+	PROTECT(car);
+	PROTECT(cdr);
+	R_gc_internal(0);
+	UNPROTECT(2);
+	if (NO_FREE_NODES())
+	    mem_err_cons();
     }
     GET_FREE_NODE(s);
 #if VALGRIND_LEVEL > 2
-    VALGRIND_MAKE_READABLE(s, sizeof(*s)); 
+    VALGRIND_MAKE_READABLE(s, sizeof(*s));
 #endif
     s->sxpinfo = UnmarkedNodeTemplate.sxpinfo;
     TYPEOF(s) = LISTSXP;
@@ -1736,13 +1770,13 @@ SEXP NewEnvironment(SEXP namelist, SEXP valuelist, SEXP rho)
     SEXP v, n, newrho;
 
     if (FORCE_GC || NO_FREE_NODES()) {
-      PROTECT(namelist);
-      PROTECT(valuelist);
-      PROTECT(rho);
-      R_gc_internal(0);
-      UNPROTECT(3);
-      if (NO_FREE_NODES())
-	mem_err_cons();
+	PROTECT(namelist);
+	PROTECT(valuelist);
+	PROTECT(rho);
+	R_gc_internal(0);
+	UNPROTECT(3);
+	if (NO_FREE_NODES())
+	    mem_err_cons();
     }
     GET_FREE_NODE(newrho);
 #if VALGRIND_LEVEL > 2
@@ -1771,12 +1805,12 @@ SEXP attribute_hidden mkPROMISE(SEXP expr, SEXP rho)
 {
     SEXP s;
     if (FORCE_GC || NO_FREE_NODES()) {
-      PROTECT(expr);
-      PROTECT(rho);
-      R_gc_internal(0);
-      UNPROTECT(2);
-      if (NO_FREE_NODES())
-	mem_err_cons();
+	PROTECT(expr);
+	PROTECT(rho);
+	R_gc_internal(0);
+	UNPROTECT(2);
+	if (NO_FREE_NODES())
+	    mem_err_cons();
     }
     GET_FREE_NODE(s);
 #if VALGRIND_LEVEL > 2
@@ -1795,9 +1829,11 @@ SEXP attribute_hidden mkPROMISE(SEXP expr, SEXP rho)
 /* All vector objects  must be a multiple of sizeof(ALIGN) */
 /* bytes so that alignment is preserved for all objects */
 
-/* allocString is now in Rinlinedfuns.h */
+/* allocString is now a macro */
 
-/* Allocate a vector object on the heap */
+/* Allocate a vector object.  This ensures only validity of list-like
+   SEXPTYPES (as the elements must be initialized).  Initializing of
+   other vector types is done in do_makevector */
 
 SEXP allocVector(SEXPTYPE type, R_len_t length)
 {
@@ -1878,24 +1914,24 @@ SEXP allocVector(SEXPTYPE type, R_len_t length)
     case LISTSXP:
 	return allocList(length);
     default:
-	error(_("invalid type/length (%d/%d) in vector allocation"), 
+	error(_("invalid type/length (%d/%d) in vector allocation"),
 	      type, length);
     }
 
     if (size <= NodeClassSize[1]) {
-      node_class = 1;
-      alloc_size = NodeClassSize[1];
+	node_class = 1;
+	alloc_size = NodeClassSize[1];
     }
     else {
-      node_class = LARGE_NODE_CLASS;
-      alloc_size = size;
-      for (i = 2; i < NUM_SMALL_NODE_CLASSES; i++) {
-	if (size <= NodeClassSize[i]) {
-	  node_class = i;
-	  alloc_size = NodeClassSize[i];
-	  break;
+	node_class = LARGE_NODE_CLASS;
+	alloc_size = size;
+	for (i = 2; i < NUM_SMALL_NODE_CLASSES; i++) {
+	    if (size <= NodeClassSize[i]) {
+		node_class = i;
+		alloc_size = NodeClassSize[i];
+		break;
+	    }
 	}
-      }
     }
 
     /* save current R_VSize to roll back adjustment if malloc fails */
@@ -1913,7 +1949,7 @@ SEXP allocVector(SEXPTYPE type, R_len_t length)
     if (size > 0) {
 	if (node_class < NUM_SMALL_NODE_CLASSES) {
 	    CLASS_GET_FREE_NODE(node_class, s);
-#if VALGRIND_LEVEL > 2	    
+#if VALGRIND_LEVEL > 2
 	    VALGRIND_MAKE_WRITABLE(s, 4); /* sizeof sxpinfo_struct */
 #endif
 #if VALGRIND_LEVEL > 1
@@ -1936,6 +1972,9 @@ SEXP allocVector(SEXPTYPE type, R_len_t length)
 		    s = malloc(sizeof(SEXPREC_ALIGN) + size * sizeof(VECREC));
 		}
 		if (s != NULL) success = TRUE;
+#ifdef R_MEMORY_PROFILING
+		R_ReportAllocation(sizeof(SEXPREC_ALIGN) + size * sizeof(VECREC));
+#endif
 	    }
 	    if (! success) {
 		/* reset the vector heap limit */
@@ -1985,17 +2024,18 @@ SEXP allocVector(SEXPTYPE type, R_len_t length)
  	VALGRIND_MAKE_WRITABLE(CHAR(s), actual_size);
 #endif
 	CHAR(s)[length] = 0;
-    } 
+    }
     else if (type == REALSXP){
 #if VALGRIND_LEVEL > 0
 	VALGRIND_MAKE_WRITABLE(REAL(s), actual_size);
 #endif
-    } 
+    }
     else if (type == INTSXP){
 #if VALGRIND_LEVEL > 0
 	VALGRIND_MAKE_WRITABLE(INTEGER(s), actual_size);
 #endif
     }
+    /* <FIXME> why not valgrindify LGLSXP, CPLXSXP and RAWSXP/ */
     return s;
 }
 
@@ -2004,12 +2044,20 @@ SEXP allocList(int n)
     int i;
     SEXP result;
     result = R_NilValue;
-    for (i = 0; i < n; i++) {
+    for (i = 0; i < n; i++)
 	result = CONS(R_NilValue, result);
-    }
     return result;
 }
 
+SEXP allocS4Object()
+{
+   SEXP s;
+   GC_PROT(s = allocSExpNonCons(S4SXP));
+   SET_S4_OBJECT(s);
+   return s;
+}
+
+
 /* "gc" a mark-sweep or in-place generational garbage collector */
 
 void R_gc(void)
@@ -2089,9 +2137,9 @@ static void R_gc_internal(R_size_t size_needed)
     R_V_maxused = R_MAX(R_V_maxused, R_VSize - VHEAP_FREE());
 
     BEGIN_SUSPEND_INTERRUPTS {
-      gc_start_timing();
-      RunGenCollect(size_needed);
-      gc_end_timing();
+	gc_start_timing();
+	RunGenCollect(size_needed);
+	gc_end_timing();
     } END_SUSPEND_INTERRUPTS;
 
     if (gc_reporting) {
@@ -2125,7 +2173,7 @@ SEXP attribute_hidden do_memlimits(SEXP call, SEXP op, SEXP args, SEXP env)
     SEXP ans;
     int nsize, vsize;
     R_size_t tmp;
-    
+
     checkArity(op, args);
     nsize = asInteger(CAR(args));
     vsize = asInteger(CADR(args));
@@ -2156,7 +2204,7 @@ SEXP attribute_hidden do_memoryprofile(SEXP call, SEXP op, SEXP args, SEXP env)
     BEGIN_SUSPEND_INTERRUPTS {
       int gen;
 
-      /* run a full GC to make sure that all stuff in use in in Old space */
+      /* run a full GC to make sure that all stuff in use is in Old space */
       num_old_gens_to_collect = NUM_OLD_GENERATIONS;
       R_gc();
       for (gen = 0; gen < NUM_OLD_GENERATIONS; gen++) {
@@ -2193,6 +2241,7 @@ static void reset_pp_stack(void *data)
     R_size_t *poldpps = data;
     R_PPStackSize =  *poldpps;
 }
+
 SEXP protect(SEXP s)
 {
     if (R_PPStackTop >= R_PPStackSize) {
@@ -2222,7 +2271,7 @@ void unprotect(int l)
     if (R_PPStackTop >=  l)
 	R_PPStackTop -= l;
     else
-	error(_("unprotect(): stack imbalance"));
+	error(_("unprotect(): only %d protected items"), R_PPStackTop);
 }
 
 
@@ -2279,10 +2328,11 @@ void *R_chk_calloc(size_t nelem, size_t elsize)
 	return(NULL);
 #endif
     p = calloc(nelem, elsize);
-    if(!p) error(_("Calloc could not allocate (%d of %d) memory"), 
+    if(!p) error(_("Calloc could not allocate (%d of %d) memory"),
 		 nelem, elsize);
     return(p);
 }
+
 void *R_chk_realloc(void *ptr, size_t size)
 {
     void *p;
@@ -2291,6 +2341,7 @@ void *R_chk_realloc(void *ptr, size_t size)
     if(!p) error(_("Realloc could not re-allocate (size %d) memory"), size);
     return(p);
 }
+
 void R_chk_free(void *ptr)
 {
     /* S-PLUS warns here, but there seems no reason to do so */
@@ -2373,6 +2424,12 @@ void R_SetExternalPtrProtected(SEXP s, SEXP p)
     EXTPTR_PROT(s) = p;
 }
 
+#define USE_TYPE_CHECKING
+
+#if defined(USE_TYPE_CHECKING_STRICT) && !defined(USE_TYPE_CHECKING)
+# define USE_TYPE_CHECKING
+#endif
+
 
 /* The following functions are replacements for the accessor macros.
    They are used by code that does not have direct access to the
@@ -2385,42 +2442,169 @@ int (OBJECT)(SEXP x) { return OBJECT(x); }
 int (MARK)(SEXP x) { return MARK(x); }
 int (TYPEOF)(SEXP x) { return TYPEOF(x); }
 int (NAMED)(SEXP x) { return NAMED(x); }
+int (TRACE)(SEXP x) { return TRACE(x); }
+int (LEVELS)(SEXP x) { return LEVELS(x); }
 
-void (SET_ATTRIB)(SEXP x, SEXP v) { CHECK_OLD_TO_NEW(x, v); ATTRIB(x) = v; }
+void (SET_ATTRIB)(SEXP x, SEXP v) { 
+#ifdef USE_TYPE_CHECKING
+    if(TYPEOF(v) != LISTSXP && TYPEOF(v) != NILSXP) 
+	error("value of 'SET_ATTRIB' must be a pairlist or NULL, not a '%s'", 
+	      type2char(TYPEOF(x)));
+#endif
+    CHECK_OLD_TO_NEW(x, v); 
+    ATTRIB(x) = v; 
+}
 void (SET_OBJECT)(SEXP x, int v) { SET_OBJECT(x, v); }
 void (SET_TYPEOF)(SEXP x, int v) { SET_TYPEOF(x, v); }
 void (SET_NAMED)(SEXP x, int v) { SET_NAMED(x, v); }
+void (SET_TRACE)(SEXP x, int v) { SET_TRACE(x, v); }
+int (SETLEVELS)(SEXP x, int v) { return SETLEVELS(x, v); }
+
+/* S4 object testing */
+int (IS_S4_OBJECT)(SEXP x){ return IS_S4_OBJECT(x); }
+void (SET_S4_OBJECT)(SEXP x){ SET_S4_OBJECT(x); }
+void (UNSET_S4_OBJECT)(SEXP x){ UNSET_S4_OBJECT(x); }
 
 /* Vector Accessors */
 int (LENGTH)(SEXP x) { return LENGTH(x); }
 int (TRUELENGTH)(SEXP x) { return TRUELENGTH(x); }
-char *(R_CHAR)(SEXP x) { return CHAR(x); }
-SEXP (STRING_ELT)(SEXP x, int i) { return STRING_ELT(x, i); }
-SEXP (VECTOR_ELT)(SEXP x, int i) { return VECTOR_ELT(x, i); }
-int (LEVELS)(SEXP x) { return LEVELS(x); }
+void (SETLENGTH)(SEXP x, int v) { SETLENGTH(x, v); }
+void (SET_TRUELENGTH)(SEXP x, int v) { SET_TRUELENGTH(x, v); }
+
+char *(R_CHAR)(SEXP x) {
+#ifdef USE_TYPE_CHECKING
+    if(TYPEOF(x) != CHARSXP) 
+	error("%s() can only be applied to a '%s', not a '%s'", 
+	      "CHAR", "CHARSXP", type2char(TYPEOF(x)));
+#endif
+    return CHAR(x);
+}
+
+SEXP (STRING_ELT)(SEXP x, int i) {
+#ifdef USE_TYPE_CHECKING
+    if(TYPEOF(x) != STRSXP)
+	error("%s() can only be applied to a '%s', not a '%s'", 
+	      "STRING_ELT", "character vector", type2char(TYPEOF(x)));
+#endif
+    return STRING_ELT(x, i);
+}
+
+SEXP (VECTOR_ELT)(SEXP x, int i) {
+#ifdef USE_TYPE_CHECKING_STRICT
+    /* We need to allow vector-like types here */
+    if(TYPEOF(x) != VECSXP && 
+       TYPEOF(x) != EXPRSXP && 
+       TYPEOF(x) != WEAKREFSXP)
+	error("%s() can only be applied to a '%s', not a '%s'", 
+	      "VECTOR_ELT", "list", type2char(TYPEOF(x)));
+#elif defined(USE_TYPE_CHECKING)
+    /* also allow STRSXP */
+    if(TYPEOF(x) != VECSXP && TYPEOF(x) != STRSXP &&
+       TYPEOF(x) != EXPRSXP && 
+       TYPEOF(x) != WEAKREFSXP)
+	error("%s() can only be applied to a '%s', not a '%s'", 
+	      "VECTOR_ELT", "list", type2char(TYPEOF(x)));
+#endif
+    return VECTOR_ELT(x, i);
+}
+
+int *(LOGICAL)(SEXP x) {
+#ifdef USE_TYPE_CHECKING_STRICT
+    if(TYPEOF(x) != LGLSXP)
+	error("%s() can only be applied to a '%s', not a '%s'",
+	      "LOGICAL",  "logical", type2char(TYPEOF(x)));
+#elif defined(USE_TYPE_CHECKING)
+    /* Currently harmless, and quite widely used */
+    if(TYPEOF(x) != LGLSXP && TYPEOF(x) != INTSXP)
+	error("%s() can only be applied to a '%s', not a '%s'",
+	      "LOGICAL",  "logical", type2char(TYPEOF(x)));
+#endif
+  return LOGICAL(x); 
+}
+
+int *(INTEGER)(SEXP x) {
+#ifdef USE_TYPE_CHECKING
+    if(TYPEOF(x) != INTSXP && TYPEOF(x) != LGLSXP)
+        error("%s() can only be applied to a '%s', not a '%s'",
+	      "INTEGER", "integer", type2char(TYPEOF(x)));
+#endif
+    return INTEGER(x); 
+}
+
+Rbyte *(RAW)(SEXP x) { 
+#ifdef USE_TYPE_CHECKING
+    if(TYPEOF(x) != RAWSXP) 
+	error("%s() can only be applied to a '%s', not a '%s'", 
+	      "RAW", "raw", type2char(TYPEOF(x)));
+#endif
+    return RAW(x); 
+}
+
+double *(REAL)(SEXP x) { 
+#ifdef USE_TYPE_CHECKING
+    if(TYPEOF(x) != REALSXP) 
+	error("%s() can only be applied to a '%s', not a '%s'", 
+	      "REAL", "numeric", type2char(TYPEOF(x)));
+#endif
+    return REAL(x); 
+}
+
+Rcomplex *(COMPLEX)(SEXP x) { 
+#ifdef USE_TYPE_CHECKING
+    if(TYPEOF(x) != CPLXSXP)
+	error("%s() can only be applied to a '%s', not a '%s'", 
+	      "COMPLEX", "complex", type2char(TYPEOF(x)));
+#endif
+    return COMPLEX(x); 
+}
 
-int *(LOGICAL)(SEXP x) { return LOGICAL(x); }
-int *(INTEGER)(SEXP x) { return INTEGER(x); }
-Rbyte *(RAW)(SEXP x) { return RAW(x); }
-double *(REAL)(SEXP x) { return REAL(x); }
-Rcomplex *(COMPLEX)(SEXP x) { return COMPLEX(x); }
 SEXP *(STRING_PTR)(SEXP x) { return STRING_PTR(x); }
+
 SEXP *(VECTOR_PTR)(SEXP x)
 {
   error(_("not safe to return vector pointer"));
   return NULL;
 }
 
-void (SETLENGTH)(SEXP x, int v) { SETLENGTH(x, v); }
-void (SET_TRUELENGTH)(SEXP x, int v) { SET_TRUELENGTH(x, v); }
-void (SET_STRING_ELT)(SEXP x, int i, SEXP v) { CHECK_OLD_TO_NEW(x, v); STRING_ELT(x, i) = v; }
-SEXP (SET_VECTOR_ELT)(SEXP x, int i, SEXP v) { CHECK_OLD_TO_NEW(x, v); return VECTOR_ELT(x, i) =v; }
-int (SETLEVELS)(SEXP x, int v) { return SETLEVELS(x, v); }
+void (SET_STRING_ELT)(SEXP x, int i, SEXP v) { 
+#ifdef USE_TYPE_CHECKING
+    if(TYPEOF(x) != STRSXP)
+	error("%s() can only be applied to a '%s', not a '%s'", 
+	      "SET_STRING_ELT", "character vector", type2char(TYPEOF(x)));
+    if(TYPEOF(v) != CHARSXP && TYPEOF(v) != NILSXP)
+       error("Value of SET_STRING_ELT() must be a 'CHARSXP' not a '%s'", 
+	     type2char(TYPEOF(v)));
+#endif
+    CHECK_OLD_TO_NEW(x, v);
+    STRING_ELT(x, i) = v; 
+}
+
+SEXP (SET_VECTOR_ELT)(SEXP x, int i, SEXP v) { 
+#ifdef USE_TYPE_CHECKING_STRICT
+    /*  we need to allow vector-like types here */
+    if(TYPEOF(x) != VECSXP &&
+       TYPEOF(x) != EXPRSXP &&
+       TYPEOF(x) != WEAKREFSXP) {
+	error("%s() can only be applied to a '%s', not a '%s'", 
+	      "SET_VECTOR_ELT", "list", type2char(TYPEOF(x)));
+    }
+#elif defined(USE_TYPE_CHECKING)
+    /* also allow STRSXP */
+    if(TYPEOF(x) != VECSXP && TYPEOF(x) != STRSXP &&
+       TYPEOF(x) != EXPRSXP && TYPEOF(x) != WEAKREFSXP) {
+	error("%s() can only be applied to a '%s', not a '%s'", 
+	      "SET_VECTOR_ELT", "list", type2char(TYPEOF(x)));
+    }
+#endif
+    CHECK_OLD_TO_NEW(x, v); 
+    return VECTOR_ELT(x, i) = v; 
+}
+
 
 /* List Accessors */
 SEXP (TAG)(SEXP e) { return TAG(e); }
 SEXP (CAR)(SEXP e) { return CAR(e); }
-SEXP (CDR)(SEXP e) {return CDR(e); }
+SEXP (CDR)(SEXP e) { return CDR(e); }
 SEXP (CAAR)(SEXP e) { return CAAR(e); }
 SEXP (CDAR)(SEXP e) { return CDAR(e); }
 SEXP (CADR)(SEXP e) { return CADR(e); }
@@ -2434,78 +2618,78 @@ void (SET_TAG)(SEXP x, SEXP v) { CHECK_OLD_TO_NEW(x, v); TAG(x) = v; }
 
 SEXP (SETCAR)(SEXP x, SEXP y)
 {
-  if (x == NULL || x == R_NilValue)
-    error(_("bad value"));
-  CHECK_OLD_TO_NEW(x, y);
-  CAR(x) = y;
-  return y;
+    if (x == NULL || x == R_NilValue)
+	error(_("bad value"));
+    CHECK_OLD_TO_NEW(x, y);
+    CAR(x) = y;
+    return y;
 }
 
 SEXP (SETCDR)(SEXP x, SEXP y)
 {
-  if (x == NULL || x == R_NilValue)
-    error(_("bad value"));
-  CHECK_OLD_TO_NEW(x, y);
-  CDR(x) = y;
-  return y;
+    if (x == NULL || x == R_NilValue)
+	error(_("bad value"));
+    CHECK_OLD_TO_NEW(x, y);
+    CDR(x) = y;
+    return y;
 }
 
 SEXP (SETCADR)(SEXP x, SEXP y)
 {
-  SEXP cell;
-  if (x == NULL || x == R_NilValue ||
-      CDR(x) == NULL || CDR(x) == R_NilValue)
-    error(_("bad value"));
-  cell = CDR(x);
-  CHECK_OLD_TO_NEW(cell, y);
-  CAR(cell) = y;
-  return y;
+    SEXP cell;
+    if (x == NULL || x == R_NilValue ||
+	CDR(x) == NULL || CDR(x) == R_NilValue)
+	error(_("bad value"));
+    cell = CDR(x);
+    CHECK_OLD_TO_NEW(cell, y);
+    CAR(cell) = y;
+    return y;
 }
 
 SEXP (SETCADDR)(SEXP x, SEXP y)
 {
-  SEXP cell;
-  if (x == NULL || x == R_NilValue ||
-      CDR(x) == NULL || CDR(x) == R_NilValue ||
-      CDDR(x) == NULL || CDDR(x) == R_NilValue)
-    error(_("bad value"));
-  cell = CDDR(x);
-  CHECK_OLD_TO_NEW(cell, y);
-  CAR(cell) = y;
-  return y;
+    SEXP cell;
+    if (x == NULL || x == R_NilValue ||
+	CDR(x) == NULL || CDR(x) == R_NilValue ||
+	CDDR(x) == NULL || CDDR(x) == R_NilValue)
+	error(_("bad value"));
+    cell = CDDR(x);
+    CHECK_OLD_TO_NEW(cell, y);
+    CAR(cell) = y;
+    return y;
 }
 
 #define CDDDR(x) CDR(CDR(CDR(x)))
 
 SEXP (SETCADDDR)(SEXP x, SEXP y)
 {
-  SEXP cell;
-  if (x == NULL || x == R_NilValue ||
-      CDR(x) == NULL || CDR(x) == R_NilValue ||
-      CDDR(x) == NULL || CDDR(x) == R_NilValue ||
-      CDDDR(x) == NULL || CDDDR(x) == R_NilValue)
-    error(_("bad value"));
-  cell = CDDDR(x);
-  CHECK_OLD_TO_NEW(cell, y);
-  CAR(cell) = y;
-  return y;
+    SEXP cell;
+    if (x == NULL || x == R_NilValue ||
+	CDR(x) == NULL || CDR(x) == R_NilValue ||
+	CDDR(x) == NULL || CDDR(x) == R_NilValue ||
+	CDDDR(x) == NULL || CDDDR(x) == R_NilValue)
+	error(_("bad value"));
+    cell = CDDDR(x);
+    CHECK_OLD_TO_NEW(cell, y);
+    CAR(cell) = y;
+    return y;
 }
 
 #define CD4R(x) CDR(CDR(CDR(CDR(x))))
 
 SEXP (SETCAD4R)(SEXP x, SEXP y)
 {
-  SEXP cell;
-  if (x == NULL || x == R_NilValue ||
-      CDR(x) == NULL || CDR(x) == R_NilValue ||
-      CDDR(x) == NULL || CDDR(x) == R_NilValue ||
-      CDDDR(x) == NULL || CDDDR(x) == R_NilValue ||
-      CD4R(x) == NULL || CD4R(x) == R_NilValue)
-    error(_("bad value"));
-  cell = CD4R(x);
-  CHECK_OLD_TO_NEW(cell, y);
-  CAR(cell) = y;
-  return y;
+    SEXP cell;
+    if (x == NULL || x == R_NilValue ||
+	CDR(x) == NULL || CDR(x) == R_NilValue ||
+	CDDR(x) == NULL || CDDR(x) == R_NilValue ||
+	CDDDR(x) == NULL || CDDDR(x) == R_NilValue ||
+	CD4R(x) == NULL || CD4R(x) == R_NilValue)
+	error(_("bad value"));
+    cell = CD4R(x);
+    CHECK_OLD_TO_NEW(cell, y);
+    CAR(cell) = y;
+    return y;
 }
 
 void (SET_MISSING)(SEXP x, int v) { SET_MISSING(x, v); }
@@ -2515,13 +2699,11 @@ SEXP (FORMALS)(SEXP x) { return FORMALS(x); }
 SEXP (BODY)(SEXP x) { return BODY(x); }
 SEXP (CLOENV)(SEXP x) { return CLOENV(x); }
 int (DEBUG)(SEXP x) { return DEBUG(x); }
-int (TRACE)(SEXP x) { return TRACE(x); }
 
 void (SET_FORMALS)(SEXP x, SEXP v) { CHECK_OLD_TO_NEW(x, v); FORMALS(x) = v; }
 void (SET_BODY)(SEXP x, SEXP v) { CHECK_OLD_TO_NEW(x, v); BODY(x) = v; }
 void (SET_CLOENV)(SEXP x, SEXP v) { CHECK_OLD_TO_NEW(x, v); CLOENV(x) = v; }
 void (SET_DEBUG)(SEXP x, int v) { SET_DEBUG(x, v); }
-void (SET_TRACE)(SEXP x, int v) { SET_TRACE(x, v); }
 
 /* Primitive Accessors */
 int (PRIMOFFSET)(SEXP x) { return PRIMOFFSET(x); }
@@ -2567,3 +2749,121 @@ int (HASHVALUE)(SEXP x) { return HASHVALUE(x); }
 
 void (SET_HASHASH)(SEXP x, int v) { SET_HASHASH(x, v); }
 void (SET_HASHVALUE)(SEXP x, int v) { SET_HASHVALUE(x, v); }
+
+/* Bindings accessors */
+Rboolean attribute_hidden 
+(IS_ACTIVE_BINDING)(SEXP b) {return IS_ACTIVE_BINDING(b);}
+Rboolean attribute_hidden
+(BINDING_IS_LOCKED)(SEXP b) {return BINDING_IS_LOCKED(b);}
+void attribute_hidden 
+(SET_ACTIVE_BINDING_BIT)(SEXP b) {SET_ACTIVE_BINDING_BIT(b);}
+void attribute_hidden (LOCK_BINDING)(SEXP b) {LOCK_BINDING(b);}
+void attribute_hidden (UNLOCK_BINDING)(SEXP b) {UNLOCK_BINDING(b);}
+
+
+/*******************************************/
+/* Non-sampling memory use profiler
+   reports all large vector heap
+   allocations and all calls to GetNewPage */
+/*******************************************/
+
+#ifndef R_MEMORY_PROFILING
+
+SEXP attribute_hidden do_Rprofmem(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    errorcall(call,_("memory profiling is not available on this system"));
+    return R_NilValue; /* not reached */
+}
+
+#else
+static int R_IsMemReporting;  /* Rboolean more appropriate? */
+static FILE *R_MemReportingOutfile;
+static R_size_t R_MemReportingThreshold;
+
+static void R_OutputStackTrace(FILE *file)
+{
+    int newline = 0;
+    RCNTXT *cptr;
+
+    for (cptr = R_GlobalContext; cptr; cptr = cptr->nextcontext) {
+	if ((cptr->callflag & (CTXT_FUNCTION | CTXT_BUILTIN))
+	    && TYPEOF(cptr->call) == LANGSXP) {
+	    SEXP fun = CAR(cptr->call);
+	    if (!newline) newline = 1;
+	    fprintf(file, "\"%s\" ",
+		    TYPEOF(fun) == SYMSXP ? CHAR(PRINTNAME(fun)) :
+		    "<Anonymous>");
+	}
+    }
+    if (newline) fprintf(file, "\n");
+}
+
+static void R_ReportAllocation(R_size_t size)
+{
+    int newline = 0;
+
+    if (R_IsMemReporting) {
+	if(size > R_MemReportingThreshold) {
+	    fprintf(R_MemReportingOutfile,"%ld :", (unsigned long) size);
+	    R_OutputStackTrace(R_MemReportingOutfile);
+	}
+    }
+    return;
+}
+
+static void R_ReportNewPage(void)
+{
+    int newline = 0;
+
+    if (R_IsMemReporting) {
+	fprintf(R_MemReportingOutfile, "new page:");
+	R_OutputStackTrace(R_MemReportingOutfile);
+    }
+    return;
+}
+
+
+static void R_EndMemReporting()
+{
+    if(R_MemReportingOutfile != NULL) {
+	/* does not fclose always flush? */
+	fflush(R_MemReportingOutfile);
+	fclose(R_MemReportingOutfile);
+	R_MemReportingOutfile=NULL;
+    }
+    R_IsMemReporting = 0;
+    return;
+}
+
+static void R_InitMemReporting(char *filename, int append,
+			       R_size_t threshold)
+{
+    if(R_MemReportingOutfile != NULL) R_EndMemReporting();
+    R_MemReportingOutfile = fopen(filename, append ? "a" : "w");
+    if (R_MemReportingOutfile == NULL)
+	error(_("Rprofmem: cannot open output file '%s'"), filename);
+    R_MemReportingThreshold = threshold;
+    R_IsMemReporting = 1;
+    return;
+}
+
+SEXP attribute_hidden do_Rprofmem(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    char *filename;
+    R_size_t threshold;
+    int append_mode;
+
+    checkArity(op, args);
+    if (!isString(CAR(args)) || (LENGTH(CAR(args))) != 1)
+	errorcall(call, _("invalid '%s' argument"), "filename");
+    append_mode = asLogical(CADR(args));
+    filename = R_ExpandFileName(CHAR(STRING_ELT(CAR(args), 0)));
+    threshold = REAL(CADDR(args))[0];
+    if (strlen(filename))
+	R_InitMemReporting(filename, append_mode, threshold);
+    else
+	R_EndMemReporting();
+    return R_NilValue;
+}
+
+#endif /* R_MEMORY_PROFILING */
