@@ -171,35 +171,41 @@ void printVector(SEXP x, int indx, int quote)
 /* print R vector x[];	if(indx) print indices; if(quote) quote strings */
     int n;
 
-    if ((n = LENGTH(x)) != 0)
+    if ((n = LENGTH(x)) != 0) {
+	int n_pr = (n <= R_print.max +1) ? n : R_print.max;
+	/* '...max +1'  ==> will omit at least 2 ==> plural in msg below */
 	switch (TYPEOF(x)) {
 	case LGLSXP:
-	    printLogicalVector(LOGICAL(x), n, indx);
+	    printLogicalVector(LOGICAL(x), n_pr, indx);
 	    break;
 	case INTSXP:
-	    printIntegerVector(INTEGER(x), n, indx);
+	    printIntegerVector(INTEGER(x), n_pr, indx);
 	    break;
 	case REALSXP:
-	    printRealVector(REAL(x), n, indx);
+	    printRealVector(REAL(x), n_pr, indx);
 	    break;
 	case STRSXP:
 	    if (quote)
-		printStringVector(STRING_PTR(x), n, '"', indx);
+		printStringVector(STRING_PTR(x), n_pr, '"', indx);
 	    else
-		printStringVector(STRING_PTR(x), n, 0, indx);
+		printStringVector(STRING_PTR(x), n_pr, 0, indx);
 	    break;
 	case CPLXSXP:
-	    printComplexVector(COMPLEX(x), n, indx);
+	    printComplexVector(COMPLEX(x), n_pr, indx);
 	    break;
 	case RAWSXP:
-	    printRawVector(RAW(x), n, indx);
+	    printRawVector(RAW(x), n_pr, indx);
 	    break;
 	}
+	if(n_pr < n)
+		Rprintf(" [ reached getOption(\"max.print\") -- omitted %d entries ]]\n",
+			n - n_pr);
+    }
     else
 #define PRINT_V_0						\
 	switch (TYPEOF(x)) {					\
 	case LGLSXP:	Rprintf("logical(0)\n");	break;	\
-	case INTSXP:						\
+	case INTSXP:	Rprintf("integer(0)\n");	break;	\
 	case REALSXP:	Rprintf("numeric(0)\n");	break;	\
 	case CPLXSXP:	Rprintf("complex(0)\n");	break;	\
 	case STRSXP:	Rprintf("character(0)\n");	break;	\
@@ -297,7 +303,7 @@ static void printNamedStringVector(SEXP * x, int n, int quote, SEXP * names)
 			   R_print.gap, ""))
 
 static void printNamedRawVector(Rbyte * x, int n, SEXP * names)
-    PRINT_N_VECTOR(formatRaw(x, n, &w), 
+    PRINT_N_VECTOR(formatRaw(x, n, &w),
                    Rprintf("%s%*s", EncodeRaw(x[k]), R_print.gap,""))
 
 void printNamedVector(SEXP x, SEXP names, int quote, char *title)
@@ -307,28 +313,35 @@ void printNamedVector(SEXP x, SEXP names, int quote, char *title)
     if (title != NULL)
 	 Rprintf("%s\n", title);
 
-    if ((n = LENGTH(x)) != 0)
+    if ((n = LENGTH(x)) != 0) {
+	int n_pr = (n <= R_print.max +1) ? n : R_print.max;
+	/* '...max +1'  ==> will omit at least 2 ==> plural in msg below */
 	switch (TYPEOF(x)) {
 	case LGLSXP:
-	    printNamedLogicalVector(LOGICAL(x), n, STRING_PTR(names));
+	    printNamedLogicalVector(LOGICAL(x), n_pr, STRING_PTR(names));
 	    break;
 	case INTSXP:
-	    printNamedIntegerVector(INTEGER(x), n, STRING_PTR(names));
+	    printNamedIntegerVector(INTEGER(x), n_pr, STRING_PTR(names));
 	    break;
 	case REALSXP:
-	    printNamedRealVector(REAL(x), n, STRING_PTR(names));
+	    printNamedRealVector(REAL(x), n_pr, STRING_PTR(names));
 	    break;
 	case CPLXSXP:
-	    printNamedComplexVector(COMPLEX(x), n, STRING_PTR(names));
+	    printNamedComplexVector(COMPLEX(x), n_pr, STRING_PTR(names));
 	    break;
 	case STRSXP:
 	    if(quote) quote = '"';
-	    printNamedStringVector(STRING_PTR(x), n, quote, STRING_PTR(names));
+	    printNamedStringVector(STRING_PTR(x), n_pr, quote, STRING_PTR(names));
 	    break;
 	case RAWSXP:
-	    printNamedRawVector(RAW(x), n, STRING_PTR(names));
+	    printNamedRawVector(RAW(x), n_pr, STRING_PTR(names));
 	    break;
 	}
+	if(n_pr < n)
+		Rprintf(" [ reached getOption(\"max.print\") -- omitted %d entries ]]\n",
+			n - n_pr);
+
+    }
     else {
 	Rprintf("named ");
 	PRINT_V_0;
