@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2005   The R Development Core Team.
+ *  Copyright (C) 1998-2006   The R Development Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
  */
 
 /* <UTF8>
@@ -39,12 +39,12 @@
 
 #ifdef SUPPORT_MBCS
 #include <wchar.h> /* for btowc */
+#include <R_ext/rlocale.h> /* for btowc */
 #endif
 
 /* The size of vector initially allocated by scan */
 #define SCAN_BLOCKSIZE		1000
 /* The size of the console buffer */
-#define CONSOLE_BUFFER_SIZE	1024
 #define CONSOLE_PROMPT_SIZE	256
 
 #define NO_COMCHAR 100000 /* won't occur even in Unicode */
@@ -54,7 +54,7 @@
 #define MAX_STRINGS	10000
 
 
-static unsigned char  ConsoleBuf[CONSOLE_BUFFER_SIZE];
+static unsigned char  ConsoleBuf[CONSOLE_BUFFER_SIZE+1];
 static unsigned char *ConsoleBufp;
 static int  ConsoleBufCnt;
 static char  ConsolePrompt[CONSOLE_PROMPT_SIZE];
@@ -148,12 +148,12 @@ static SEXP insertString(char *str, HashData *d)
 static int ConsoleGetchar()
 {
     if (--ConsoleBufCnt < 0) {
+	ConsoleBuf[CONSOLE_BUFFER_SIZE] = '\0';
 	if (R_ReadConsole(ConsolePrompt, ConsoleBuf,
 			  CONSOLE_BUFFER_SIZE, 0) == 0) {
 	    R_ClearerrConsole();
 	    return R_EOF;
 	}
-	R_ParseCnt++;
 	ConsoleBufp = ConsoleBuf;
 	ConsoleBufCnt = strlen((char *)ConsoleBuf);
 	ConsoleBufCnt--;
@@ -381,6 +381,14 @@ static int scanchar(Rboolean inQuote, LocalData *d)
     return next;
 }
 
+/* utility to close connections after interrupts */
+static void scan_cleanup(void *data)
+{
+    LocalData *ld = data;
+    if(!ld->ttyflag && !ld->wasopen) ld->con->close(ld->con);
+    if (ld->quotesave) free(ld->quotesave);
+}
+
 #include "RBufferUtils.h"
 
 /*XX  Can we pass this routine an R_StringBuffer? appears so.
@@ -549,8 +557,6 @@ static R_INLINE void expected(char *what, char *got, LocalData *d)
 	while ((c = scanchar(FALSE, d)) != R_EOF && c != '\n')
 	    ;
     }
-    else
-	if(!d->wasopen) d->con->close(d->con);
     error(_("scan() expected '%s', got '%s'"), what, got);
 }
 
@@ -646,6 +652,7 @@ static SEXP scanVector(SEXPTYPE type, int maxitems, int maxlines,
     }
 
     for (;;) {
+	if(n % 10000 == 9999) R_CheckUserInterrupt();
 	if (bch == R_EOF) {
 	    if (d->ttyflag) R_ClearerrConsole();
 	    break;
@@ -745,7 +752,6 @@ static SEXP scanFrame(SEXP what, int maxitems, int maxlines, int flush,
 
     nc = length(what);
     if (!nc) {
-	    if (!d->ttyflag & !d->wasopen) d->con->close(d->con);
 	    error(_("empty 'what' specified"));
     }
 
@@ -759,7 +765,6 @@ static SEXP scanFrame(SEXP what, int maxitems, int maxlines, int flush,
 	w = VECTOR_ELT(what, i);
 	if (!isNull(w)) {
 	    if (!isVector(w)) {
-		if (!d->ttyflag & !d->wasopen) d->con->close(d->con);
 		error(_("invalid 'what' specified"));
 	    }
 	    if(TYPEOF(w) == STRSXP) nstring++;
@@ -786,6 +791,7 @@ static SEXP scanFrame(SEXP what, int maxitems, int maxlines, int flush,
     strip = asLogical(stripwhite);
 
     for (;;) {
+	if(linesread % 1000 == 999) R_CheckUserInterrupt();
 
 	if (bch == R_EOF) {
 	    if (d->ttyflag) R_ClearerrConsole();
@@ -904,11 +910,12 @@ static SEXP scanFrame(SEXP what, int maxitems, int maxlines, int flush,
     return ans;
 }
 
-SEXP do_scan(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_scan(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, file, sep, what, stripwhite, dec, quotes, comstr;
     int i, c, nlines, nmax, nskip, flush, fill, blskip, multiline, escapes;
     char *p, *vmax;
+    RCNTXT cntxt;
     LocalData data = {NULL, 0, 0, 0, NULL, NULL, NO_COMCHAR, 0, 0, FALSE,
 		      FALSE, 0};
     data.NAstrings = R_NilValue;
@@ -1020,6 +1027,13 @@ SEXP do_scan(SEXP call, SEXP op, SEXP args, SEXP rho)
     ans = R_NilValue;		/* -Wall */
     data.save = 0;
 
+    /* set up a context which will close the connection if there is 
+       an error or user interrupt */
+    begincontext(&cntxt, CTXT_CCODE, call, R_BaseEnv, R_BaseEnv,
+		 R_NilValue, R_NilValue);
+    cntxt.cend = &scan_cleanup;
+    cntxt.cenddata = &data;
+
     switch (TYPEOF(what)) {
     case LGLSXP:
     case INTSXP:
@@ -1036,10 +1050,10 @@ SEXP do_scan(SEXP call, SEXP op, SEXP args, SEXP rho)
 			blskip, multiline, &data);
 	break;
     default:
-	if (!data.ttyflag && !data.wasopen)
-	    data.con->close(data.con);
 	errorcall(call, _("invalid 'what' specified"));
     }
+    endcontext(&cntxt);
+
     /* we might have a character that was unscanchar-ed.
        So pushback if possible */
     if (data.save && !data.ttyflag && data.wasopen) {
@@ -1054,7 +1068,7 @@ SEXP do_scan(SEXP call, SEXP op, SEXP args, SEXP rho)
     return ans;
 }
 
-SEXP do_countfields(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_countfields(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, file, sep,  bns, quotes, comstr;
     int nfields, nskip, i, c, inquote, quote = 0;
@@ -1234,7 +1248,7 @@ SEXP do_countfields(SEXP call, SEXP op, SEXP args, SEXP rho)
    or a factor if as.is == FALSE. */
 
 
-SEXP do_typecvt(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden do_typecvt(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP cvec, a, dup, levs, dims, names, dec;
     SEXP rval = R_NilValue; /* -Wall */
@@ -1424,7 +1438,7 @@ SEXP do_typecvt(SEXP call, SEXP op, SEXP args, SEXP env)
     return rval;
 }
 
-SEXP do_readln(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_readln(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     int c;
     char buffer[MAXELTSIZE], *bufp = buffer;
@@ -1464,7 +1478,7 @@ SEXP do_readln(SEXP call, SEXP op, SEXP args, SEXP rho)
 }
 
 /* Works with digits, but OK in UTF-8 */
-SEXP do_menu(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_menu(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     int c, j;
     double first;
@@ -1510,7 +1524,7 @@ SEXP do_menu(SEXP call, SEXP op, SEXP args, SEXP rho)
 /* simplified version of readLines, with skip of blank lines and
    comment-only lines */
 #define BUF_SIZE 1000
-SEXP do_readtablehead(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_readtablehead(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP file, comstr, ans = R_NilValue, ans2, quotes, sep;
     int nlines, i, c, quote=0, nread, nbuf, buf_size = BUF_SIZE, blskip;
@@ -1668,15 +1682,6 @@ no_more_lines:
    quote is a numeric vector
  */
 
-static void writecon(Rconnection con, char *format, ...)
-{
-    va_list(ap);
-    va_start(ap, format);
-    /* Parentheses added for FC4 with gcc4 and -D_FORTIFY_SOURCE=2 */
-    (con->vfprintf)(con, format, ap);
-    va_end(ap);
-}
-
 static Rboolean isna(SEXP x, int indx)
 {
     Rcomplex rc;
@@ -1742,16 +1747,33 @@ static char *EncodeElement2(SEXP x, int indx, Rboolean quote,
     return EncodeElement(x, indx, quote ? '"' : 0, cdec);
 }
 
+typedef struct wt_info {
+    Rboolean wasopen;
+    Rconnection con;
+    R_StringBuffer *buf;
+    int savedigits;
+} wt_info;
 
-SEXP do_writetable(SEXP call, SEXP op, SEXP args, SEXP rho)
+/* utility to cleanup e.g. after interrpts */
+static void wt_cleanup(void *data)
+{
+    wt_info *ld = data;
+    if(!ld->wasopen) ld->con->close(ld->con);
+    R_FreeStringBuffer(ld->buf);    
+    R_print.digits = ld->savedigits;
+}
+
+SEXP attribute_hidden do_writetable(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP x, sep, rnames, eol, na, dec, quote, xj;
-    int nr, nc, i, j, qmethod, savedigits;
+    int nr, nc, i, j, qmethod;
     Rboolean wasopen, quote_rn = FALSE, *quote_col;
     Rconnection con;
-    char *csep, *ceol, *cna, cdec, *tmp;
+    char *csep, *ceol, *cna, cdec, *tmp=NULL /* -Wall */;
     SEXP *levels;
     R_StringBuffer strBuf = {NULL, 0, MAXELTSIZE};
+    wt_info wi;
+    RCNTXT cntxt;
 
     checkArity(op, args);
 
@@ -1802,7 +1824,14 @@ SEXP do_writetable(SEXP call, SEXP op, SEXP args, SEXP rho)
     }
     R_AllocStringBuffer(0, &strBuf);
     PrintDefaults(R_NilValue);
-    savedigits = R_print.digits; R_print.digits = DBL_DIG;/* MAX precision */
+    wi.savedigits = R_print.digits; R_print.digits = DBL_DIG;/* MAX precision */
+    wi.con = con;
+    wi.wasopen = wasopen;
+    wi.buf = &strBuf;
+    begincontext(&cntxt, CTXT_CCODE, call, R_BaseEnv, R_BaseEnv,
+		 R_NilValue, R_NilValue);
+    cntxt.cend = &wt_cleanup;
+    cntxt.cenddata = &wi;
 
     if(isVectorList(x)) { /* A data frame */
 
@@ -1818,28 +1847,37 @@ SEXP do_writetable(SEXP call, SEXP op, SEXP args, SEXP rho)
 	}
 
 	for(i = 0; i < nr; i++) {
+	    if(i % 1000 == 999) R_CheckUserInterrupt();
 	    if(!isNull(rnames))
-		writecon(con, "%s%s",
-			 EncodeElement2(rnames, i, quote_rn, qmethod,
-					&strBuf, cdec), csep);
+		Rconn_printf(con, "%s%s",
+			     EncodeElement2(rnames, i, quote_rn, qmethod,
+					    &strBuf, cdec), csep);
 	    for(j = 0; j < nc; j++) {
 		xj = VECTOR_ELT(x, j);
-		if(j > 0) writecon(con, "%s", csep);
+		if(j > 0) Rconn_printf(con, "%s", csep);
 		if(isna(xj, i)) tmp = cna;
 		else {
 		    if(!isNull(levels[j])) {
-			tmp = EncodeElement2(levels[j], INTEGER(xj)[i] - 1,
-					     quote_col[j], qmethod,
-					     &strBuf, cdec);
+			/* We cannot assume factors have integer levels */
+			if(TYPEOF(xj) == INTSXP)
+			    tmp = EncodeElement2(levels[j], INTEGER(xj)[i] - 1,
+						 quote_col[j], qmethod,
+						 &strBuf, cdec);
+			else if(TYPEOF(xj) == REALSXP)
+			    tmp = EncodeElement2(levels[j], REAL(xj)[i] - 1,
+						 quote_col[j], qmethod,
+						 &strBuf, cdec);
+			else
+			    error("column %s claims to be a factor but does not have numeric codes", j+1);
 		    } else {
 			tmp = EncodeElement2(xj, i, quote_col[j], qmethod,
 					     &strBuf, cdec);
 		    }
 		    /* if(cdec) change_dec(tmp, cdec, TYPEOF(xj)); */
 		}
-		writecon(con, "%s", tmp);
+		Rconn_printf(con, "%s", tmp);
 	    }
-	    writecon(con, "%s", ceol);
+	    Rconn_printf(con, "%s", ceol);
 	}
 
     } else { /* A matrix */
@@ -1851,26 +1889,26 @@ SEXP do_writetable(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    errorcall(call, _("corrupt matrix -- dims not not match length"));
 
 	for(i = 0; i < nr; i++) {
+	    if(i % 1000 == 999) R_CheckUserInterrupt();
 	    if(!isNull(rnames))
-		writecon(con, "%s%s",
-			 EncodeElement2(rnames, i, quote_rn, qmethod,
-					&strBuf, cdec), csep);
+		Rconn_printf(con, "%s%s",
+			     EncodeElement2(rnames, i, quote_rn, qmethod,
+					    &strBuf, cdec), csep);
 	    for(j = 0; j < nc; j++) {
-		if(j > 0) writecon(con, "%s", csep);
+		if(j > 0) Rconn_printf(con, "%s", csep);
 		if(isna(x, i + j*nr)) tmp = cna;
 		else {
 		    tmp = EncodeElement2(x, i + j*nr, quote_col[j], qmethod,
 					&strBuf, cdec);
 		    /* if(cdec) change_dec(tmp, cdec, TYPEOF(x)); */
 		}
-		writecon(con, "%s", tmp);
+		Rconn_printf(con, "%s", tmp);
 	    }
-	    writecon(con, "%s", ceol);
+	    Rconn_printf(con, "%s", ceol);
 	}
 
     }
-    if(!wasopen) con->close(con);
-    R_FreeStringBuffer(&strBuf);
-    R_print.digits = savedigits;
+    endcontext(&cntxt);
+    wt_cleanup(&wi);
     return R_NilValue;
 }
