@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2000, 2001, 2004-5  the R Development Core Team
+ *  Copyright (C) 2000, 2001, 2004-6  the R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,9 +14,11 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
  */
 
+
+/* We need to know the sizes of certain internal structures */
 #define USE_RINTERNALS
 
 #ifdef HAVE_CONFIG_H
@@ -32,7 +34,7 @@
       important.
    2) an object gets charged for all the space allocated on the heap
       and all the nodes specifically due to it, but not for the 
-      space for its name nor for builtins it references
+      space for its name nor for .Internals it references.
 */
 SEXP csduplicated(SEXP x);  /* from unique.c */
 
@@ -41,6 +43,7 @@ static R_size_t objectsize(SEXP s)
     int i;
     R_size_t cnt = 0, vcnt = 0;
     SEXP tmp, dup;
+    Rboolean isVec = FALSE;
     
     switch (TYPEOF(s)) {
     case NILSXP:
@@ -66,16 +69,20 @@ static R_size_t objectsize(SEXP s)
 	break;
     case CHARSXP:
 	vcnt = BYTE2VEC(length(s)+1);
+	isVec = TRUE;
 	break;
     case LGLSXP:
     case INTSXP:
 	vcnt = INT2VEC(length(s));
+	isVec = TRUE;
 	break;
     case REALSXP:
 	vcnt = FLOAT2VEC(length(s));
+	isVec = TRUE;
 	break;
     case CPLXSXP:
 	vcnt = COMPLEX2VEC(length(s));
+	isVec = TRUE;
 	break;
     case STRSXP:
 	vcnt = PTR2VEC(length(s));
@@ -85,6 +92,7 @@ static R_size_t objectsize(SEXP s)
 	    if(tmp != NA_STRING && !LOGICAL(dup)[i]) 
 		cnt += objectsize(tmp);
 	}
+	isVec = TRUE;
 	break;
     case DOTSXP:
     case ANYSXP:
@@ -97,6 +105,7 @@ static R_size_t objectsize(SEXP s)
 	vcnt = PTR2VEC(length(s));
 	for (i = 0; i < length(s); i++)
 	    cnt += objectsize(VECTOR_ELT(s, i));
+	isVec = TRUE;
 	break;
     case BCODESXP:
 	break;
@@ -108,20 +117,33 @@ static R_size_t objectsize(SEXP s)
     /* WEAKREFSXP is internally a vector */
     case RAWSXP:
 	vcnt = BYTE2VEC(length(s));
+	isVec = TRUE;
 	break;
+    case S4SXP:
+        break;
     default:
 	UNIMPLEMENTED_TYPE("object.size", s);
     }
-    cnt += 8 * vcnt;
-    /* add in node space */
-    cnt += sizeof(SEXPREC);
+    /* add in node space:
+       we need to take into account the rounding up that goes on 
+       in the node classes. */
+    if(isVec) {
+	cnt += sizeof(SEXPREC_ALIGN);
+	if (vcnt > 16) cnt += 8*vcnt;
+	else if (vcnt > 8) cnt += 128;
+	else if (vcnt > 6) cnt += 64;
+	else if (vcnt > 4) cnt += 48;
+	else if (vcnt > 2) cnt += 32;
+	else if (vcnt > 1) cnt += 16;
+	else if (vcnt > 0) cnt += 8;
+    } else cnt += sizeof(SEXPREC);
     /* add in attributes */
     cnt += objectsize(ATTRIB(s));
     return(cnt);
 }
 
 
-SEXP do_objectsize(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden do_objectsize(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP ans;
     R_size_t cnt = 0;
